﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;
using System.Runtime.Serialization;
using System.Data.SqlTypes;
using Crossrail.IMS.Common;

namespace Crossrail.IMS.DataGateway
{
    [DataContract]
    public class Innovation
    {
        public Innovation()
        {
            Init();
        }

        /// <summary>
        /// Initialize default value for properties.
        /// </summary>
        private void Init()
        {
            this.DateReceipt = SqlDateTime.MinValue.Value;
            this.SubmittedForCompetitionDate = SqlDateTime.MinValue.Value;
        }

        [System.Runtime.Serialization.OnDeserializing]
        private void OnDeserializing(StreamingContext ctx)
        {
            Init();
        }

        [DataMember]
        public int ID { get; set; }

        [DataMember]
        [AllowSearch]
        public string InnovationRef { get; set; }

        [DataMember]
        [AllowSearch]
        public string InnovationTitle { get; set; }

        [DataMember]
        [AllowSearch]
        public string Description { get; set; }

        [DataMember]
        [AllowSearch]
        public string InnovationContext { get; set; }

        [DataMember]
        public int? HowCanWeHelpID { get; set; }

        [DataMember]
        public int? IndividualOrSharedInnovationID { get; set; }

        [DataMember]
        [AllowSearch]
        public string Who { get; set; }

        [DataMember]
        [AllowSearch]
        public DateTime? DateReceipt { get; set; }

        [DataMember]
        [AllowSearch]
        public string ParentCompany { get; set; }

        [DataMember]
        [AllowSearch]
        public string ContactFirstName { get; set; }

        [DataMember]
        [AllowSearch]
        public string ContactSurName { get; set; }

        [DataMember]
        [AllowSearch]
        public string ContactTelephone { get; set; }

        [DataMember]
        [AllowSearch]
        public string ContactEmail { get; set; }

        [DataMember]
        public int? ProposingSiteProjectID { get; set; }

        [DataMember]
        [AllowSearch]
        public string SponsoringChampion { get; set; }

        [DataMember]
        public int? Area { get; set; }

        [DataMember]
        [AllowSearch]
        public string DetailedDescription { get; set; }

        [DataMember]
        [AllowSearch]
        public string Keywords { get; set; }

        [DataMember]
        public int? ThemeID { get; set; }

        [DataMember]
        public int? SubThemeID { get; set; }

        [DataMember]
        [AllowSearch]
        public string Benefit { get; set; }

        [DataMember]
        public int? BenefitClearlyUnderstoodId { get; set; }

        [DataMember]
        [AllowSearch]
        public string WhoWillBenefit { get; set; }

        [DataMember]
        [AllowSearch]
        public string FunctionalSponsor { get; set; }

        [DataMember]
        public int? SuitablyDevelopedToShareID { get; set; }

        [DataMember]
        public int? CurrentIRLID { get; set; }

        [DataMember]
        public int? TargetIRLID { get; set; }

        [DataMember]
        [AllowSearch]
        public string DesiredOutcome { get; set; }

        [DataMember]
        [AllowSearch]
        public string Actions { get; set; }

        [DataMember]
        [AllowSearch]
        public bool? IsSubmitted { get; set; }

        [DataMember]
        [AllowSearch]
        public DateTime SubmittedForCompetitionDate { get; set; }

        [DataMember]
        [AllowSearch]
        public bool? IsSafety { get; set; }

        [DataMember]
        public bool? IsInspiration { get; set; }

        [DataMember]
        public bool? IsCollaboration { get; set; }

        [DataMember]
        [AllowSearch]
        public bool? IsIntegrity { get; set; }

        [DataMember]
        [AllowSearch]
        public bool? IsRespect { get; set; }

        [DataMember]
        [AllowSearch]
        public string Notes { get; set; }

        [DataMember]
        [ForeignKey("ProposingSiteProjectID")]
        [AllowSearch]
        public InnovationData ProposingItem { get; set; }

        [DataMember]
        [ForeignKey("ThemeID")]
        [AllowSearch]
        public InnovationData Theme { get; set; }

        [DataMember]
        [ForeignKey("SubThemeID")]
        [AllowSearch]
        public SubTheme SubTheme { get; set; }

        [DataMember]
        [ForeignKey("CurrentIRLID")]
        [AllowSearch]
        public InnovationData CurrentIRL { get; set; }

        [DataMember]
        [NotMapped]
        [AllowSearch]
        public InnovationData CurrentStage { get; set; }

        [DataMember]
        public bool IsTemp { get; set; }

        [DataMember]
        [NotMapped]
        public int InnovationProgressId { get; set; }

        [DataMember]
        [NotMapped]
        public int InnovationFinancialId { get; set; }

        [ForeignKey("TargetIRLID")]
        [AllowSearch]
        public InnovationData TargetIRL { get; set; }

        [ForeignKey("HowCanWeHelpID")]
        [AllowSearch]
        public InnovationData HowCanWeHelp { get; set; }

        [ForeignKey("IndividualOrSharedInnovationID")]
        [AllowSearch]
        public InnovationData IndividualOrSharedInnovation { get; set; }

        [ForeignKey("BenefitClearlyUnderstoodId")]
        [AllowSearch]
        public InnovationData BenefitClearlyUnderstood { get; set; }

        [ForeignKey("SuitablyDevelopedToShareID")]
        [AllowSearch]
        public InnovationData SuitablyDevelopedToShare { get; set; }

        [ForeignKey("Area")]
        [AllowSearch]
        public InnovationData AreaData { get; set; }

        [DataMember]
        [ForeignKey("IdeaTypeID")]
        [AllowSearch]
        public InnovationData IdeaType { get; set; }

        [DataMember]
        [ForeignKey("IdeaMaturityID")]
        [AllowSearch]
        public InnovationData IdeaMaturity { get; set; }

        [DataMember]
        [ForeignKey("PrimaryBenefitID")]
        [AllowSearch]
        public InnovationData PrimaryBenefit { get; set; }

        [DataMember]
        [ForeignKey("InnovationSourceID")]
        [AllowSearch]
        public InnovationData InnovationSource { get; set; }

        [DataMember]
        public decimal? HoursDeveloping { get; set; }

        [DataMember]
        public int? IdeaTypeID { get; set; }

        [DataMember]
        public int? IdeaMaturityID { get; set; }

        [DataMember]
        public int? PrimaryBenefitID { get; set; }

        [DataMember]
        public int? InnovationSourceID { get; set; }

        [DataMember]
        [NotMapped]
        public string NextSteps { get; set; }

        [DataMember]
        public string TaskGroupMembers { get; set; }

        [DataMember]
        public int? AssignedInnovationTeamMemberID { get; set; }

        [DataMember]
        public ICollection<InnovationProposingSiteProject1> ProposingItems1 { get; set; }

        [DataMember]
        public ICollection<InnovationProposingSiteProject2> ProposingItems2 { get; set; }

        [DataMember]
        public string IR_IssueOpportunity { get; set; }

        [DataMember]
        public string IR_InnovativeSol { get; set; }

        [DataMember]
        public string IR_PrimaryBenefit { get; set; }

        [DataMember]
        public string IR_DesiredOutcome { get; set; }

        [DataMember]
        public string IR_FundingSought { get; set; }

        [DataMember]
        public string IR_Assumptions { get; set; }

        [DataMember]
        public ICollection<InnovationRemitImp> InnovationRemitImps { get; set; }
        [DataMember]
        public ICollection<InnovationRemitAuthProceed> InnovationRemitAuthProceeds { get; set; }
        [DataMember]
        public ICollection<InnovationRemitResponsible> InnovationRemitResponsibles { get; set; }
        [DataMember]
        public ICollection<InnovationRemitSuccessFailure> InnovationRemitSuccessFailures { get; set; }
        [DataMember]
        public ICollection<InnovationRemitSuppoter> InnovationRemitSuppoters { get; set; }
    }

    [DataContract]
    public class InnovationProposingSiteProject1
    {
        [ForeignKey("InnovationID")]
        public Innovation Innovation { get; set; }

        [ForeignKey("ProposingID")]
        public InnovationData Proposing { get; set; }

        [DataMember]
        public int InnovationID { get; set; }
        [DataMember]
        public int ProposingID { get; set; }
    }

    [DataContract]
    public class InnovationProposingSiteProject2
    {
        [ForeignKey("InnovationID")]
        public Innovation Innovation { get; set; }

        [ForeignKey("ProposingID")]
        public InnovationData Proposing { get; set; }

        [DataMember]
        public int InnovationID { get; set; }
        [DataMember]
        public int ProposingID { get; set; }
    }
}