﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;
using System.Runtime.Serialization;
using System.Data.SqlTypes;

namespace Crossrail.IMS.DataGateway
{
    [DataContract]
    public class InnovationProgress
    {
        public InnovationProgress()
        {
            Init();
        }

        /// <summary>
        /// Initialize default value for properties.
        /// </summary>
        private void Init()
        {
            this.DatefirstInterviewConducted = SqlDateTime.MinValue.Value;
            this.DateOfFirstResponse = SqlDateTime.MinValue.Value;
            this.FollowupDeadline = SqlDateTime.MinValue.Value;
        }

        [System.Runtime.Serialization.OnDeserializing]
        private void OnDeserializing(StreamingContext ctx)
        {
            Init();
        }

        /// <summary>
        /// Gets or sets the ID.
        /// </summary>
        /// <value>
        /// The ID.
        /// </value>
        [DataMember]
        public int ID { get; set; }

        /// <summary>
        /// Gets or sets the innovation ref.
        /// </summary>
        /// <value>
        /// The innovation ref.
        /// </value>
        [DataMember]
        public string InnovationRef { get; set; }

        /// <summary>
        /// Gets or sets the latest update.
        /// </summary>
        /// <value>
        /// The latest update.
        /// </value>
        [DataMember]
        public string LatestUpdate { get; set; }

        /// <summary>
        /// Gets or sets the current stage complete.
        /// </summary>
        /// <value>
        /// The current stage complete.
        /// </value>
        [DataMember]
        public decimal CurrentStageComplete { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is innovator contact details checked.
        /// </summary>
        /// <value>
        /// <c>true</c> if this instance is innovator contact details checked; otherwise, <c>false</c>.
        /// </value>
        [DataMember]
        public bool? IsInnovatorContactDetailsChecked { get; set; }

        /// <summary>
        /// Gets or sets the datefirst interview conducted.
        /// </summary>
        /// <value>
        /// The datefirst interview conducted.
        /// </value>
        [DataMember]
        public DateTime DatefirstInterviewConducted { get; set; }

        /// <summary>
        /// Gets or sets the date of first response.
        /// </summary>
        /// <value>
        /// The date of first response.
        /// </value>
        [DataMember]
        public DateTime DateOfFirstResponse { get; set; }

        /// <summary>
        /// Gets or sets the followup deadline.
        /// </summary>
        /// <value>
        /// The followup deadline.
        /// </value>
        [DataMember]
        public DateTime FollowupDeadline { get; set; }

        /// <summary>
        /// Gets or sets the next steps.
        /// </summary>
        /// <value>
        /// The next steps.
        /// </value>
        [DataMember]
        public string NextSteps { get; set; }

        /// <summary>
        /// Gets or sets the why failed support.
        /// </summary>
        /// <value>
        /// The why failed support.
        /// </value>
        [DataMember]
        public string WhyFailedSupport { get; set; }

        /// <summary>
        /// Gets or sets the how to help.
        /// </summary>
        /// <value>
        /// The how to help.
        /// </value>
        [DataMember]
        public string HowToHelp { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is novel idea.
        /// </summary>
        /// <value>
        /// <c>true</c> if this instance is novel idea; otherwise, <c>false</c>.
        /// </value>
        [DataMember]
        public bool? IsNovelIdea { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is understood.
        /// </summary>
        /// <value>
        /// <c>true</c> if this instance is understood; otherwise, <c>false</c>.
        /// </value>
        [DataMember]
        public bool? IsUnderstood { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is ready for discovery evaluation.
        /// </summary>
        /// <value>
        /// <c>true</c> if this instance is ready for discovery evaluation; otherwise, <c>false</c>.
        /// </value>
        [DataMember]
        public bool? IsReadyForDiscoveryEvaluation { get; set; }

        /// <summary>
        /// Gets or sets the who benefits.
        /// </summary>
        /// <value>
        /// The who benefits.
        /// </value>
        [DataMember]
        public int WhoBenefits { get; set; }

        /// <summary>
        /// Gets or sets the what magnitude required.
        /// </summary>
        /// <value>
        /// The what magnitude required.
        /// </value>
        [DataMember]
        public int WhatMagnitudeRequired { get; set; }

        /// <summary>
        /// Gets or sets the what impact to productivity.
        /// </summary>
        /// <value>
        /// The what impact to productivity.
        /// </value>
        [DataMember]
        public int WhatImpactToProductivity { get; set; }

        /// <summary>
        /// Gets or sets the how idea improve quality.
        /// </summary>
        /// <value>
        /// The how idea improve quality.
        /// </value>
        [DataMember]
        public int HowIdeaImproveQuality { get; set; }

        /// <summary>
        /// Gets or sets the how strong evidence impact.
        /// </summary>
        /// <value>
        /// The how strong evidence impact.
        /// </value>
        [DataMember]
        public int HowStrongEvidenceImpact { get; set; }

        /// <summary>
        /// Gets or sets the what flexibility idea afford.
        /// </summary>
        /// <value>
        /// The what flexibility idea afford.
        /// </value>
        [DataMember]
        public int WhatFlexibilityIdeaAfford { get; set; }

        /// <summary>
        /// Gets or sets the what magnitude idea importsto CRL.
        /// </summary>
        /// <value>
        /// The what magnitude idea importsto CRL.
        /// </value>
        [DataMember]
        public int WhatMagnitudeIdeaImportstoCRL { get; set; }

        /// <summary>
        /// Gets or sets the competitive sensitivity with idea.
        /// </summary>
        /// <value>
        /// The competitive sensitivity with idea.
        /// </value>
        [DataMember]
        public int CompetitiveSensitivityWithIdea { get; set; }

        /// <summary>
        /// Gets or sets the how well developed idea.
        /// </summary>
        /// <value>
        /// The how well developed idea.
        /// </value>
        [DataMember]
        public int HowWellDevelopedIdea { get; set; }

        /// <summary>
        /// Gets or sets the what levelof support.
        /// </summary>
        /// <value>
        /// The what levelof support.
        /// </value>
        [DataMember]
        public int WhatLevelofSupport { get; set; }

        /// <summary>
        /// Gets or sets the what levelof effort required.
        /// </summary>
        /// <value>
        /// The what levelof effort required.
        /// </value>
        [DataMember]
        public int WhatLevelofEffortRequired { get; set; }

        /// <summary>
        /// Gets or sets the innovation.
        /// </summary>
        /// <value>
        /// The innovation.
        /// </value>
        [DataMember]
        public Innovation Innovation { get; set; }

        /// <summary>
        /// Gets or sets the current stage.
        /// </summary>
        /// <value>
        /// The current stage.
        /// </value>
        [DataMember]
        public InnovationData CurrentStage { get; set; }

		/// <summary>
		/// Gets or sets the last edit.
		/// </summary>
		/// <value>
		/// The last edit.
		/// </value>
		[DataMember]
		public DateTime? LastEdit { get; set; }

        [ForeignKey("WhoBenefits")]
        public AttractivenessComplexity WhoBenefitsData { get; set; }

        [ForeignKey("WhatMagnitudeRequired")]
        public AttractivenessComplexity WhatMagnitudeRequiredData { get; set; }

        [ForeignKey("WhatImpactToProductivity")]
        public AttractivenessComplexity WhatImpactToProductivityData { get; set; }

        [ForeignKey("HowIdeaImproveQuality")]
        public AttractivenessComplexity HowIdeaImproveQualityData { get; set; }

        [ForeignKey("HowStrongEvidenceImpact")]
        public AttractivenessComplexity HowStrongEvidenceImpactData { get; set; }

        [ForeignKey("WhatFlexibilityIdeaAfford")]
        public AttractivenessComplexity WhatFlexibilityIdeaAffordData { get; set; }

        [ForeignKey("WhatMagnitudeIdeaImportstoCRL")]
        public AttractivenessComplexity WhatMagnitudeIdeaImportstoCRLData { get; set; }

        [ForeignKey("CompetitiveSensitivityWithIdea")]
        public AttractivenessComplexity CompetitiveSensitivityWithIdeaData { get; set; }

        [ForeignKey("HowWellDevelopedIdea")]
        public AttractivenessComplexity HowWellDevelopedIdeaData { get; set; }

        [ForeignKey("WhatLevelofSupport")]
        public AttractivenessComplexity WhatLevelofSupportData { get; set; }

        [ForeignKey("WhatLevelofEffortRequired")]
        public AttractivenessComplexity WhatLevelofEffortRequiredData { get; set; }

        [DataMember]
        public string ProgrammeManagersRecommendation { get; set; }

        [DataMember]
        public string WorkingGroupRecommendations { get; set; }

        [DataMember]
        public string ForumDecision { get; set; }
    }
}