﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using System.ServiceModel;
using Crossrail.IMS.Common;
using Crossrail.IMS.DataGateway;
using Crossrail.IMS.DataGateway.Dtos;
using Crossrail.IMS.DataGateway.Repositories;
using Crossrail.IMS.DataGateway.Entities;

namespace Crossrail.IMS.DataGateway
{

    /// <summary>
    /// Data Gateway Service
    /// </summary>
    [ServiceContract]
    public interface IDataGatewayService
    {
        [OperationContract]
        void AuditLoginUser(string userName);

        #region Innovation Data
        [OperationContract]
        InnovationData GetInnovationDataById(int id);

        /// <summary>
        /// Adds the innovation data.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        int AddInnovationData(InnovationData innovationData, string userName);

        /// <summary>
        /// Updates the innovation data.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        int UpdateInnovationData(InnovationData innovationData, string userName);

        /// <summary>
        /// Deletes the innovation data.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        int DeleteInnovationData(int innovationDataId, string userName);

        /// <summary>
        /// Gets the innovation data by category.
        /// </summary>
        /// <param name="category">The category.</param>
        /// <returns></returns>
        [OperationContract]
        List<InnovationData> GetInnovationDataByCategory(InnovationDataCategory category);

        /// <summary>
        /// Finds the specified id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns></returns>
        [OperationContract]
        InnovationData FindInnovationData(int id);

        /// <summary>
        /// Determines whether [is innovation data value exist] [the specified id].
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="dataValue">The data value.</param>
        /// <returns>
        ///   <c>true</c> if [is innovation data value exist] [the specified id]; otherwise, <c>false</c>.
        /// </returns>
        [OperationContract]
        bool IsInnovationDataValueExist(int id, string dataValue, InnovationDataCategory category);
        #endregion

        #region Sub Theme
        /// <summary>
        /// Adds the sub theme.
        /// </summary>
        /// <param name="subTheme">The sub theme.</param>
        /// <returns></returns>
        [OperationContract]
        int AddSubTheme(SubTheme subTheme, string userName);

        /// <summary>
        /// Updates the innovation data.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        int UpdateSubTheme(SubTheme subTheme, string userName);

        /// <summary>
        /// Deletes the innovation data.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        int DeleteSubTheme(int subThemeId, string userName);

        /// <summary>
        /// Gets the innovation data by category.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<SubTheme> GetSubThemes();

        /// <summary>
        /// Determines whether [is sub theme name exist] [the specified id].
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="dataValue">The data value.</param>
        /// <returns>
        ///   <c>true</c> if [is sub theme name exist] [the specified id]; otherwise, <c>false</c>.
        /// </returns>
        [OperationContract]
        bool IsSubThemeNameExist(int id, string dataValue);

        /// <summary>
        /// Finds the sub theme.
        /// </summary>
        /// <param name="subThemeId">The sub theme id.</param>
        /// <returns></returns>
        [OperationContract]
        SubTheme FindSubTheme(int subThemeId);

        /// <summary>
        /// Get sub themes by themeId
        /// </summary>
        /// <param name="themeID"></param>
        /// <returns></returns>
        [OperationContract]
        List<SubTheme> GetSubThemeByThemeID(int themeID);

        #endregion

        #region User
        /// <summary>
        /// Get all roles
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<Role> GetAllRoles();

        /// <summary>
        /// Add user to the system
        /// </summary>
        /// <param name="user"></param>
        /// <returns></returns>
        [OperationContract]
        int AddUser(User user, string userName);

        /// <summary>
        /// Get all users of the system
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<User> GetAllSystemUsers();

        /// <summary>
        /// Get user by userID
        /// </summary>
        /// <param name="userID"></param>
        /// <returns></returns>
        [OperationContract]
        User GetUser(int userID);

        /// <summary>
        /// Update user
        /// </summary>
        /// <param name="user"></param>
        /// <returns></returns>
        [OperationContract]
        int UpdateUser(User user, string userName);

        /// <summary>
        /// Check whether a user is valid or not
        /// </summary>
        /// <param name="userName"></param>
        /// <returns></returns>
        [OperationContract]
        bool IsValidUser(string userName);

        /// <summary>
        /// Get user by username
        /// </summary>
        /// <param name="userName"></param>
        /// <returns></returns>
        [OperationContract]
        User GetUserByUserName(string userName);
        #endregion

        #region AttractivenessComplexity

        /// <summary>
        /// Gets the attractiveness complexity by question id.
        /// </summary>
        /// <param name="questionId">The question id.</param>
        /// <returns></returns>
        [OperationContract]
        List<AttractivenessComplexity> GetAttractivenessComplexityByQuestionId(AttractivenessComplexityQuestionId questionId);

        #endregion

        #region Innovation Progress

        /// <summary>
        /// Creates the specified innovation progress.
        /// </summary>
        /// <param name="innovationProgress">The innovation progress.</param>
        /// <returns></returns>
        [OperationContract]
        int CreateInnovationProgress(InnovationProgress innovationProgress, string userName);

        /// <summary>
        /// Updates the innovation progress.
        /// </summary>
        /// <param name="innovationProgress">The innovation progress.</param>
        /// <returns></returns>
        [OperationContract]
        int UpdateInnovationProgress(InnovationProgress innovationProgress, string userName);

        [OperationContract]
        AttractivenessComplexity GetAnswersById(int id);

        #endregion

        #region People Support        
        [OperationContract]
        List<PeopleSupport> GetAllPeopleSupportByInnovationId(int innovationId);
        #endregion

        #region Actions Required
        [OperationContract]
        List<ActionsRequired> GetAllActionsRequiredByInnovationId(int innovationId);
        #endregion

        #region Resources Required
        [OperationContract]
        List<ResourcesRequired> GetAllResourcesRequiredByInnovationId(int innovationId);
        #endregion

		#region People Support Temporary
		[OperationContract]
        List<PeopleSupportTemporary> GetAllPeopleSupportTemporaryByInnovationId(int innovationId, Guid temptId);

		[OperationContract]
		int AddPeopleSupportTemporary(PeopleSupportTemporary peopleSupport, string userName);

		[OperationContract]
		int UpdatePeopleSupportTemporary(PeopleSupportTemporary peopleSupport, string userName);

		[OperationContract]
		int DeletePeopleSupportTemporary(int id, string userName);
		
		[OperationContract]
		void FillPeopleSupportTemporaryByInnovationId(int innovationId);

		[OperationContract]
		void CleanupPeopleSupportTemporaryByInnovationId(int innovationId);

        [OperationContract]
        int SavePeopleSupport(int innovationId, string userName, List<PeopleSupportTemporary> peopleSupportTemporaryList);
		#endregion

        #region Actions Required Temporary
    
        [OperationContract]
        int AddActionsRequiredTemporary(ActionsRequiredTemporary actionsRequired, string userName);
        
        [OperationContract]
        int UpdateActionsRequiredTemporary(ActionsRequiredTemporary actionsRequired, string userName);
      
        [OperationContract]
        int DeleteActionsRequiredTemporary(int id, string userName);
       
        [OperationContract]
        List<ActionsRequiredTemporary> GetAllActionsRequiredTemporaryByInnovationId(int innovationId, Guid temptId);

		[OperationContract]
		void FillActionsRequiredTemporaryByInnovationId(int innovationId);

		[OperationContract]
		void CleanupActionsRequiredTemporaryByInnovationId(int innovationId);

	    [OperationContract]
	    int SaveActionsRequired(int innovationId, string userName,
	                            List<ActionsRequiredTemporary> actionsRequiredTemporaryList);
        #endregion

        #region Resources Required Temporary
        [OperationContract]
        int AddResourcesRequiredTemporary(ResourcesRequiredTemporary actionsRequired, string userName);
     
        [OperationContract]
        int UpdateResourcesRequiredTemporary(ResourcesRequiredTemporary actionsRequired, string userName);
      
        [OperationContract]
        int DeleteResourcesRequiredTemporary(int id, string userName);

        [OperationContract]
        List<ResourcesRequiredTemporary> GetAllResourcesRequiredTemporaryByInnovationId(int innovationId, Guid temptId);

		[OperationContract]
		void FillResourceRequiredTemporaryByInnovationId(int innovationId);

		[OperationContract]
		void CleanupResourceRequiredTemporaryByInnovationId(int innovationId);

		[OperationContract]
		int SaveResourcesRequired(int innovationId, string userName,
									   List<ResourcesRequiredTemporary> resourcesRequiredTemporaryList);
        #endregion

        #region Link Document Temporary
        [OperationContract]
        int AddLinkedDocumentsTemporary(LinkedDocumentsTemporary actionsRequired, string userName);

        [OperationContract]
        int UpdateLinkedDocumentsTemporary(LinkedDocumentsTemporary actionsRequired, string userName);

        [OperationContract]
        int DeleteLinkedDocumentsTemporary(int id, string userName);

        [OperationContract]
        List<LinkedDocumentsTemporary> GetAllLinkedDocumentsTemporaryByInnovationId(int innovationId, Guid temptId);

        [OperationContract]
        void FillLinkedDocumentsTemporaryByInnovationId(int innovationId);

        [OperationContract]
        void CleanupLinkedDocumentsTemporaryByInnovationId(int innovationId);

        [OperationContract]
        int SaveLinkedDocuments(int innovationId, string userName,
                                       List<LinkedDocumentsTemporary> resourcesRequiredTemporaryList);
        #endregion

        #region Report
        /// Gets the innovation period name
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<InnovationPeriod> GetInnovationPeriods(string filePath);

        /// <summary>
        /// Gets the innovation period name
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<DataNewSubmissionReport> exportNewSubmissionReport(IList<InnovationPeriod> periods);

        /// <summary>
        /// Gets the innovation period name
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<DataStageInnovationReport> exportStageSubmissionReport(IList<InnovationPeriod> periods);

        /// <summary>
        /// Gets the innovation period name
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<DataNewSubmissionReport> exportExtractSubmissionReport(IList<InnovationPeriod> periods);
        #endregion

        #region Innovation Details

        [OperationContract]
        Innovation GetInnovation(int innovationId);

        [OperationContract]
        InnovationProgress GetInnovationProgress(int innovationId);

        #endregion

        #region Online Innovation

        /// <summary>
        /// Creates the specified innovation .
        /// </summary>
        [OperationContract]
        int AddOnlineInnovation(Innovation innovation, string userName);

        /// <summary>
        /// Get innovation by innovationID
        /// </summary>
        [OperationContract]
        Innovation GetOnlineInnovationById(int innovationId);


        /// <summary>
        /// Update innovation
        /// </summary>
        [OperationContract]
        int UpdateOnlineInnovation(Innovation innovation, string userName);

        /// <summary>
        /// Delete innovation
        /// </summary>
        /// <param name="userName"></param>
        /// <returns></returns>
        [OperationContract]
        bool DeleteOnlineInnovation(int innovationId, string userName);

        /// <summary>
        /// Get all innovations
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<Innovation> GetAllInnovations(string userName, int pageNumber, int pageSize, string sortColumn, string sortDir, out int total);

        /// <summary>
        /// Cancel the Innovation
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        int CancelinnovationDetail(int innovationId, string innovationRef, string userName);

        #endregion

        #region Import Innovation
        /// <summary>
        /// Adds the innovation.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        string AddInnovation(Innovation innovation, string userName);

        /// <summary>
        /// Updates the innovation.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        int UpdateInnovation(Innovation innovation, string userName); 
        #endregion

        #region Innovation Financial

        /// <summary>
        /// Creates the specified innovation .
        /// </summary>
        [OperationContract]
        int AddInnovationFinancial(InnovationFinancial innovationFinancial, string userName);


        /// <summary>
        /// Get innovation by innovationFinancialId
        /// </summary>
        [OperationContract]
        InnovationFinancial GetInnovationFinancialById(int innovationFinancialId);

        /// <summary>
        /// Get innovation financial by innovationID
        /// </summary>
        [OperationContract]
        InnovationFinancial GetInnovationFinancialByInnovationId(int innovationId);

        /// <summary>
        /// Update innovation
        /// </summary>
        [OperationContract]
        int UpdateInnovationFinancial(InnovationFinancial innovationFinancial, string userName);

        /// <summary>
        /// Delete innovation
        /// </summary>
        /// <param name="userName"></param>
        /// <returns></returns>
        [OperationContract]
        bool DeleteInnovationFinancial(int innovationFinancialId, string userName);

        #endregion

        #region Utilities

        /// <summary>
        /// Gets the next innovation ref.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        string GetNextInnovationRef();

        #endregion

        #region Search Innovations

        /// <summary>
        /// Search innovations by criteria. Result is filtered by user's role
        /// </summary>
        /// <param name="criteria"></param>
        /// <param name="userName"></param>
        /// <returns></returns>
        [OperationContract]
        List<Innovation> SearchInnovation(SearchInnovationCriteria criteria, string userName, int pageNumber, int pageSize, string sortColumn, string sortDir, out int total);

        #endregion

        /// <summary>
        /// Gets the innocation progress.
        /// </summary>
        /// <param name="innovationId">The innovation id.</param>
        /// <returns></returns>
        [OperationContract]
        InnovationProgress GetInnovationProgressByInRef(string inRef);

        /// <summary>
        /// Gets the innocation progress.
        /// </summary>
        /// <param name="innovationId">The innovation id.</param>
        /// <returns></returns>
        [OperationContract]
        InnovationFinancial GetInnovationFinancialByInRef(string inRef);

        [OperationContract]
        string GetContactEmailByRoleId(int roleId, int psdId);

        [OperationContract]
        string GetContactEmailByPSDId(int psdId);

		#region LinkedDocuments

		[OperationContract]
		int AddLinkedDocument(LinkedDocument linkedDocument, string userName);

		[OperationContract]
		int UpdateLinkedDocument(LinkedDocument linkedDocument, string userName);

		[OperationContract]
		int DeleteLinkedDocument(int id, string userName);

		[OperationContract]
		List<LinkedDocument> GetLinkedDocumentsByInnovationId(int innovationId);

		[OperationContract]
		LinkedDocument GetLinkedDocumentById(int id);

		[OperationContract]
		void ResetDocumentStatus(int innovationId, string userName);

		#endregion LinkedDocuments

        #region Remit

        [OperationContract]
        List<InnovationRemitImp> GetInnovationRemitImps(int innovationId);

        [OperationContract]
        List<InnovationRemitResponsible> GetInnovationRemitResponsibles(int innovationId);

        [OperationContract]
        List<InnovationRemitSuccessFailure> GetInnovationRemitSuccessFailures(int innovationId);

        [OperationContract]
        List<InnovationRemitAuthProceed> GetInnovationRemitAuthProceeds(int innovationId);

        [OperationContract]
        List<InnovationRemitSuppoter> GetInnovationRemitSuppoters(int innovationId);

        #endregion
    }
}