﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Crossrail.IMS.Common;

namespace Crossrail.IMS.DataGateway.Repositories
{
    public interface IUserRepository
    {
        /// <summary>
        /// Add a user to the system
        /// </summary>
        /// <param name="user"></param>
        int AddUser(User user, string userName);

        /// <summary>
        /// Get all roles from the system
        /// </summary>
        /// <returns></returns>
        List<Role> GetAllRoles();

        /// <summary>
        /// Get all domain accounts of the current logged in user
        /// </summary>
        /// <returns></returns>
        List<ActiveDirectoryUser> GetAllADUsers();

        /// <summary>
        /// Get all user of the system
        /// </summary>
        /// <returns></returns>
        List<User> GetAllSystemUsers();

        /// <summary>
        /// Get user by userID
        /// </summary>
        /// <param name="userID"></param>
        /// <returns></returns>
        User GetUser(int userID);

        /// <summary>
        /// Get user by username
        /// </summary>
        /// <param name="userName"></param>
        /// <returns></returns>
        User GetUser(string userName);

        /// <summary>
        /// Update user
        /// </summary>
        /// <param name="user"></param>
        /// <returns></returns>
        int UpdateUser(User user, string userName);

        /// <summary>
        /// Check whether a user is valid or not
        /// </summary>
        /// <param name="userName"></param>
        /// <returns></returns>
        bool IsValidUser(string userName);
    }
}