﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;
using Crossrail.IMS.Common;
using Crossrail.IMS.DataGateway.Utilities;

namespace Crossrail.IMS.DataGateway.Repositories
{
    /// <summary>
    /// This class is used to import new innovation from XML.
    /// </summary>
    public class ImportInnovationRepository : IImportInnovationRepository
    {
        /// <summary>
        /// Import new innovation to database
        /// </summary>
        /// <param name="innovation">innovation object contains data gotten from xml file</param>
        /// <returns>result after importing. Return error message or Innovation Ref</returns>
        public string AddInnovation(Innovation innovation, string userName)
        {
            try
            {
                using (var context = new MainContext())
                {
                    string strInnovationRef = CommonUtil.GetNextInnovationRef(); //value of innovation Ref will be imported.
                    var _innovationDataOfCurrentStage = context.InnovationDatas
                        .Where(_innovationData => _innovationData.DataValue.ToLower() == "submitted" && _innovationData.Category == (int)InnovationDataCategory.Stage)
                        .First<InnovationData>(); //get innovation data of Stage.
                    InnovationProgress _innovationProgress = new InnovationProgress(); //create innovation progress object to import to DB.
                    _innovationProgress.CurrentStage = _innovationDataOfCurrentStage;
                    innovation.InnovationRef = strInnovationRef;
                    _innovationProgress.InnovationRef = strInnovationRef;
                    _innovationProgress.Innovation = innovation;
                    InnovationFinancial _innovationFinancial = new InnovationFinancial(); //create innovation financial object to import to DB.
                    _innovationFinancial.InnovationRef = strInnovationRef;
                    _innovationFinancial.Innovation = innovation;
                    context.Innovations.Add(innovation);
                    context.InnovationProgresses.Add(_innovationProgress);
                    //  context.InnovationFinancials.Add(_innovationFinancial);
                    int intResult = context.SaveChanges(userName); //result after importing.
                    return "Success: " + strInnovationRef;
                }
            }
            catch (Exception ex)
            {
                return "Error: " + ex.Message;
            }
        }

        /// <summary>
        /// Update Innovation
        /// </summary>
        /// <param name="innovation"></param>
        /// <returns></returns>
        public int UpdateInnovation(Innovation innovation, string userName)
        {
            using (var context = new MainContext())
            {
                context.Entry(innovation).State = EntityState.Modified;
                return context.SaveChanges(userName);
            }
        }

        public string GetContactEmailByRoleId(int roleId, int psdId)
        {
            using (var context = new MainContext())
            {
                var temp = (from d in context.UserProposingSiteProjects
                            where d.InnovationDataID == psdId && d.User.RoleID == roleId
                            select d.User).ToList();
                string retString = string.Empty;
                foreach (var item in temp)
                {
                    retString += item.Email + ";";
                }
                return retString;
            }
        }

        public string GetContactEmailByPSDId(int psdId)
        {
            using (var context = new MainContext())
            {
                var temp = context.Users.Where(c => c.ChampionFors.Any(u => u.ChampionForID == psdId)).Select(u => u.Email).ToArray();
                return string.Join(";", temp);
            }
        }
    }
}