﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using Crossrail.IMS.Common;
using System.Data.Entity;

namespace Crossrail.IMS.DataGateway.Repositories
{
	public class InnovationDataRepository : IInnovationDataRepository
	{
        public InnovationDataRepository()
        {
            Database.SetInitializer<MainContext>(null);
        }

		/// <summary>
		/// Adds the innovation data.
		/// </summary>
		/// <returns></returns>
        public int AddInnovationData(InnovationData innovationData, string userName)
		{
			try
			{
                using (var context = new MainContext())
                {
                    //Create tempt innovation                    
                    context.InnovationDatas.Add(innovationData);
                    context.SaveChanges(userName);
                    return innovationData.ID;
                }
			}
			catch (Exception ex)
			{
				throw ex;
			}
			
		}

        public InnovationData GetInnovationDataById(int id)
        {
            try
            {
                using (var context = new MainContext())
                {
                    return context.InnovationDatas.First(i => i.ID == id);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

		/// <summary>
		/// Updates the innovation data.
		/// </summary>
		/// <returns></returns>
        public int UpdateInnovationData(InnovationData innovationData, string userName)
		{
            try
            {
                using (var context = new MainContext())
                {
                    var ent = context.InnovationDatas.Find(innovationData.ID);
                    if (ent != null)
                    {
                        ent.DataValue = innovationData.DataValue;
                        ent.Category = innovationData.Category;
                        context.Entry(ent).State = EntityState.Modified;
                        return context.SaveChanges(userName);
                    }
                    else
                    {
                        return -1;
                    }
                    
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            
		}

		/// <summary>
		/// Deletes the innovation data.
		/// </summary>
		/// <returns></returns>
        public int DeleteInnovationData(int innovationDataId, string userName)
		{
            try
            {
                using (var context = new MainContext())
                {
                    InnovationData innovationData = context.InnovationDatas.Find(innovationDataId);
                    if (innovationData != null)
                    {
                        context.InnovationDatas.Remove(innovationData);
                        return context.SaveChanges(userName);
                    }

                    return 0;
                }
            }
            catch (System.Data.SqlClient.SqlException slqException)
            {
                throw slqException;
            }
            catch (Exception ex)
            {
                throw ex;
            }
            
		}

		/// <summary>
		/// Gets the innovation data by category.
		/// </summary>
		/// <param name="category">The category.</param>
		/// <returns></returns>
		public List<InnovationData> GetInnovationDataByCategory(InnovationDataCategory category)
		{
            using (var context = new MainContext())
            {
                List<InnovationData> result = context.InnovationDatas.Where(data => data.Category == (int)category).ToList();
                return result;
            }
		}

		/// <summary>
		/// Finds the specified id.
		/// </summary>
		/// <param name="id">The innovation data id.</param>
		/// <returns></returns>
		public InnovationData Find(int id)
		{
            using (var context = new MainContext())
            {
                return context.InnovationDatas.Find(id);
            }
		}

		/// <summary>
		/// Determines whether [is innovation data value exist] [the specified id].
		/// </summary>
		/// <param name="id">The id.</param>
		/// <param name="dataValue">The data value.</param>
		/// <returns>
		///   <c>true</c> if [is innovation data value exist] [the specified id]; otherwise, <c>false</c>.
		/// </returns>
		public bool IsInnovationDataValueExist(int id, string dataValue , InnovationDataCategory category)
		{
            try
            {
                using (var context = new MainContext())
                {
                    if (id == 0)
                    {
                        return (context.InnovationDatas.Any(d => d.DataValue == dataValue && d.Category == (int)category));
                    }
                    else
                    {
                        return (context.InnovationDatas.Any(d => d.DataValue == dataValue && d.ID != id && d.Category == (int)category));
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
		}
	}
}