﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace Crossrail.IMS.DataGateway.Repositories
{
	public class InnovationProgressRepository : IInnovationProgressRepository
	{
		/// <summary>
		/// The _main context
		/// </summary>
		//private MainContext _mainContext = new MainContext();

		/// <summary>
		/// Creates the innocation progress.
		/// </summary>
		/// <param name="innovationProgress">The innovation progress.</param>
		/// <returns></returns>
        public int CreateInnovationProgress(InnovationProgress innovationProgress, string userName)
		{
			try
			{
				using (var _mainContext = new MainContext())
				{
					var innovation = _mainContext.Innovations.Find(innovationProgress.Innovation.ID);
					innovationProgress.Innovation = innovation;

					if (innovationProgress.CurrentStage != null)
					{
						var innovationData = _mainContext.InnovationDatas.Find(innovationProgress.CurrentStage.ID);
						innovationProgress.CurrentStage = innovationData;
					}
					_mainContext.InnovationProgresses.Add(innovationProgress);
                    return _mainContext.SaveChanges(userName);
				}
			}
			catch (Exception ex)
			{
				throw ex;
			}
		}

		/// <summary>
		/// Updates the innovation progress.
		/// </summary>
		/// <param name="innovationProgress">The innovation progress.</param>
		/// <returns></returns>
        public int UpdateInnovationProgress(InnovationProgress innovationProgress, string userName)
		{
			try
			{
				using (var _mainContext = new MainContext())
				{
					var ent = _mainContext.InnovationProgresses.Find(innovationProgress.ID);
					if (ent != null)
					{
						var innovation = _mainContext.Innovations.Find(innovationProgress.Innovation.ID);
						_mainContext.Innovations.Attach(innovation);
						ent.Innovation = innovation;

						if (innovationProgress.CurrentStage != null)
						{
							var innovationData = _mainContext.InnovationDatas.Find(innovationProgress.CurrentStage.ID);
							if (innovationData != null)
							{
								_mainContext.InnovationDatas.Attach(innovationData);
								ent.CurrentStage = innovationData;
							}
						}

						innovationProgress.LastEdit = DateTime.Now;

						var attachedEntry = _mainContext.Entry(ent);
						attachedEntry.CurrentValues.SetValues(innovationProgress);
					}

                    return _mainContext.SaveChanges(userName);
				}
			}
			catch (Exception ex)
			{
				throw ex;
			}
			
			return 0;
		}

		/// <summary>
		/// Gets the innocation progress.
		/// </summary>
		/// <param name="innovationId">The innovation id.</param>
		/// <returns></returns>
		public InnovationProgress GetInnovationProgress(int innovationId)
		{
			try
			{
				using (var _mainContext = new MainContext())
				{
					var innovation = _mainContext.Innovations.Find(innovationId);
					if (innovation != null)
					{
						var progress =
							_mainContext.InnovationProgresses.Include("CurrentStage")
										.FirstOrDefault(p => p.Innovation.ID == innovation.ID);

						if (progress != null)
						{
							progress.Innovation = innovation;
						}

						return progress;
					}
				}
			}
			catch (Exception ex)
			{
				throw ex;
			}

			return null;
		}

        /// <summary>
        /// Gets the innocation progress.
        /// </summary>
        /// <param name="innovationId">The innovation id.</param>
        /// <returns></returns>
        public InnovationProgress GetInnovationProgressByInRef(string inRef)
        {
            try
            {
                using (var _mainContext = new MainContext())
                {
                    var innovation = _mainContext.Innovations.Where(x => x.InnovationRef == inRef).FirstOrDefault();
                    if (innovation != null)
                    {
                        var progress =
                            _mainContext.InnovationProgresses.Include("CurrentStage").Where(x => x.InnovationRef == innovation.InnovationRef || x.Innovation.ID == innovation.ID).
                                FirstOrDefault();
                        return progress;
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return null;
        }
	}
}