﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.Entity;
using System.Data;
using Crossrail.IMS.Common;
using Crossrail.IMS.DataGateway.Utilities;
using log4net;
using System.Data.Entity.Validation;

namespace Crossrail.IMS.DataGateway.Repositories
{
    public class InnovationRepository : IInnovationRepository
    {
        private static readonly ILog log = LogManager.GetLogger(typeof(InnovationRepository).Name);
        public InnovationRepository()
        {
            Database.SetInitializer<MainContext>(null);
        }

        public int AddInnovation(Innovation innovation, string userName)
        {
            try
            {
                using (var context = new MainContext())
                {
                    //Create tempt innovation
                    //innovation.IsTemp = true;
                    innovation.InnovationRef = CommonUtil.GetNextInnovationRef();
                    context.Innovations.Add(innovation);
                    context.InnovationTrackers.Add(new InnovationTracker
                    {
                        ID = Guid.NewGuid(),
                        InnovationRef = innovation.InnovationRef,
                        IsCurrent = true,
                        Stage = innovation.CurrentStage == null ? "NULL" : context.InnovationDatas.Where(c => c.ID == innovation.CurrentStage.ID).Select(c => c.DataValue).FirstOrDefault() ?? "NULL",
                        PreviousStage = "NULL",
                        TimeAndDate = DateTime.Now
                    });
                    context.SaveChanges(userName);
                    return innovation.ID;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

        }

        public Innovation GetInnovationById(int innovationId)
        {
            Innovation result = null;
            using (var context = new MainContext())
            {
                result = context.Innovations
                    .Include(c => c.ProposingItems1)
                    .Include(c => c.ProposingItems2)
                    .FirstOrDefault(i => i.ID == innovationId);
            }
            return result;
        }

        public int UpdateInnovation(Innovation innovation, string userName)
        {
            using (var context = new MainContext())
            {
                var systemInnovation = context.Innovations
                    .Include(c => c.ProposingItems1)
                    .Include(c => c.ProposingItems2)
                    .FirstOrDefault(i => i.ID == innovation.ID);

                if (systemInnovation != null)
                {
                    var oldStage = context.InnovationProgresses.Where(c => c.Innovation.ID == systemInnovation.ID).Select(c => c.CurrentStage).Select(c => c.DataValue).FirstOrDefault();

                    systemInnovation.InnovationTitle = innovation.InnovationTitle;
                    systemInnovation.Description = innovation.Description;
                    systemInnovation.InnovationContext = innovation.InnovationContext;
                    systemInnovation.HowCanWeHelpID = innovation.HowCanWeHelpID;
                    systemInnovation.IndividualOrSharedInnovationID = innovation.IndividualOrSharedInnovationID;
                    systemInnovation.Who = innovation.Who;
                    systemInnovation.DateReceipt = innovation.DateReceipt;
                    systemInnovation.ParentCompany = innovation.ParentCompany;
                    systemInnovation.ContactFirstName = innovation.ContactFirstName;
                    systemInnovation.ContactSurName = innovation.ContactSurName;
                    systemInnovation.ContactTelephone = innovation.ContactTelephone;
                    systemInnovation.ContactEmail = innovation.ContactEmail;
                    systemInnovation.ProposingSiteProjectID = innovation.ProposingSiteProjectID;
                    systemInnovation.SponsoringChampion = innovation.SponsoringChampion;
                    systemInnovation.Area = innovation.Area;
                    systemInnovation.DetailedDescription = innovation.DetailedDescription;
                    systemInnovation.Keywords = innovation.Keywords;
                    systemInnovation.ThemeID = innovation.ThemeID;
                    systemInnovation.SubThemeID = innovation.SubThemeID;
                    systemInnovation.Benefit = innovation.Benefit;
                    systemInnovation.BenefitClearlyUnderstoodId = innovation.BenefitClearlyUnderstoodId;
                    systemInnovation.WhoWillBenefit = innovation.WhoWillBenefit;
                    systemInnovation.FunctionalSponsor = innovation.FunctionalSponsor;
                    systemInnovation.SuitablyDevelopedToShareID = innovation.SuitablyDevelopedToShareID;
                    systemInnovation.CurrentIRLID = innovation.CurrentIRLID;
                    systemInnovation.TargetIRLID = innovation.TargetIRLID;
                    systemInnovation.DesiredOutcome = innovation.DesiredOutcome;
                    systemInnovation.Actions = innovation.Actions;
                    systemInnovation.IsSubmitted = innovation.IsSubmitted;
                    systemInnovation.SubmittedForCompetitionDate = innovation.SubmittedForCompetitionDate;
                    systemInnovation.IsSafety = innovation.IsSafety;
                    systemInnovation.IsInspiration = innovation.IsInspiration;
                    systemInnovation.IsCollaboration = innovation.IsCollaboration;
                    systemInnovation.IsIntegrity = innovation.IsIntegrity;
                    systemInnovation.IsRespect = innovation.IsRespect;
                    systemInnovation.Notes = innovation.Notes;
                    systemInnovation.IdeaTypeID = innovation.IdeaTypeID;
                    systemInnovation.IdeaMaturityID = innovation.IdeaMaturityID;
                    systemInnovation.PrimaryBenefitID = innovation.PrimaryBenefitID;
                    systemInnovation.InnovationSourceID = innovation.InnovationSourceID;
                    systemInnovation.HoursDeveloping = innovation.HoursDeveloping;
                    systemInnovation.TaskGroupMembers = innovation.TaskGroupMembers;
                    systemInnovation.AssignedInnovationTeamMemberID = innovation.AssignedInnovationTeamMemberID;

                    systemInnovation.IR_Assumptions = innovation.IR_Assumptions;
                    systemInnovation.IR_DesiredOutcome = innovation.IR_DesiredOutcome;
                    systemInnovation.IR_FundingSought = innovation.IR_FundingSought;
                    systemInnovation.IR_InnovativeSol = innovation.IR_InnovativeSol;
                    systemInnovation.IR_IssueOpportunity = innovation.IR_IssueOpportunity;
                    systemInnovation.IR_PrimaryBenefit = innovation.IR_PrimaryBenefit;


                    systemInnovation.IsTemp = false;

                    systemInnovation.ProposingItems1.Clear();
                    systemInnovation.ProposingItems2.Clear();
                    if (innovation.ProposingItems1 != null)
                    {
                        foreach (var item in innovation.ProposingItems1)
                        {
                            systemInnovation.ProposingItems1.Add(item);
                        }
                    }
                    if (innovation.ProposingItems2 != null)
                    {
                        foreach (var item in innovation.ProposingItems2)
                        {
                            systemInnovation.ProposingItems2.Add(item);
                        }
                    }
                    var list = context.InnovationRemitAuthProceeds.Where(c => c.InnovationID == systemInnovation.ID).ToList();
                    foreach (var temp in list)
                    {
                        context.InnovationRemitAuthProceeds.Remove(temp);
                    }
                    if (innovation.InnovationRemitAuthProceeds != null)
                    {
                        foreach (var item in innovation.InnovationRemitAuthProceeds)
                        {
                            item.InnovationID = systemInnovation.ID;
                            context.InnovationRemitAuthProceeds.Add(item);
                        }
                    }
                    foreach (var temp in context.InnovationRemitImps.Where(c => c.InnovationID == systemInnovation.ID))
                    {
                        context.InnovationRemitImps.Remove(temp);
                    }
                    if (innovation.InnovationRemitImps != null)
                    {
                        foreach (var item in innovation.InnovationRemitImps)
                        {
                            item.InnovationID = systemInnovation.ID;
                            context.InnovationRemitImps.Add(item);
                        }
                    }
                    foreach (var temp in context.InnovationRemitResponsibles.Where(c => c.InnovationID == systemInnovation.ID))
                    {
                        context.InnovationRemitResponsibles.Remove(temp);
                    }
                    if (innovation.InnovationRemitResponsibles != null)
                    {
                        foreach (var item in innovation.InnovationRemitResponsibles)
                        {
                            item.InnovationID = systemInnovation.ID;
                            context.InnovationRemitResponsibles.Add(item);
                        }
                    }
                    foreach (var temp in context.InnovationRemitSuccessFailures.Where(c => c.InnovationID == systemInnovation.ID))
                    {
                        context.InnovationRemitSuccessFailures.Remove(temp);
                    }
                    if (innovation.InnovationRemitSuccessFailures != null)
                    {
                        foreach (var item in innovation.InnovationRemitSuccessFailures)
                        {
                            item.InnovationID = systemInnovation.ID;
                            context.InnovationRemitSuccessFailures.Add(item);
                        }
                    }
                    foreach (var temp in context.InnovationRemitSuppoters.Where(c => c.InnovationID == systemInnovation.ID))
                    {
                        context.InnovationRemitSuppoters.Remove(temp);
                    }
                    if (innovation.InnovationRemitSuppoters != null)
                    {
                        foreach (var item in innovation.InnovationRemitSuppoters)
                        {
                            item.InnovationID = systemInnovation.ID;
                            context.InnovationRemitSuppoters.Add(item);
                        }
                    }

                    var newStage = innovation.CurrentStage == null ? null : context.InnovationDatas.Where(c => c.ID == innovation.CurrentStage.ID).Select(c => c.DataValue).FirstOrDefault();
                    if (oldStage != newStage)
                    {
                        var history = context.InnovationTrackers.Where(c => c.InnovationRef == systemInnovation.InnovationRef).ToList();
                        foreach (var item in history)
                        {
                            item.IsCurrent = false;
                        }
                        context.InnovationTrackers.Add(new InnovationTracker
                        {
                            ID = Guid.NewGuid(),
                            InnovationRef = systemInnovation.InnovationRef,
                            IsCurrent = true,
                            Stage = newStage,
                            PreviousStage = oldStage,
                            TimeAndDate = DateTime.Now
                        });
                    }

                    context.Entry(systemInnovation).State = EntityState.Modified;
                    try
                    {
                        return context.SaveChanges(userName);
                    }
                    catch (DbEntityValidationException ex)
                    {
                        // Retrieve the error messages as a list of strings.
                        var errorMessages = ex.EntityValidationErrors
                                .SelectMany(x => x.ValidationErrors)
                                .Select(x => x.ErrorMessage);

                        // Join the list to a single string.
                        var fullErrorMessage = string.Join("; ", errorMessages);

                        // Combine the original exception message with the new one.
                        var exceptionMessage = string.Concat(ex.Message, " The validation errors are: ", fullErrorMessage);

                        // Throw a new DbEntityValidationException with the improved exception message.
                        throw new DbEntityValidationException(exceptionMessage, ex.EntityValidationErrors);
                    }
                }
            }
            return -1;
        }

        public bool DeleteInnovation(int innovationId, string userName)
        {
            using (var context = new MainContext())
            {
                var innovation = context.Innovations.FirstOrDefault(c => c.ID == innovationId);
                if (innovation != null)
                {
                    context.Innovations.Remove(innovation);
                    context.SaveChanges(userName);
                }
            }
            return true;
        }

        public int CancelinnovationDetail(int innovationId, string innovationRef, string userName)
        {
            int result = 0;
            try
            {
                using (var context = new MainContext())
                {
                    //Delete Innovation
                    Innovation innovation = context.Innovations.Where(i => i.ID == innovationId).First();
                    if (innovation.IsTemp == true)
                    {
                        //Delete Financial
                        InnovationFinancial innovationFinancial = context.InnovationFinancials.Where(i => i.Innovation.ID == innovationId).First();
                        if (innovationFinancial != null)
                        {
                            context.InnovationFinancials.Remove(innovationFinancial);

                        }

                        //Delete LinkedDocuments
                        List<LinkedDocument> linkedDocuments = context.LinkedDocuments.Where(i => i.Innovation.ID == innovationId).ToList();
                        foreach (var item in linkedDocuments)
                        {
                            context.LinkedDocuments.Remove(item);
                        }

                        //Delete PeopleSupport
                        List<PeopleSupport> peopleSupports = context.PeopleSupports.Where(i => i.Innovation.ID == innovationId).ToList();
                        foreach (var item in peopleSupports)
                        {
                            context.PeopleSupports.Remove(item);
                        }

                        //Delete ResourcesRequired
                        List<ResourcesRequired> resourcesRequireds = context.ResourcesRequireds.Where(i => i.Innovation.ID == innovationId).ToList();
                        foreach (var item in resourcesRequireds)
                        {
                            context.ResourcesRequireds.Remove(item);
                        }

                        //Delete ActionsRequired
                        List<ActionsRequired> actionsRequireds = context.ActionsRequireds.Where(i => i.Innovation.ID == innovationId).ToList();
                        foreach (var item in actionsRequireds)
                        {
                            context.ActionsRequireds.Remove(item);
                        }

                        //Delete Progress
                        List<InnovationProgress> innovationProgresses = context.InnovationProgresses.Where(i => i.Innovation.ID == innovationId).ToList();
                        foreach (var item in innovationProgresses)
                        {
                            context.InnovationProgresses.Remove(item);
                        }

                        var list = context.InnovationRemitAuthProceeds.Where(i => i.InnovationID == innovationId).ToList();
                        foreach (var item in list)
                        {
                            context.InnovationRemitAuthProceeds.Remove(item);
                        }

                        var list1 = context.InnovationRemitImps.Where(i => i.InnovationID == innovationId).ToList();
                        foreach (var item in list1)
                        {
                            context.InnovationRemitImps.Remove(item);
                        }

                        var list2 = context.InnovationRemitResponsibles.Where(i => i.InnovationID == innovationId).ToList();
                        foreach (var item in list2)
                        {
                            context.InnovationRemitResponsibles.Remove(item);
                        }

                        var list3 = context.InnovationRemitSuccessFailures.Where(i => i.InnovationID == innovationId).ToList();
                        foreach (var item in list3)
                        {
                            context.InnovationRemitSuccessFailures.Remove(item);
                        }

                        var list4 = context.InnovationRemitSuppoters.Where(i => i.InnovationID == innovationId).ToList();
                        foreach (var item in list4)
                        {
                            context.InnovationRemitSuppoters.Remove(item);
                        }

                        //Delete Innovation
                        context.Innovations.Remove(innovation);
                    }

                    result = context.SaveChanges();
                }
            }
            catch (System.Data.SqlClient.SqlException slqException)
            {
                throw slqException;
            }
            catch (Exception ex)
            {
                throw ex;
            }


            return result;
        }

        public List<Innovation> GetAllInnovations(string userName, int pageNumber, int pageSize, string sortColumn, string sortDir, out int total)
        {
            var result = new List<Innovation>();
            total = 0;
            using (var context = new MainContext())
            {
                var user = context.Users.FirstOrDefault(u => u.UserName == userName);
                if (user != null)
                {
                    var role = context.Roles.FirstOrDefault(r => r.ID == user.RoleID);
                    if (role != null)
                    {
                        var proposingItem = new List<int>();
                        bool isCompareProposingItem = false;

                        switch (role.RoleName)
                        {
                            case Constant.Role.SystemAdministrator:
                            case Constant.Role.InnovationTeam:
                                {
                                    //can access all records
                                    break;
                                }
                            case Constant.Role.InnovationChampion:
                            case Constant.Role.ReadOnly:
                                {
                                    //must compare the proposing item?
                                    isCompareProposingItem = true;

                                    //get all proposing items of the current user

                                    proposingItem = (from item in context.UserProposingSiteProjects
                                                     where item.UserID == user.ID
                                                     select item.InnovationDataID).ToList();
                                    break;
                                }
                        }
                        var q = context.InnovationProgresses.Include(c => c.CurrentStage).Select(c => new { InnovationID = c.Innovation.ID, CurrentStage = c.CurrentStage.DataValue, NextSteps = c.NextSteps });
                        var query = (from innovation in context.Innovations
                                         .Include("Theme")
                                         //.Include("SubTheme") - CR -remove this field
                                         //.Include("CurrentIRL") - CR - remove this field
                                         .Include("ProposingItem")
                                     where (!isCompareProposingItem
                                            || (innovation.ProposingSiteProjectID != null
                                                && proposingItem.Contains(innovation.ProposingSiteProjectID.Value)))
                                           && innovation.IsTemp == false
                                     select new
                                     {
                                         Innovation = innovation,
                                         CurrentStage = q.Where(c => c.InnovationID == innovation.ID).Select(c => c.CurrentStage).FirstOrDefault(),
                                         NextSteps = q.Where(c => c.InnovationID == innovation.ID).Select(c => c.NextSteps).FirstOrDefault()
                                     }
                                    );

                        IQueryable<Innovation> tmpResults = null;
                        if (sortColumn == "InnovationTitle")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = query.OrderByDescending(i => i.Innovation.InnovationTitle).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = query.OrderBy(i => i.Innovation.InnovationTitle).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "ProposingItem")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = query.OrderByDescending(i => i.Innovation.ProposingItem.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = query.OrderBy(i => i.Innovation.ProposingItem.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "ThemeName")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = query.OrderByDescending(i => i.Innovation.Theme.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = query.OrderBy(i => i.Innovation.Theme.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "CurrentStage")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = query.OrderByDescending(i => i.CurrentStage).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = query.OrderBy(i => i.CurrentStage).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "NullableSubmittedForCompetitionDate")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = query.OrderByDescending(i => i.Innovation.SubmittedForCompetitionDate).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = query.OrderBy(i => i.Innovation.SubmittedForCompetitionDate).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "NextStepsDisplayString")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = query.OrderByDescending(i => i.NextSteps).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = query.OrderBy(i => i.NextSteps).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = query.OrderByDescending(i => i.Innovation.InnovationRef).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = query.OrderBy(i => i.Innovation.InnovationRef).Select(c => c.Innovation).AsQueryable();
                            }
                        }

                        total = tmpResults.Count();
                        tmpResults = tmpResults.Skip((pageNumber - 1) * pageSize).Take(pageSize);

                        foreach (var innovation in tmpResults.ToList())
                        {
                            var innovationProgress = context.InnovationProgresses.Include(x => x.CurrentStage).FirstOrDefault(
                                item => item.Innovation != null && item.Innovation.ID == innovation.ID);
                            if (innovationProgress != null)
                            {
                                innovation.CurrentStage = innovationProgress.CurrentStage;
                                innovation.NextSteps = string.IsNullOrEmpty(innovationProgress.NextSteps)
                                                           ? string.Empty
                                                           : innovationProgress.NextSteps.Trim();
                            }
                            var theme = context.InnovationDatas.FirstOrDefault(c => c.ID == innovation.ThemeID);
                            if (theme != null)
                            {
                                innovation.Theme = theme;
                            }
                            var proposing = context.InnovationDatas.FirstOrDefault(c => c.ID == innovation.ProposingSiteProjectID);
                            if (proposing != null)
                            {
                                innovation.ProposingItem = proposing;
                            }

                            result.Add(innovation);
                        }
                    }
                }
            }

            return result;
        }

        public List<InnovationRemitImp> GetInnovationRemitImps(int innovationId)
        {
            using (var db = new MainContext())
            {
                return db.InnovationRemitImps.Where(c => c.InnovationID == innovationId).ToList();
            }
        }

        public List<InnovationRemitResponsible> GetInnovationRemitResponsibles(int innovationId)
        {
            using (var db = new MainContext())
            {
                return db.InnovationRemitResponsibles.Where(c => c.InnovationID == innovationId).ToList();
            }
        }

        public List<InnovationRemitSuccessFailure> GetInnovationRemitSuccessFailures(int innovationId)
        {
            using (var db = new MainContext())
            {
                return db.InnovationRemitSuccessFailures.Where(c => c.InnovationID == innovationId).ToList();
            }
        }

        public List<InnovationRemitAuthProceed> GetInnovationRemitAuthProceeds(int innovationId)
        {
            using (var db = new MainContext())
            {
                return db.InnovationRemitAuthProceeds.Where(c => c.InnovationID == innovationId).ToList();
            }
        }

        public List<InnovationRemitSuppoter> GetInnovationRemitSuppoters(int innovationId)
        {
            using (var db = new MainContext())
            {
                return db.InnovationRemitSuppoters.Where(c => c.InnovationID == innovationId).ToList();
            }
        }
    }
}