﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;

namespace Crossrail.IMS.DataGateway.Repositories
{
    public class LinkedDocumentsTemporaryRepository : ILinkedDocumentsTemporaryRepository
    {
        public int AddLinkedDocumentsTemporary(LinkedDocumentsTemporary linkedDocumentsTemporary, string userName)
        {
            try
            {
                using (var _mainContext = new MainContext())
                {
                    linkedDocumentsTemporary.Innovation = null;
                    _mainContext.LinkedDocumentTemps.Add(linkedDocumentsTemporary);
                    _mainContext.SaveChanges(userName);
                    return linkedDocumentsTemporary.ID;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public int UpdateLinkedDocumentsTemporary(LinkedDocumentsTemporary linkedDocumentsTemporary, string userName)
        {
            try
            {
                using (var _mainContext = new MainContext())
                {
                    var ent = _mainContext.LinkedDocumentTemps.Find(linkedDocumentsTemporary.ID);
                    if (ent != null)
                    {
                        var attachedEntry = _mainContext.Entry(ent);
                        attachedEntry.CurrentValues.SetValues(linkedDocumentsTemporary);
                    }
                    else
                    {
                        _mainContext.Entry(linkedDocumentsTemporary).State = EntityState.Modified;
                    }

                    return _mainContext.SaveChanges(userName);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public int DeleteLinkedDocumentsTemporary(int id, string userName)
        {
            try
            {
                using (var _mainContext = new MainContext())
                {
                    var entity = _mainContext.LinkedDocumentTemps.Find(id);
                    if (entity != null)
                    {
                        _mainContext.LinkedDocumentTemps.Remove(entity);
                        return _mainContext.SaveChanges(userName);
                    }
                    return 0;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public List<LinkedDocumentsTemporary> GetAllLinkedDocumentsTemporaryByInnovationId(int innovationId, Guid temptId)
        {
            try
            {
                using (var _mainContext = new MainContext())
                {
                    List<LinkedDocumentsTemporary> result = null;
                    if (innovationId != 0)
                    {
                        result = _mainContext.LinkedDocumentTemps.Where(p => p.Innovation.ID == innovationId).ToList();
                    }
                    else
                    {
                        result = _mainContext.LinkedDocumentTemps.Where(p => p.TemptId == temptId).ToList();
                    }
                    return result;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public void FillLinkedDocumentsTemporaryByInnovationId(int innovationId)
        {
            try
            {
                using (var _mainContext = new MainContext())
                {
                    List<LinkedDocument> result = _mainContext.LinkedDocuments.Where(p => p.Innovation.ID == innovationId).ToList();
                    var innovation = _mainContext.Innovations.Find(innovationId);
                    result.ForEach(p =>
                    {
                        var temp = new LinkedDocumentsTemporary()
                        {
                            Innovation = innovation,
                            OrgID = p.ID,
                            DocumentPath = p.DocumentPath,
                            DocumentDescription = p.DocumentDescription,
                            DocumentURL = p.DocumentURL,
                            TemptId = p.TemptId
                        };

                        _mainContext.LinkedDocumentTemps.Add(temp);
                    });

                    _mainContext.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public void CleanupLinkedDocumentsTemporaryByInnovationId(int innovationId)
        {
            try
            {
                using (var _mainContext = new MainContext())
                {
                    var linkedDocument = _mainContext.LinkedDocuments.Where(p => p.Innovation.ID == innovationId).FirstOrDefault();
                    if (linkedDocument != null)
                    {
                        var result = _mainContext.LinkedDocumentTemps.Where(p => p.TemptId == linkedDocument.TemptId || p.Innovation.ID == innovationId).ToList();
                        result.ForEach(p =>

                            _mainContext.LinkedDocumentTemps.Remove(p)
                        );

                        _mainContext.SaveChanges();
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public int SaveLinkedDocuments(int innovationId, string userName, List<LinkedDocumentsTemporary> linkedDocumentsTemporaryList)
        {
            try
            {
                using (var _mainContext = new MainContext())
                {
                    var innovation = _mainContext.Innovations.Find(innovationId);

                    var linkedDocumentsTemporarys = _mainContext.LinkedDocuments.Where(p => p.Innovation.ID == innovationId).ToList();
                    linkedDocumentsTemporarys.ForEach(p =>
                    {
                        var tempEntity = linkedDocumentsTemporaryList.Find(e => e.OrgID == p.ID);
                        //Delete
                        if (ReferenceEquals(tempEntity, null))
                        {
                            _mainContext.LinkedDocuments.Remove(p);
                        }
                        //Update
                        else
                        {
                            _mainContext.Innovations.Attach(innovation);
                            var ent = _mainContext.LinkedDocuments.Find(p.ID);
                            ent.Innovation = innovation;
                            ent.DocumentDescription = tempEntity.DocumentDescription;
                            ent.DocumentPath = tempEntity.DocumentPath;
                            ent.DocumentURL = tempEntity.DocumentURL;
                            ent.TemptId = tempEntity.TemptId;
                            _mainContext.Entry(ent).State = EntityState.Modified;
                        }
                    });

                    linkedDocumentsTemporaryList.ForEach(t =>
                    {
                        var p = linkedDocumentsTemporarys.Find(e => e.ID == t.OrgID);
                        if (ReferenceEquals(p, null))
                        {
                            //Add New
                            _mainContext.LinkedDocuments.Add(new LinkedDocument()
                            {
                                DocumentDescription = t.DocumentDescription,
                                DocumentPath = t.DocumentPath,
                                DocumentURL = t.DocumentURL,
                                TemptId = t.TemptId,
                                Innovation = innovation
                            });
                        }
                    });
                    return _mainContext.SaveChanges(userName);
                }

                return 0;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
    }
}