﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Web;
using Crossrail.IMS.Common;
using Crossrail.IMS.DataGateway.Dtos;
using Infragistics.Documents.Excel;

namespace Crossrail.IMS.DataGateway.Repositories
{
    public class ReportInnovationRepository : IReportInnovationRepository
    {
        public List<SubTheme> getSubThemesByThemeId(int themeId)
        {
            using (var context = new MainContext())
            {
                var ret = (from p in context.SubThemes
                           where p.InnovationData.ID == themeId
                           select p).ToList();
                return ret;
            }
        }

        public List<InnovationPeriod> GetInnovationPeriods(string filePath)
        {
            using (var context = new MainContext())
            {
                //filePath = ConfigurationManager.AppSettings["PeriodFilePath"].ToString();
                //Hung Hoang : Use the relative file path
                string fileName = ConfigurationManager.AppSettings["PeriodFilePath"].ToString();
                filePath = Path.Combine(System.Web.Hosting.HostingEnvironment.ApplicationPhysicalPath, "App_Data", fileName);

                Workbook workbook = Workbook.Load(filePath);
                int colNo = workbook.ActiveWorksheet.Columns.Count();
                int rowNo = workbook.ActiveWorksheet.Rows.Count();
                var resultList = new List<InnovationPeriod>();
                for (int i = 0; i < rowNo; i++)
                {
                    if (i == 0)
                    {
                        continue;
                    }
                    var item = new InnovationPeriod
                                   {
                                       Code = workbook.ActiveWorksheet.Rows[i].Cells[0].Value.ToString(), //excelReader.GetString(0),
                                       Name = workbook.ActiveWorksheet.Rows[i].Cells[1].Value.ToString(),
                                       LongName = workbook.ActiveWorksheet.Rows[i].Cells[2].Value.ToString(),
                                       Fiscal_Period = workbook.ActiveWorksheet.Rows[i].Cells[3].Value.ToString(),
                                       FiscalYearCode = workbook.ActiveWorksheet.Rows[i].Cells[4].Value.ToString(),
                                       StartDate =
                                           DateTime.ParseExact(workbook.ActiveWorksheet.Rows[i].Cells[5].Value.ToString(),
                                                               "dd/MM/yyyy HH:mm:ss", null),
                                       EndDate =
                                           DateTime.ParseExact(workbook.ActiveWorksheet.Rows[i].Cells[6].Value.ToString(),
                                                               "dd/MM/yyyy HH:mm:ss", null)
                                   };
                    resultList.Add(item);
                }
                var first = new InnovationPeriod
                                {
                                    Code = string.Empty,
                                    Name = "---select---"
                                };
                resultList.Insert(0, first);
                return resultList;
            }
        }

        public List<DataNewSubmissionReport> exportNewSubmissionReport(IList<InnovationPeriod> periods)
        {
            using (var context = new MainContext())
            {
                var returnList = new List<DataNewSubmissionReport>();
                var innovationDtos = new List<InnovationDto>();
                foreach (var item in periods)
                {
                    var ins = (from p in context.Innovations
                               where (item.EndDate == null && p.DateReceipt.Value.Year == item.StartDate.Year) || (p.DateReceipt >= item.StartDate && p.DateReceipt <= item.EndDate)
                               select new InnovationDto
                               {
                                   PeriodName = item.Name,
                                   Innovation = p
                               }).ToList();
                    innovationDtos.AddRange(ins.ToList());
                }

                var themeIds = innovationDtos.Select(x => x.Innovation).Select(s => s.ThemeID).Distinct().ToList();
                var subThemeIds = innovationDtos.Select(x => x.Innovation).Select(s => s.SubThemeID).Distinct().ToList();

                foreach (var inItem in innovationDtos.GroupBy(x => x.PeriodName).ToList())
                {
                    var reportItem = new DataNewSubmissionReport();
                    reportItem.Period = inItem.FirstOrDefault().PeriodName;
                    reportItem.Total = inItem.Count(x => x.PeriodName == inItem.FirstOrDefault().PeriodName);
                    reportItem.Themes = context.InnovationDatas.Where(x => themeIds.Contains(x.ID)).ToList();
                    reportItem.SubThemes = context.InnovationDatas.Where(x => subThemeIds.Contains(x.ID)).ToList();
                    reportItem.SubThemeList = context.SubThemes.Where(x => themeIds.Contains(x.InnovationData.ID)).ToList();
                    reportItem.Innovations = innovationDtos.Where(t => t.PeriodName == inItem.FirstOrDefault().PeriodName).Select(x => x.Innovation).ToList();

                    returnList.Add(reportItem);


                }
                return returnList;
            }
        }

        public List<DataStageInnovationReport> exportStageSubmissionReport(IList<InnovationPeriod> periods)
        {
            using (var context = new MainContext())
            {
                var returnList = new List<DataStageInnovationReport>();
                var innovationDtos = new List<InnovationDto>();
                foreach (var item in periods)
                {
                    var ins = from p in context.Innovations
                              where (item.StartDate == null && p.DateReceipt.Value.Year == item.EndDate.Year) || (p.DateReceipt >= item.StartDate && p.DateReceipt <= item.EndDate)
                              select new InnovationDto
                              {
                                  PeriodName = item.Name,
                                  Innovation = p,
                                  InnovationStage = context.InnovationProgresses.Where(x => x.InnovationRef == p.InnovationRef).FirstOrDefault().CurrentStage.DataValue
                              };
                    innovationDtos.AddRange(ins.ToList());
                }

                var themeIds = innovationDtos.Select(x => x.Innovation).Select(s => s.ThemeID).Distinct().ToList();
                var subThemeIds = innovationDtos.Select(x => x.Innovation).Select(s => s.SubThemeID).Distinct().ToList();
                var stages = innovationDtos.Select(s => s.InnovationStage).Distinct().ToList();

                foreach (var inItem in innovationDtos.GroupBy(x => x.InnovationStage).ToList())
                {
                    var reportItem = new DataStageInnovationReport
                    {
                        Total = inItem.Count(x => x.InnovationStage == inItem.FirstOrDefault().InnovationStage),
                        Themes = context.InnovationDatas.Where(x => themeIds.Contains(x.ID)).ToList(),
                        SubThemes = context.InnovationDatas.Where(x => subThemeIds.Contains(x.ID)).ToList(),
                        Innovations = innovationDtos.Where(t => t.InnovationStage == inItem.FirstOrDefault().InnovationStage).Select(x => x.Innovation).ToList(),
                        Stage = inItem.FirstOrDefault().InnovationStage,
                        SubThemeList = context.SubThemes.Where(x => themeIds.Contains(x.InnovationData.ID)).ToList(),
                    };
                    returnList.Add(reportItem);


                }
                return returnList;
            }
        }

        public List<DataNewSubmissionReport> exportExtractSubmissionReport(IList<InnovationPeriod> periods)
        {
            using (var context = new MainContext())
            {
                var returnList = new List<DataNewSubmissionReport>();
                var innovationDtos = new List<InnovationDto>();
                foreach (var item in periods)
                {
                    var ins = from p in context.Innovations
                              where (item.StartDate == null && p.DateReceipt.Value.Year == item.StartDate.Year) || (p.DateReceipt >= item.StartDate && p.DateReceipt <= item.EndDate)
                              select new InnovationDto
                              {
                                  PeriodName = item.Name,
                                  Innovation = p
                              };
                    innovationDtos.AddRange(ins.ToList());
                }

                var themeIds = innovationDtos.Select(x => x.Innovation).Select(s => s.ThemeID).Distinct().ToList();
                var subThemeIds = innovationDtos.Select(x => x.Innovation).Select(s => s.SubThemeID).Distinct().ToList();

                foreach (var inItem in innovationDtos.GroupBy(x => x.PeriodName).ToList())
                {
                    var reportItem = new DataNewSubmissionReport
                    {
                        Period = inItem.FirstOrDefault().PeriodName,
                        Total = inItem.Count(x => x.PeriodName == inItem.FirstOrDefault().PeriodName),
                        Themes = context.InnovationDatas.Where(x => themeIds.Contains(x.ID)).ToList(),
                        SubThemes = context.InnovationDatas.Where(x => subThemeIds.Contains(x.ID)).ToList(),
                        Innovations = innovationDtos.Where(t => t.PeriodName == inItem.FirstOrDefault().PeriodName).Select(x => x.Innovation).ToList(),
                    };
                    returnList.Add(reportItem);


                }
                return returnList;
            }
        }

    }
}