﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Objects;
using System.Data.Objects.SqlClient;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using Crossrail.IMS.DataGateway.Entities;
using Crossrail.IMS.Common;

namespace Crossrail.IMS.DataGateway.Repositories
{
    public class SearchInnovationRepository : ISearchInnovationRepository
    {
        public SearchInnovationRepository()
        {
            Database.SetInitializer<MainContext>(null);
        }

        public List<Innovation> SearchInnovation(SearchInnovationCriteria criteria, string userName, int pageNumber, int pageSize, string sortColumn, string sortDir, out int total)
        {
            if (criteria.SearchMode == SearchInnovationMode.Restricted)
                return SearchInnovationWithRestrictedMode(criteria, userName, pageNumber, pageSize, sortColumn, sortDir, out total);

            return SearchInnovationWithFullMode(criteria.SearchText, userName, pageNumber, pageSize, sortColumn, sortDir, out total);
        }

        private List<Innovation> SearchInnovationWithRestrictedMode(SearchInnovationCriteria criteria, string userName, int pageNumber, int pageSize, string sortColumn, string sortDir, out int total)
        {
            var results = new List<Innovation>();
            total = 0;
            using (var context = new MainContext())
            {
                var user = context.Users.FirstOrDefault(u => u.UserName == userName);
                if (user != null)
                {
                    var role = context.Roles.FirstOrDefault(r => r.ID == user.RoleID);
                    if (role != null)
                    {
                        IQueryable<Innovation> query = context.Innovations
                            .Include("Theme")
                            .Include("SubTheme")
                            .Include("CurrentIRL")
                            .Include("ProposingItem");

                        if (!string.IsNullOrEmpty(criteria.InnovationRef))
                            query = query.Where(i => i.InnovationRef.Contains(criteria.InnovationRef));
                        if (!string.IsNullOrEmpty(criteria.InnovationTitle))
                            query = query.Where(i => i.InnovationTitle.Contains(criteria.InnovationTitle));

                        if (criteria.ProposingItemID.HasValue)
                            query = query.Where(i => i.ProposingSiteProjectID == criteria.ProposingItemID.Value);
                        if (criteria.ThemeID.HasValue)
                            query = query.Where(i => i.ThemeID == criteria.ThemeID.Value);
                        if (criteria.SubThemeID.HasValue)
                            query = query.Where(i => i.SubThemeID == criteria.SubThemeID.Value);
                        if (criteria.CurrentIRLID.HasValue)
                            query = query.Where(i => i.CurrentIRLID == criteria.CurrentIRLID.Value);
                        if (criteria.CompetitionDateFrom.HasValue)
                            query =
                                query.Where(
                                    i =>
                                    EntityFunctions.TruncateTime(i.SubmittedForCompetitionDate)
                                    >= EntityFunctions.TruncateTime(criteria.CompetitionDateFrom.Value));

                        if (criteria.CompetitionDateTo.HasValue)
                            query =
                                query.Where(i => EntityFunctions.TruncateTime(i.SubmittedForCompetitionDate)
                                                 <= EntityFunctions.TruncateTime(criteria.CompetitionDateTo.Value));

                        query = query.Where(i => i.IsTemp == false);

                        switch (role.RoleName)
                        {
                            case Constant.Role.InnovationChampion:
                            case Constant.Role.ReadOnly:
                                {
                                    var proposingItems = (from item in context.UserProposingSiteProjects
                                                          where item.UserID == user.ID
                                                          select item.InnovationDataID).ToList();
                                    query =
                                        query.Where(
                                            i =>
                                            i.ProposingSiteProjectID.HasValue &&
                                            proposingItems.Contains(i.ProposingSiteProjectID.Value));
                                    break;
                                }
                        }

                        if (criteria.CurrentStageID.HasValue)
                        {
                            var query2 = (from item in context.InnovationProgresses
                                          where item.CurrentStage.ID == criteria.CurrentStageID.Value
                                          select item.Innovation.ID).ToList();
                            query = query.Where(i => query2.Contains(i.ID));
                        }

                        var q = context.InnovationProgresses.Include(c => c.CurrentStage).Select(c => new { InnovationID = c.Innovation.ID, CurrentStage = c.CurrentStage.DataValue, NextSteps = c.NextSteps });
                        var q1 = query.Select(innovation => new
                        {
                            Innovation = innovation,
                            CurrentStage = q.Where(c => c.InnovationID == innovation.ID).Select(c => c.CurrentStage).FirstOrDefault(),
                            NextSteps = q.Where(c => c.InnovationID == innovation.ID).Select(c => c.NextSteps).FirstOrDefault()
                        });

                        IQueryable<Innovation> tmpResults = null;
                        if (sortColumn == "InnovationTitle")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.InnovationTitle).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.InnovationTitle).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "ProposingItem")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.ProposingItem.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.ProposingItem.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "ThemeName")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.Theme.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.Theme.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "CurrentStage")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.CurrentStage).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.CurrentStage).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "NullableSubmittedForCompetitionDate")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.SubmittedForCompetitionDate).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.SubmittedForCompetitionDate).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "NextStepsDisplayString")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.NextSteps).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.NextSteps).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.InnovationRef).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.InnovationRef).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        total = tmpResults.Count();
                        tmpResults = tmpResults.Skip((pageNumber - 1) * pageSize).Take(pageSize);

                        foreach (var innovation in tmpResults.ToList())
                        {
                            var innovationProgress =
                                context.InnovationProgresses.Include(x => x.CurrentStage).FirstOrDefault(x => x.Innovation.ID == innovation.ID);
                            if (innovationProgress != null)
                            {
                                innovation.CurrentStage = innovationProgress.CurrentStage;
                                innovation.NextSteps = string.IsNullOrEmpty(innovationProgress.NextSteps)
                                                           ? string.Empty
                                                           : innovationProgress.NextSteps.Trim();
                            }
                            var theme = context.InnovationDatas.FirstOrDefault(c => c.ID == innovation.ThemeID);
                            if (theme != null)
                            {
                                innovation.Theme = theme;
                            }
                            var proposing = context.InnovationDatas.FirstOrDefault(c => c.ID == innovation.ProposingSiteProjectID);
                            if (proposing != null)
                            {
                                innovation.ProposingItem = proposing;
                            }

                            results.Add(innovation);
                        }
                    }
                }

                return results;
            }
        }

        private List<Innovation> SearchInnovationWithFullMode(string criteria, string userName, int pageNumber, int pageSize, string sortColumn, string sortDir, out int total)
        {
            if (string.IsNullOrWhiteSpace(criteria))
            {
                var innovationRepository = new InnovationRepository();
                return innovationRepository.GetAllInnovations(userName, pageNumber, pageSize, sortColumn, sortDir, out total);
            }


            var result = new List<Innovation>();
            total = 0;
            using (var context = new MainContext())
            {
                var user = context.Users.FirstOrDefault(u => u.UserName == userName);
                if (user != null)
                {
                    var role = context.Roles.FirstOrDefault(r => r.ID == user.RoleID);
                    if (role != null)
                    {
                        decimal dec;
                        bool isNumeric = decimal.TryParse(criteria, out dec);

                        #region innovation

                        var query = (from innovation in context.Innovations
                                         //.Include(i => i.HowCanWeHelp) CR P521/C1 - remove this field
                                         .Include(i => i.IndividualOrSharedInnovation)
                                         .Include(i => i.ProposingItem)
                                         .Include(i => i.Theme)
                                         .Include(i => i.SubTheme)
                                         .Include(i => i.BenefitClearlyUnderstood)
                                         .Include(i => i.SuitablyDevelopedToShare)
                                         .Include(i => i.CurrentIRL)
                                         .Include(i => i.TargetIRL)
                                         .Include(i => i.AreaData)
                                         .Include(i => i.IdeaType) //CR P521/C1 - new field
                                         .Include(i => i.PrimaryBenefit) //CR P521/C1 - new field
                                         .Include(i => i.IdeaMaturity) //CR P521/C1 - new field
                                         .Include(i => i.InnovationSource) //CR P521/C1 - new field
                                     where innovation.IsTemp == false
                                           &&
                                           (
                                               innovation.InnovationRef.Contains(criteria)
                                               || innovation.InnovationTitle.Contains(criteria)
                                               || innovation.Description.Contains(criteria)
                                               || innovation.InnovationContext.Contains(criteria)
                                               ||
                                         //(innovation.HowCanWeHelp != null &&
                                         //innovation.HowCanWeHelp.DataValue.Contains(criteria))
                                         //||
                                               (innovation.IndividualOrSharedInnovation != null &&
                                                innovation.IndividualOrSharedInnovation.DataValue.Contains(criteria))
                                               || innovation.Who.Contains(criteria)
                                               || innovation.ParentCompany.Contains(criteria)
                                               || innovation.ContactFirstName.Contains(criteria)
                                               || innovation.ContactSurName.Contains(criteria)
                                               || innovation.ContactTelephone.Contains(criteria)
                                               || innovation.ContactEmail.Contains(criteria)
                                               ||
                                               (innovation.ProposingItem != null &&
                                                innovation.ProposingItem.DataValue.Contains(criteria))
                                               || innovation.SponsoringChampion.Contains(criteria)
                                               || innovation.DetailedDescription.Contains(criteria)
                                               || innovation.Keywords.Contains(criteria)
                                               ||
                                               (innovation.Theme != null &&
                                                innovation.Theme.DataValue.Contains(criteria))
                                               ||
                                               (innovation.SubTheme != null &&
                                                innovation.SubTheme.DataValue.Contains(criteria))
                                               || innovation.Benefit.Contains(criteria)
                                               ||
                                               (innovation.BenefitClearlyUnderstood != null &&
                                                innovation.BenefitClearlyUnderstood.DataValue.Contains(criteria))
                                               || (innovation.WhoWillBenefit.Contains(criteria))
                                               || (innovation.FunctionalSponsor.Contains(criteria))
                                               ||
                                               (innovation.SuitablyDevelopedToShare != null &&
                                                innovation.SuitablyDevelopedToShare.DataValue.Contains(criteria))
                                               ||
                                               (innovation.CurrentIRL != null &&
                                                innovation.CurrentIRL.DataValue.Contains(criteria))
                                               ||
                                               (innovation.TargetIRL != null &&
                                                innovation.TargetIRL.DataValue.Contains(criteria))
                                               || innovation.DesiredOutcome.Contains(criteria)
                                               || innovation.Actions.Contains(criteria)
                                               || innovation.Notes.Contains(criteria)
                                               ||
                                               (innovation.AreaData != null &&
                                                innovation.AreaData.DataValue.Contains(criteria))
                                               ||
                                               (innovation.IdeaType != null &&
                                                innovation.IdeaType.DataValue.Contains(criteria)) //CR P521/C1 - new field
                                               ||
                                               (innovation.PrimaryBenefit != null &&
                                                innovation.PrimaryBenefit.DataValue.Contains(criteria)) //CR P521/C1 - new field
                                               ||
                                               (innovation.IdeaMaturity != null &&
                                                innovation.IdeaMaturity.DataValue.Contains(criteria)) //CR P521/C1 - new field
                                               ||
                                               (innovation.InnovationSource != null
                                                && innovation.InnovationSource.DataValue.Contains(criteria)) //CR P521/C1 - new field
                                               ||
                                               SqlFunctions.StringConvert(innovation.HoursDeveloping).Contains(criteria) //CR P521/C1 - new field
                                           )
                                     select innovation.ID);
                        #endregion

                        #region innovation financial

                        var query2 = (from innovationFinancial in context.InnovationFinancials
                                      where (isNumeric && innovationFinancial.TotalAuthorisedInvestment == dec)
                                            || SqlFunctions.StringConvert(innovationFinancial.ExpenditureToDate).Contains(criteria)
                                            || SqlFunctions.StringConvert(innovationFinancial.CRLAuthorisedInvestment).Contains(criteria) //(isNumeric && innovationFinancial.CRLAuthorisedInvestment == dec)
                                            || SqlFunctions.StringConvert(innovationFinancial.TSLGFundsSecured).Contains(criteria) //(isNumeric && innovationFinancial.TSLGFundsSecured == dec)
                                            || SqlFunctions.StringConvert(innovationFinancial.TfLFundsSecured).Contains(criteria) //(isNumeric && innovationFinancial.TfLFundsSecured == dec)
                                            || SqlFunctions.StringConvert(innovationFinancial.RIAFundsSecured).Contains(criteria) //(isNumeric && innovationFinancial.RIAFundsSecured == dec)
                                            || SqlFunctions.StringConvert(innovationFinancial.BISFundsSecured).Contains(criteria) //(isNumeric && innovationFinancial.BISFundsSecured == dec)
                                            || innovationFinancial.FundingComments.Contains(criteria)
                                            || innovationFinancial.IncifPurchaseOrderNumber.Contains(criteria)
                                            || SqlFunctions.StringConvert(innovationFinancial.IncifMaximumValueofWork).Contains(criteria) //(isNumeric && innovationFinancial.IncifMaximumValueofWork == dec)
                                            || innovationFinancial.IncifDelegatedAuthorityName.Contains(criteria)
                                            || innovationFinancial.IncifDelegatedAuthorityTitle.Contains(criteria)
                                            || innovationFinancial.IncifDelegatedAuthorityContactNumber.Contains(criteria)
                                            || innovationFinancial.ICSCRef.Contains(criteria)
                                            || innovationFinancial.IARef.Contains(criteria)
                                            || SqlFunctions.StringConvert(innovationFinancial.IAMaximumValueofWork).Contains(criteria)//(isNumeric && innovationFinancial.IAMaximumValueofWork == dec)
                                            || innovationFinancial.ExternalRef.Contains(criteria)
                                            || SqlFunctions.StringConvert(innovationFinancial.ExternalMaximumValueofWork).Contains(criteria) //(isNumeric && innovationFinancial.ExternalMaximumValueofWork == dec)
                                            || innovationFinancial.ExternalDelegatedAuthorityName.Contains(criteria)
                                            || innovationFinancial.ExternalDelegatedAuthorityContactNumber.Contains(criteria)
                                            || innovationFinancial.ExternalDelegatedAuthorityTitle.Contains(criteria)
                                            || innovationFinancial.ExternalFundingComments.Contains(criteria)
                                      select innovationFinancial.Innovation.ID);

                        #endregion

                        #region innovation progress

                        var query3 = from innovationProgress in context.InnovationProgresses
                                         .Include(i => i.WhoBenefitsData)
                                         .Include(i => i.WhatMagnitudeRequiredData)
                                         .Include(i => i.WhatImpactToProductivityData)
                                         .Include(i => i.HowIdeaImproveQualityData)
                                         .Include(i => i.HowStrongEvidenceImpactData)
                                         .Include(i => i.WhatFlexibilityIdeaAffordData)
                                         .Include(i => i.WhatMagnitudeIdeaImportstoCRLData)
                                         .Include(i => i.CompetitiveSensitivityWithIdeaData)
                                         .Include(i => i.HowWellDevelopedIdeaData)
                                         .Include(i => i.WhatLevelofSupportData)
                                         .Include(i => i.WhatLevelofEffortRequiredData)
                                         .Include(i => i.CurrentStage)
                                     where //innovationProgress.LatestUpdate.Contains(criteria) CR P521/C1 - remove this field
                                         //||
                                           innovationProgress.CurrentStage.DataValue.Contains(@"~!@#$%^&*()_+|?><.") // hack to fix nullable problems
                                           ||
                                           SqlFunctions.StringConvert(innovationProgress.CurrentStageComplete).Contains(
                                               criteria)
                                           || innovationProgress.NextSteps.Contains(criteria)
                                           || innovationProgress.WhyFailedSupport.Contains(criteria)
                                           || innovationProgress.HowToHelp.Contains(criteria)
                                           ||
                                           (innovationProgress.WhoBenefitsData != null &&
                                            innovationProgress.WhoBenefitsData.AnswerContent.Contains(criteria))
                                           ||
                                           (innovationProgress.WhatMagnitudeRequiredData != null &&
                                            innovationProgress.WhatMagnitudeRequiredData.AnswerContent.Contains(criteria))
                                           ||
                                           (innovationProgress.WhatImpactToProductivityData != null &&
                                            innovationProgress.WhatImpactToProductivityData.AnswerContent.Contains(
                                                criteria))
                                           || (innovationProgress.HowIdeaImproveQualityData != null &&
                                               innovationProgress.HowIdeaImproveQualityData.AnswerContent.Contains(
                                                   criteria))
                                           || (innovationProgress.HowStrongEvidenceImpactData != null
                                               &&
                                               innovationProgress.HowStrongEvidenceImpactData.AnswerContent.Contains(
                                                   criteria))
                                           || (innovationProgress.WhatFlexibilityIdeaAffordData != null &&
                                               innovationProgress.WhatFlexibilityIdeaAffordData.AnswerContent.Contains(
                                                   criteria))
                                           || (innovationProgress.WhatMagnitudeIdeaImportstoCRLData != null &&
                                               innovationProgress.WhatMagnitudeIdeaImportstoCRLData.AnswerContent.
                                                   Contains(criteria))
                                           || (innovationProgress.CompetitiveSensitivityWithIdeaData != null &&
                                               innovationProgress.CompetitiveSensitivityWithIdeaData.AnswerContent.
                                                   Contains(criteria))
                                           || (innovationProgress.HowWellDevelopedIdeaData != null
                                               &&
                                               innovationProgress.HowWellDevelopedIdeaData.AnswerContent.Contains(
                                                   criteria))
                                           || (innovationProgress.WhatLevelofSupportData != null
                                               &&
                                               innovationProgress.WhatLevelofSupportData.AnswerContent.Contains(criteria))
                                           || (innovationProgress.WhatLevelofEffortRequiredData != null
                                               &&
                                               innovationProgress.WhatLevelofEffortRequiredData.AnswerContent.Contains(
                                                   criteria))
                                     select innovationProgress.Innovation.ID;
                        #endregion

                        #region Actions required

                        var query4 = from actionsRequired in context.ActionsRequireds
                                     where actionsRequired.Actions.Contains(criteria)
                                        || actionsRequired.Owner.Contains(criteria)
                                     select actionsRequired.Innovation.ID;
                        #endregion

                        #region People support

                        var query5 = from peopleSupport in context.PeopleSupports
                                     where peopleSupport.Name.Contains(criteria)
                                        || peopleSupport.Reason.Contains(criteria)
                                     select peopleSupport.Innovation.ID;
                        #endregion

                        #region Resources Required

                        var query6 = from resourcesRequired in context.ResourcesRequireds
                                     where resourcesRequired.Name.Contains(criteria)
                                        || resourcesRequired.Role.Contains(criteria)
                                        || SqlFunctions.StringConvert(resourcesRequired.Hours).Contains(criteria)
                                        || resourcesRequired.Output.Contains(criteria)
                                     select resourcesRequired.Innovation.ID;
                        #endregion

                        #region link document

                        var query7 = from linkDocument in context.LinkedDocuments
                                     where linkDocument.DocumentDescription.Contains(criteria)
                                     || linkDocument.DocumentURL.Contains(criteria)
                                     select linkDocument.Innovation.ID;
                        #endregion

                        #region populate result
                        var innovationIDs = query.Union(query2).Union(query3).Union(query4).Union(query5).Union(query6).Union(query7).ToList();

                        var resultQuery = from innovation in context.Innovations
                                              .Include(i => i.Theme)
                                              .Include(i => i.SubTheme)
                                              .Include(i => i.CurrentIRL)
                                              .Include(i => i.ProposingItem)
                                          where innovationIDs.Contains(innovation.ID)
                                          select innovation;

                        switch (role.RoleName)
                        {
                            case Constant.Role.InnovationChampion:
                            case Constant.Role.ReadOnly:
                                {
                                    var proposingItems = (from item in context.UserProposingSiteProjects
                                                          where item.UserID == user.ID
                                                          select item.InnovationDataID).ToList();
                                    resultQuery =
                                        resultQuery.Where(
                                            i =>
                                            i.ProposingSiteProjectID.HasValue &&
                                            proposingItems.Contains(i.ProposingSiteProjectID.Value));
                                    break;
                                }
                        }
                        var q = context.InnovationProgresses.Include(c => c.CurrentStage).Select(c => new { InnovationID = c.Innovation.ID, CurrentStage = c.CurrentStage.DataValue, NextSteps = c.NextSteps });
                        var q1 = resultQuery.Select(innovation => new
                        {
                            Innovation = innovation,
                            CurrentStage = q.Where(c => c.InnovationID == innovation.ID).Select(c => c.CurrentStage).FirstOrDefault(),
                            NextSteps = q.Where(c => c.InnovationID == innovation.ID).Select(c => c.NextSteps).FirstOrDefault()
                        });

                        IQueryable<Innovation> tmpResults = null;
                        if (sortColumn == "InnovationTitle")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.InnovationTitle).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.InnovationTitle).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "ProposingItem")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.ProposingItem.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.ProposingItem.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "ThemeName")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.Theme.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.Theme.DataValue).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "CurrentStage")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.CurrentStage).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.CurrentStage).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "NullableSubmittedForCompetitionDate")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.SubmittedForCompetitionDate).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.SubmittedForCompetitionDate).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else if (sortColumn == "NextStepsDisplayString")
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.NextSteps).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.NextSteps).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        else
                        {
                            if (sortDir == "desc")
                            {
                                tmpResults = q1.OrderByDescending(i => i.Innovation.InnovationRef).Select(c => c.Innovation).AsQueryable();
                            }
                            else
                            {
                                tmpResults = q1.OrderBy(i => i.Innovation.InnovationRef).Select(c => c.Innovation).AsQueryable();
                            }
                        }
                        total = tmpResults.Count();
                        tmpResults = tmpResults.Skip((pageNumber - 1) * pageSize).Take(pageSize);

                        foreach (var innovation in tmpResults.ToList())
                        {
                            var innovationProgress =
                                context.InnovationProgresses.Include(x => x.CurrentStage).FirstOrDefault(x => x.Innovation.ID == innovation.ID);
                            if (innovationProgress != null)
                            {
                                innovation.CurrentStage = innovationProgress.CurrentStage;
                                innovation.NextSteps = string.IsNullOrEmpty(innovationProgress.NextSteps)
                                                           ? string.Empty
                                                           : innovationProgress.NextSteps.Trim();
                            }
                            var theme = context.InnovationDatas.FirstOrDefault(c => c.ID == innovation.ThemeID);
                            if (theme != null)
                            {
                                innovation.Theme = theme;
                            }
                            var proposing = context.InnovationDatas.FirstOrDefault(c => c.ID == innovation.ProposingSiteProjectID);
                            if (proposing != null)
                            {
                                innovation.ProposingItem = proposing;
                            }

                            result.Add(innovation);
                        }
                        #endregion
                    }
                }

            }

            return result;
        }
    }
}