﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;

namespace Crossrail.IMS.DataGateway.Repositories
{
	public class SubThemeRepository : ISubThemeRepository
	{

		/// <summary>
		/// The _main context
		/// </summary>
		

        public SubThemeRepository()
        {
            
            
        }


		/// <summary>
		/// Adds the sub theme.
		/// </summary>
		/// <param name="subTheme">The sub theme.</param>
		/// <returns></returns>
        public int AddSubTheme(SubTheme subTheme, string userName)
		{
			try
			{
                using (var context = new MainContext())
                {
                    InnovationData theme = context.InnovationDatas.Find(subTheme.InnovationData.ID);
                    subTheme.InnovationData = theme;
                    context.SubThemes.Add(subTheme);

                    context.SaveChanges(userName);
                    return subTheme.ID;
                }
			}
			catch (Exception ex)
			{
				throw ex;
			}
		}

		/// <summary>
		/// Updates the innovation data.
		/// </summary>
		/// <returns></returns>
        public int UpdateSubTheme(SubTheme subTheme, string userName)
		{
			try
			{
                using (var context = new MainContext())
                {
                    var ent = context.SubThemes.Find(subTheme.ID);
                    if (ent != null)
                    {
                        var theme = context.InnovationDatas.Find(subTheme.InnovationData.ID);
                        context.InnovationDatas.Attach(theme);
                        ent.InnovationData = theme;

                        var attachedEntry = context.Entry(ent);
                        attachedEntry.CurrentValues.SetValues(subTheme);

                        context.Entry(ent).State = EntityState.Modified;
                        return context.SaveChanges(userName);
                    }
                    else
                    {
                        return -1;
                    }
                }
				
			}
			catch (Exception ex)
			{					
				throw ex;
			}
			
			
		}

		/// <summary>
		/// Deletes the innovation data.
		/// </summary>
		/// <returns></returns>
        public int DeleteSubTheme(int subThemeId, string userName)
		{
            using (var context = new MainContext())
            {
                SubTheme subTheme = context.SubThemes.Find(subThemeId);

                if (subTheme != null)
                {
                    context.SubThemes.Remove(subTheme);
                    return context.SaveChanges(userName);
                }

                return 0;
            }
		}

		/// <summary>
		/// Gets the innovation data by category.
		/// </summary>
		/// <returns></returns>
		public List<SubTheme> GetSubThemes()
		{
			try
			{
                using (var context = new MainContext())
                {

                    List<SubTheme> result = context.SubThemes.Include("InnovationData").ToList();

                    return result;
                }
			}
			catch (Exception ex)
			{
				throw ex;
			}
			
		}

		/// <summary>
		/// Finds the sub theme.
		/// </summary>
		/// <param name="subThemeId">The sub theme id.</param>
		/// <returns></returns>
		public SubTheme FindSubTheme(int subThemeId)
		{
            using (var context = new MainContext())
            {
                return context.SubThemes.Find(subThemeId);
            }
			
		}

		/// <summary>
		/// Determines whether [is sub theme name exist] [the specified id].
		/// </summary>
		/// <param name="id">The id.</param>
		/// <param name="dataValue">The data value.</param>
		/// <returns>
		///   <c>true</c> if [is sub theme name exist] [the specified id]; otherwise, <c>false</c>.
		/// </returns>
		public bool IsSubThemeNameExist(int id, string dataValue)
		{
            using (var context = new MainContext())
            {
                if (id == 0)
                {
                    return (context.SubThemes.Any(d => d.DataValue == dataValue));
                }
                else
                {
                    return (context.SubThemes.Any(d => d.DataValue == dataValue && d.ID != id));
                }
            }
		}

        public List<SubTheme> GetSubThemeByThemeID(int themeID)
        {
            using (var context = new MainContext())
            {
                var result = from item in context.SubThemes.Include("InnovationData")
                             where item.InnovationData.ID == themeID
                             select item;

                return result.ToList();
            }
        }
	}
}