﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.Entity;
using System.Data.Entity.ModelConfiguration;
using System.ComponentModel.DataAnnotations.Schema;
using System.Data.Entity.Infrastructure;
using System.Data.Metadata.Edm;
using System.Data.Objects;
using System.Data.SqlClient;

namespace Crossrail.IMS.DataGateway
{
    public class MainContext : DbContext
    {
        /// <summary>
        /// List of User
        /// </summary>
        public DbSet<User> Users { get; set; }

        /// <summary>
        /// List of Role
        /// </summary>
        public DbSet<Role> Roles { get; set; }

        /// <summary>
        /// List of Innovation
        /// </summary>
        public DbSet<Innovation> Innovations { get; set; }

        /// <summary>
        /// List of UserProposingSiteProject
        /// </summary>
        public DbSet<UserProposingSiteProject> UserProposingSiteProjects { get; set; }

        /// <summary>
        /// List of InnovationFinancial
        /// </summary>
        public DbSet<InnovationFinancial> InnovationFinancials { get; set; }

        /// <summary>
        /// List of InnovationData
        /// </summary>
        public DbSet<InnovationData> InnovationDatas { get; set; }

        public DbSet<InnovationProposingSiteProject1> InnovationProposingSiteProject1s { get; set; }

        public DbSet<InnovationProposingSiteProject2> InnovationProposingSiteProject2s { get; set; }

        public DbSet<InnovationTracker> InnovationTrackers { get; set; }

        public DbSet<InnovationRemitImp> InnovationRemitImps { get; set; }

        public DbSet<InnovationRemitAuthProceed> InnovationRemitAuthProceeds { get; set; }

        public DbSet<InnovationRemitResponsible> InnovationRemitResponsibles { get; set; }

        public DbSet<InnovationRemitSuccessFailure> InnovationRemitSuccessFailures { get; set; }

        public DbSet<InnovationRemitSuppoter> InnovationRemitSuppoters { get; set; }

        public DbSet<UserChampionFor> UserChampionFors { get; set; }

        /// <summary>
        /// Gets or sets the sub themes.
        /// </summary>
        /// <value>
        /// The sub themes.
        /// </value>
        public DbSet<SubTheme> SubThemes { get; set; }

        /// <summary>
        /// Gets or sets the innovation progresses.
        /// </summary>
        /// <value>
        /// The innovation progresses.
        /// </value>
        public DbSet<InnovationProgress> InnovationProgresses { get; set; }

        /// <summary>
        /// Gets or sets the actions requireds.
        /// </summary>
        /// <value>
        /// The actions requireds.
        /// </value>
        public DbSet<ActionsRequired> ActionsRequireds { get; set; }

        /// <summary>
        /// Gets or sets the actions required temps.
        /// </summary>
        /// <value>
        /// The actions required temps.
        /// </value>
        public DbSet<ActionsRequiredTemporary> ActionsRequiredTemps { get; set; }

        /// <summary>
        /// Gets or sets the people supports.
        /// </summary>
        /// <value>
        /// The people supports.
        /// </value>
        public DbSet<PeopleSupport> PeopleSupports { get; set; }

        /// <summary>
        /// Gets or sets the people support temps.
        /// </summary>
        /// <value>
        /// The people support temps.
        /// </value>
        public DbSet<PeopleSupportTemporary> PeopleSupportTemps { get; set; }

        /// <summary>
        /// Gets or sets the resources requireds.
        /// </summary>
        /// <value>
        /// The resources requireds.
        /// </value>
        public DbSet<ResourcesRequired> ResourcesRequireds { get; set; }

        /// <summary>
        /// Gets or sets the resources required temps.
        /// </summary>
        /// <value>
        /// The resources required temps.
        /// </value>
        public DbSet<ResourcesRequiredTemporary> ResourcesRequiredTemps { get; set; }

        /// <summary>
        /// Gets or sets the attractiveness complexities.
        /// </summary>
        /// <value>
        /// The attractiveness complexities.
        /// </value>
        public DbSet<AttractivenessComplexity> AttractivenessComplexities { get; set; }

        /// <summary>
        /// Gets or sets the AuditLogs.
        /// </summary>
        /// <value>
        /// The AuditLog.
        /// </value>
        public DbSet<AuditLog> AuditLogs { get; set; }

        public DbSet<LinkedDocument> LinkedDocuments { get; set; }

        public DbSet<LinkedDocumentsTemporary> LinkedDocumentTemps { get; set; }

        public DbSet<InnvationCurrentRefNumber> InnvationCurrentRefNumbers { get; set; }

        /// <summary>
        /// Create mapping class model to tables
        /// </summary>
        /// <param name="m"></param>
        protected override void OnModelCreating(DbModelBuilder m)
        {
            m.Configurations.Add(new UserMap());
            m.Configurations.Add(new RoleMap());
            m.Configurations.Add(new InnovationMap());
            m.Configurations.Add(new InnovationDataMap());
            m.Configurations.Add(new InnovationTrackerMap());
            m.Configurations.Add(new InnovationRemitImpMap());
            m.Configurations.Add(new InnovationRemitAuthProceedMap());
            m.Configurations.Add(new InnovationRemitResponsibleMap());
            m.Configurations.Add(new InnovationRemitSuccessFailureMap());
            m.Configurations.Add(new InnovationRemitSuppoterMap());
            m.Configurations.Add(new SubThemeMap());
            m.Configurations.Add(new UserProposingSiteProjectMap());
            m.Configurations.Add(new UserChampionForMap());
            m.Configurations.Add(new InnovationProgressMap());
            m.Configurations.Add(new ActionsRequiredMap());
            m.Configurations.Add(new ActionsRequiredTempMap());
            m.Configurations.Add(new PeopleSupportMap());
            m.Configurations.Add(new PeopleSupportTempMap());
            m.Configurations.Add(new ResourcesRequiredMap());
            m.Configurations.Add(new ResourcesRequiredTempMap());
            m.Configurations.Add(new AttractivenessComplexityMap());
            m.Configurations.Add(new InnovationFinancialMap());
            m.Configurations.Add(new AuditLogMap());
            m.Configurations.Add(new LinkedDocumentMap());
            m.Configurations.Add(new InnvationCurrentRefNumberMap());
            m.Configurations.Add(new LinkedDocumentTempMap());
            m.Configurations.Add(new InnovationProposingSiteProject1Map());
            m.Configurations.Add(new InnovationProposingSiteProject2Map());

            base.Configuration.LazyLoadingEnabled = false;
            base.OnModelCreating(m);
        }

        /// <summary>
        /// Override the current SaveChanges method
        /// </summary>
        /// <returns></returns>
        public override int SaveChanges()
        {
            //throw new NullReferenceException("You must enter the current UserName - logged-in user");
            return this.SaveChanges("Anonymous");
        }

        /// <summary>
        /// Override the SaveChange method for Audit Trail functionality 
        /// </summary>
        /// <returns></returns>
        public int SaveChanges(string userName)
        {
            //Get all Added/Deleted/Modified entities (not Unmodified or Detached)
            foreach (var entity in this.ChangeTracker.Entries()
                .Where(p => p.State == System.Data.EntityState.Added
                    || p.State == System.Data.EntityState.Deleted
                    || p.State == System.Data.EntityState.Modified))
            {
                // For each changed record, get the audit record entries and add them
                foreach (AuditLog auditLog in GetAuditRecordsForChange(entity, userName))
                {
                    this.AuditLogs.Add(auditLog);
                }
            }

            return base.SaveChanges();
        }

        /// <summary>
        /// Come up with the details of AuditLog record. 
        /// </summary>
        /// <param name="dbEntry">The current Entry</param>
        /// <param name="userName">The Logged-in user</param>
        /// <returns>List of AuditLog</returns>
        private List<AuditLog> GetAuditRecordsForChange(DbEntityEntry dbEntry, string userName)
        {
            List<AuditLog> result = new List<AuditLog>();

            //In case of an event has fired outside the MainContext, we have to manually audit, ignore that case, avoid duplicated records.
            //I.E. login - manually insert 1 record into Audit table
            if (dbEntry.Entity.GetType().Equals(typeof(AuditLog)))
                return result;

            //Event datetime
            DateTime changeTime = DateTime.Now;

            //Get the Table() attribute, if one exists
            TableAttribute tableAttr = dbEntry.Entity.GetType().GetCustomAttributes(typeof(TableAttribute), false).SingleOrDefault() as TableAttribute;

            // Get table name (if it has a Table attribute, use that, otherwise get the pluralized name)
            string tableName = tableAttr != null ? tableAttr.Name : dbEntry.Entity.GetType().Name;
            if (tableName.Equals("AuditLog") || tableName.Equals("PeopleSupportTemporary") || tableName.Equals("ActionsRequiredTemporary")
                || tableName.Equals("ResourcesRequiredTemporary") || tableName.Equals("InnvationCurrentRefNumber"))
                return result;

            //Get primary key value (If you have more than one key column, this will need to be adjusted)
            if (dbEntry.State == System.Data.EntityState.Added)
            {
                //For Inserts, just add the whole record
                //If the entity implements IDescribableEntity, use the description from Describe(), otherwise use ToString()
                result.Add(new AuditLog()
                {
                    UserName = userName,
                    EventDate = changeTime,
                    EventType = "Added",
                    TableName = tableName,
                    ColumnName = "*ALL",
                    RecordID = "New record"
                });
            }
            else
            {
                Type entryEntityType = dbEntry.Entity.GetType();
                string _tableName = GetTableName(entryEntityType);
                string[] _primaryKeyNames = GetPrimaryKeyNames(entryEntityType);
                List<string> RecordIDs = new List<string>();
                foreach (var key in _primaryKeyNames)
                {
                    RecordIDs.Add(dbEntry.OriginalValues.GetValue<object>(key).ToString());
                }

                //Same with deletes, do the whole record, and use either the description from Describe() or ToString()
                if (dbEntry.State == System.Data.EntityState.Deleted)
                {
                    //logical delete Innovation
                    if (dbEntry.Entity.GetType() == typeof(Innovation))
                    {
                        string whereClause = string.Join(" AND ",_primaryKeyNames.Select(key => string.Format("{0} = {1}", key, "@" + key)).ToArray());
                        List<SqlParameter> param = new List<SqlParameter>();
                        foreach (var key in _primaryKeyNames)
                        {
                            param.Add(new SqlParameter("@"+key, dbEntry.OriginalValues[key]));
                        }
                        string deletequery = string.Format("UPDATE {0} SET IsDeleted = 1 WHERE {1}", _tableName, whereClause);
                        Database.ExecuteSqlCommand(deletequery, param.ToArray());
                        dbEntry.State = System.Data.EntityState.Detached;
                    }

                    result.Add(new AuditLog()
                    {
                        UserName = userName,
                        EventDate = changeTime,
                        EventType = "Deleted",
                        TableName = tableName,
                        ColumnName = "*ALL",
                        RecordID = string.Join(",", RecordIDs.ToArray())
                    });
                }
                else if (dbEntry.State == System.Data.EntityState.Modified)
                {
                    foreach (string propertyName in dbEntry.OriginalValues.PropertyNames)
                    {
                        //For updates, we only want to capture the columns that actually changed
                        if (!object.Equals(dbEntry.OriginalValues.GetValue<object>(propertyName), dbEntry.CurrentValues.GetValue<object>(propertyName)))
                        {
                            result.Add(new AuditLog()
                            {
                                UserName = userName,
                                EventDate = changeTime,
                                EventType = "Modified",
                                TableName = tableName,
                                ColumnName = propertyName,
                                RecordID = string.Join(",", RecordIDs.ToArray())
                            });
                        }
                    }
                }
                //Otherwise, don't do anything, we don't care about Unchanged or Detached entities
            }
            return result;
        }

        private static Dictionary<Type, EntitySetBase> _mappingCache = new Dictionary<Type, EntitySetBase>();

        private EntitySetBase GetEntitySet(Type type)
        {
            if (!_mappingCache.ContainsKey(type))
            {
                ObjectContext octx = ((IObjectContextAdapter)this).ObjectContext;

                string typeName = ObjectContext.GetObjectType(type).Name;

                var es = octx.MetadataWorkspace
                                .GetItemCollection(DataSpace.SSpace)
                                .GetItems<EntityContainer>()
                                .SelectMany(c => c.BaseEntitySets
                                                .Where(e => e.Name == typeName))
                                .FirstOrDefault();

                if (es == null)
                    throw new ArgumentException("Entity type not found in GetTableName", typeName);

                _mappingCache.Add(type, es);
            }

            return _mappingCache[type];
        }

        private string GetTableName(Type type)
        {
            EntitySetBase es = GetEntitySet(type);

            return string.Format("[{0}].[{1}]",
                es.MetadataProperties["Schema"].Value,
                es.MetadataProperties["Table"].Value);
        }

        private string[] GetPrimaryKeyNames(Type type)
        {
            EntitySetBase es = GetEntitySet(type);

            return es.ElementType.KeyMembers.Select(c => c.Name).ToArray();
        }
    }

    /// <summary>
    /// Mapping User class to User table 
    /// </summary>
    public class UserMap : EntityTypeConfiguration<User>
    {
        public UserMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();
            Property(p => p.RoleID)
                 .IsRequired();
            Property(p => p.UserName)
                 .IsRequired()
                 .HasMaxLength(20);
            Property(p => p.IsDisabled)
                 .IsRequired();
            Property(p => p.FullName)
                .HasMaxLength(255);
            Property(p => p.Email)
                .HasMaxLength(100);
            Property(p => p.ContactNumber)
                .HasMaxLength(50);
            HasMany(c => c.ChampionFors);
            ToTable("User");
        }
    }

    public class UserChampionForMap : EntityTypeConfiguration<UserChampionFor>
    {
        public UserChampionForMap()
            : base()
        {
            HasKey(c => new { c.UserID, c.ChampionForID });
            Property(c => c.UserID);
            Property(c => c.ChampionForID);
            ToTable("UserChampionFor");
        }
    }

    /// <summary>
    /// Mapping User class to User table 
    /// </summary>
    public class RoleMap : EntityTypeConfiguration<Role>
    {
        public RoleMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();
            Property(p => p.RoleName)
                 .IsRequired()
                 .HasMaxLength(250);

            ToTable("Role");
        }
    }

    /// <summary>
    /// Mapping Innovation class to Innovation table 
    /// </summary>
    public class InnovationMap : EntityTypeConfiguration<Innovation>
    {
        public InnovationMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
            .HasColumnName("ID")
            .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
            .IsRequired();
            Property(p => p.InnovationRef)
                .HasMaxLength(8);
            Property(p => p.InnovationTitle)
                .HasMaxLength(256);
            Property(p => p.Description)
                .HasMaxLength(2000);
            Property(p => p.InnovationContext)
                .HasMaxLength(2000);
            Property(p => p.HowCanWeHelpID);
            Property(p => p.IndividualOrSharedInnovationID);
            Property(p => p.Who)
                .HasMaxLength(1000);
            Property(p => p.DateReceipt);
            Property(p => p.ParentCompany)
                .HasMaxLength(256);
            Property(p => p.ContactFirstName)
                .HasMaxLength(32);
            Property(p => p.ContactSurName)
                .HasMaxLength(32);
            Property(p => p.ContactTelephone)
                .HasMaxLength(128);
            Property(p => p.ContactEmail)
                .HasMaxLength(100);
            Property(p => p.SponsoringChampion)
                .HasMaxLength(256);
            Property(p => p.Area);
            Property(p => p.ProposingSiteProjectID);
            Property(p => p.DetailedDescription)
                .HasMaxLength(2000);
            Property(p => p.Keywords)
                .HasMaxLength(2000);
            Property(p => p.ThemeID);
            Property(p => p.SubThemeID);
            Property(p => p.Benefit)
                .HasMaxLength(2000);
            Property(p => p.BenefitClearlyUnderstoodId);

            Property(p => p.WhoWillBenefit)
                .HasMaxLength(2000);

            Property(p => p.FunctionalSponsor)
                .HasMaxLength(256);
            Property(p => p.SuitablyDevelopedToShareID);
            Property(p => p.CurrentIRLID);
            Property(p => p.TargetIRLID);
            Property(p => p.DesiredOutcome)
                .HasMaxLength(2000);
            Property(p => p.Actions)
                .HasMaxLength(2000);
            Property(p => p.IsSubmitted);
            Property(p => p.SubmittedForCompetitionDate);
            Property(p => p.IsSafety);
            Property(p => p.IsInspiration);
            Property(p => p.IsCollaboration);
            Property(p => p.IsIntegrity);
            Property(p => p.IsRespect);
            Property(p => p.Notes)
                .IsMaxLength();
            Property(p => p.IsTemp);

            Property(p => p.IdeaTypeID);
            Property(p => p.IdeaMaturityID);
            Property(p => p.PrimaryBenefitID);
            Property(p => p.InnovationSourceID);
            Property(p => p.HoursDeveloping);

            Property(p => p.TaskGroupMembers).HasMaxLength(1000);
            Property(p => p.AssignedInnovationTeamMemberID);

            HasMany(c => c.ProposingItems1);
            HasMany(c => c.ProposingItems2);

            Property(c => c.IR_Assumptions).HasMaxLength(2000);
            Property(c => c.IR_DesiredOutcome).HasMaxLength(2000);
            Property(c => c.IR_FundingSought).HasMaxLength(10);
            Property(c => c.IR_InnovativeSol).HasMaxLength(2000);
            Property(c => c.IR_IssueOpportunity).HasMaxLength(1000);
            Property(c => c.IR_PrimaryBenefit).HasMaxLength(2000);

            Map(c => c.Requires("IsDeleted").HasValue(false));

            HasMany(c => c.InnovationRemitImps);
            HasMany(c => c.InnovationRemitAuthProceeds);
            HasMany(c => c.InnovationRemitResponsibles);
            HasMany(c => c.InnovationRemitSuccessFailures);
            HasMany(c => c.InnovationRemitSuppoters);

            ToTable("Innovation");
        }
    }

    public class InnovationProposingSiteProject1Map : EntityTypeConfiguration<InnovationProposingSiteProject1>
    {
        public InnovationProposingSiteProject1Map()
            : base()
        {
            HasKey(c => new { c.InnovationID, c.ProposingID });
            Property(c => c.InnovationID);
            Property(c => c.ProposingID);
            ToTable("InnovationProposingSiteProject1");
        }
    }

    public class InnovationProposingSiteProject2Map : EntityTypeConfiguration<InnovationProposingSiteProject2>
    {
        public InnovationProposingSiteProject2Map()
            : base()
        {
            HasKey(c => new { c.InnovationID, c.ProposingID });
            Property(c => c.InnovationID);
            Property(c => c.ProposingID);
            ToTable("InnovationProposingSiteProject2");
        }
    }

    /// <summary>
    /// Mapping User class to User table 
    /// </summary>
    public class InnovationDataMap : EntityTypeConfiguration<InnovationData>
    {
        public InnovationDataMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();
            Property(p => p.DataValue)
                 .IsRequired()
                 .HasMaxLength(256);
            Property(p => p.Category)
                .IsRequired();

            ToTable("InnovationData");
        }
    }

    public class InnovationRemitBaseMap : EntityTypeConfiguration<InnovationRemitBase>
    {
        public InnovationRemitBaseMap()
            : base()
        {
            HasKey(c => c.ID).Property(c => c.ID).IsRequired();
            Property(c => c.InnovationID).IsRequired();
        }
    }

    public class InnovationRemitImpMap : EntityTypeConfiguration<InnovationRemitImp>
    {
        public InnovationRemitImpMap()
            : base()
        {
            Property(c => c.IR_ImpCost).HasMaxLength(10);
            Property(c => c.IR_ImpDescription).HasMaxLength(256);
            Property(c => c.IR_ImpFinish);
            Property(c => c.IR_ImpStart);
            ToTable("InnovationRemitImp");
        }
    }

    public class InnovationRemitAuthProceedMap : EntityTypeConfiguration<InnovationRemitAuthProceed>
    {
        public InnovationRemitAuthProceedMap()
            : base()
        {
            Property(c => c.IR_AuthProceedDate).HasMaxLength(100);
            Property(c => c.IR_AuthProceedName).HasMaxLength(100);
            Property(c => c.IR_HoldPointDescript).HasMaxLength(256);
            ToTable("InnovationRemitAuthProceed");
        }
    }

    public class InnovationRemitResponsibleMap : EntityTypeConfiguration<InnovationRemitResponsible>
    {
        public InnovationRemitResponsibleMap()
            : base()
        {
            Property(c => c.IR_ResponsibleHours).HasMaxLength(5);
            Property(c => c.IR_ResponsibleName).HasMaxLength(100);
            Property(c => c.IR_ResponsibleRole).HasMaxLength(256);
            ToTable("InnovationRemitResponsible");
        }
    }

    public class InnovationRemitSuccessFailureMap : EntityTypeConfiguration<InnovationRemitSuccessFailure>
    {
        public InnovationRemitSuccessFailureMap()
            : base()
        {
            Property(c => c.IR_HowMeasured).HasMaxLength(256);
            Property(c => c.IR_SuccessFailDescript).HasMaxLength(256);
            Property(c => c.IR_Whenassessed).HasMaxLength(100);
            ToTable("InnovationRemitSuccessFailure");
        }
    }

    public class InnovationRemitSuppoterMap : EntityTypeConfiguration<InnovationRemitSuppoter>
    {
        public InnovationRemitSuppoterMap()
            : base()
        {
            Property(c => c.IR_SupporterName).HasMaxLength(100);
            Property(c => c.IR_SupporterRole).HasMaxLength(100);
            ToTable("InnovationRemitSuppoter");
        }
    }

    /// <summary>
    /// SubThemeMap
    /// </summary>
    public class SubThemeMap : EntityTypeConfiguration<SubTheme>
    {
        public SubThemeMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();
            Property(p => p.DataValue)
                 .IsRequired()
                 .HasMaxLength(256);

            HasRequired(p => p.InnovationData);

            ToTable("SubTheme");
        }
    }

    /// <summary>
    /// Mapping User class to User table 
    /// </summary>
    public class UserProposingSiteProjectMap : EntityTypeConfiguration<UserProposingSiteProject>
    {
        public UserProposingSiteProjectMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();
            HasRequired(u => u.User);
            HasRequired(i => i.InnovationData);

            ToTable("UserProposingSiteProject");
        }
    }

    public class InnovationProgressMap : EntityTypeConfiguration<InnovationProgress>
    {
        public InnovationProgressMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.InnovationRef)
                 .IsRequired()
                 .HasMaxLength(8);

            Property(p => p.LatestUpdate)
                 .HasMaxLength(2000);

            Property(p => p.CurrentStageComplete);

            Property(p => p.IsInnovatorContactDetailsChecked).IsOptional();

            Property(p => p.DatefirstInterviewConducted);

            Property(p => p.DateOfFirstResponse);

            Property(p => p.FollowupDeadline);

            Property(p => p.NextSteps).HasMaxLength(2000);

            Property(p => p.WhyFailedSupport).HasMaxLength(2000);

            Property(p => p.HowToHelp).HasMaxLength(2000);

            Property(p => p.IsNovelIdea).IsOptional();

            Property(p => p.IsUnderstood).IsOptional();

            Property(p => p.IsReadyForDiscoveryEvaluation).IsOptional();

            Property(p => p.WhoBenefits);

            Property(p => p.WhatMagnitudeRequired);

            Property(p => p.WhatImpactToProductivity);

            Property(p => p.HowIdeaImproveQuality);

            Property(p => p.HowStrongEvidenceImpact);

            Property(p => p.WhatFlexibilityIdeaAfford);

            Property(p => p.WhatMagnitudeIdeaImportstoCRL);

            Property(p => p.CompetitiveSensitivityWithIdea);

            Property(p => p.HowWellDevelopedIdea);

            Property(p => p.WhatLevelofSupport);

            Property(p => p.WhatLevelofEffortRequired);

            Property(p => p.LastEdit);

            HasRequired(p => p.Innovation);

            HasOptional(i => i.CurrentStage);

            ToTable("InnovationProgress");
        }
    }

    public class InnovationTrackerMap : EntityTypeConfiguration<InnovationTracker>
    {
        public InnovationTrackerMap()
            : base()
        {
            HasKey(c => c.ID);
            Property(c => c.InnovationRef)
                 .IsRequired()
                 .HasMaxLength(8);
            Property(c => c.IsCurrent).IsRequired();
            Property(c => c.TimeAndDate).IsRequired();
            Property(c => c.Stage);
            Property(c => c.PreviousStage);

            ToTable("InnovationStatusHistory");
        }
    }

    public class InnovationFinancialMap : EntityTypeConfiguration<InnovationFinancial>
    {
        public InnovationFinancialMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.InnovationRef)
                 .IsRequired()
                 .HasMaxLength(8);

            Property(p => p.InnovationRef);
            Property(p => p.ExpenditureToDate);
            Property(p => p.TotalAuthorisedInvestment);
            Property(p => p.CRLAuthorisedInvestment);
            Property(p => p.TSLGFundsSecured);
            Property(p => p.TfLFundsSecured);
            Property(p => p.RIAFundsSecured);
            Property(p => p.BISFundsSecured);
            Property(p => p.FundingComments);
            Property(p => p.IsIncifAuthoritytoProceed).IsOptional();
            Property(p => p.IncifPurchaseOrderNumber);
            Property(p => p.IncifMaximumValueofWork);
            Property(p => p.IncifDelegatedAuthorityName);
            Property(p => p.IncifDelegatedAuthorityTitle);
            Property(p => p.IncifDelegatedAuthorityContactNumber);
            Property(p => p.IncifDelegatedAuthorityDate);
            Property(p => p.ICSCRef);
            Property(p => p.DateofICSC);
            Property(p => p.IsICSCAuthoritytoProceed).IsOptional();
            Property(p => p.IARef);
            Property(p => p.DateofIA);
            Property(p => p.IsIAAuthoritytoProceed).IsOptional();
            Property(p => p.IAMaximumValueofWork);
            Property(p => p.IsExternalAuthoritytoProceed).IsOptional();
            Property(p => p.ExternalRef);
            Property(p => p.ExternalMaximumValueofWork);
            Property(p => p.ExternalDelegatedAuthorityName);
            Property(p => p.ExternalDelegatedAuthorityTitle);
            Property(p => p.ExternalDelegatedAuthorityContactNumber);
            Property(p => p.ExternalDelegatedAuthorityDate);
            Property(p => p.ExternalFundingComments);

            HasRequired(p => p.Innovation);

            ToTable("InnovationFinancial");
        }
    }

    public class ActionsRequiredMap : EntityTypeConfiguration<ActionsRequired>
    {
        public ActionsRequiredMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.Actions).IsRequired().HasMaxLength(256);

            Property(p => p.Owner).IsRequired().HasMaxLength(128);

            Property(p => p.Completed);

            Property(p => p.TemptId);

            HasRequired(p => p.Innovation);

            ToTable("ActionsRequired");
        }
    }

    public class ActionsRequiredTempMap : EntityTypeConfiguration<ActionsRequiredTemporary>
    {
        public ActionsRequiredTempMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.Actions).IsRequired().HasMaxLength(256);

            Property(p => p.Owner).IsRequired().HasMaxLength(128);

            Property(p => p.Completed);

            Property(p => p.OrgID);

            Property(p => p.TemptId);

            //HasRequired(p => p.Innovation);

            ToTable("ActionsRequiredTemporary");
        }
    }

    public class PeopleSupportMap : EntityTypeConfiguration<PeopleSupport>
    {
        public PeopleSupportMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.Name).HasMaxLength(128);

            Property(p => p.Reason).HasMaxLength(500);

            Property(p => p.TemptId);

            Property(p => p.TemptId);

            HasRequired(p => p.Innovation);

            ToTable("PeopleSupport");
        }
    }

    public class PeopleSupportTempMap : EntityTypeConfiguration<PeopleSupportTemporary>
    {
        public PeopleSupportTempMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.Name).HasMaxLength(128);

            Property(p => p.Reason).HasMaxLength(500);

            Property(p => p.OrgID);

            Property(p => p.TemptId);

            //HasRequired(p => p.Innovation);

            ToTable("PeopleSupportTemporary");
        }
    }

    public class ResourcesRequiredMap : EntityTypeConfiguration<ResourcesRequired>
    {
        public ResourcesRequiredMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.Name).IsRequired().HasMaxLength(128);

            Property(p => p.Role).IsRequired().HasMaxLength(128);

            Property(p => p.Hours);

            Property(p => p.Output).HasMaxLength(500);

            Property(p => p.TemptId);

            HasRequired(p => p.Innovation);

            ToTable("ResourcesRequired");
        }
    }

    public class ResourcesRequiredTempMap : EntityTypeConfiguration<ResourcesRequiredTemporary>
    {
        public ResourcesRequiredTempMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.Name).IsRequired().HasMaxLength(128);

            Property(p => p.Role).IsRequired().HasMaxLength(128);

            Property(p => p.Hours);

            Property(p => p.Output).HasMaxLength(500);

            Property(p => p.OrgID);

            Property(p => p.TemptId);

            //HasRequired(p => p.Innovation);

            ToTable("ResourcesRequiredTemporary");
        }
    }

    public class AttractivenessComplexityMap : EntityTypeConfiguration<AttractivenessComplexity>
    {
        public AttractivenessComplexityMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.QuestionId).IsRequired();

            Property(p => p.AnswerContent).IsRequired().HasMaxLength(250);

            Property(p => p.AnswerScore).IsRequired();

            Property(p => p.Colour).HasMaxLength(50);

            ToTable("AttractivenessComplexity");
        }
    }

    /// <summary>
    /// Mapping AuditLog class to AuditLog table 
    /// </summary>
    public class AuditLogMap : EntityTypeConfiguration<AuditLog>
    {
        public AuditLogMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();
            Property(p => p.UserName)
                .HasMaxLength(100)
                .IsRequired();
            Property(p => p.EventType)
                .HasMaxLength(10)
                .IsRequired();
            Property(p => p.EventDate)
                 .IsRequired();
            Property(p => p.TableName)
                .HasMaxLength(100);
            Property(p => p.ColumnName)
                .HasMaxLength(100);

            ToTable("AuditLog");
        }
    }

    public class LinkedDocumentTempMap : EntityTypeConfiguration<LinkedDocumentsTemporary>
    {
        public LinkedDocumentTempMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();
            Property(p => p.DocumentPath)
                .HasMaxLength(500);
            Property(p => p.DocumentURL)
                .HasMaxLength(500);
            Property(p => p.DocumentDescription)
                 .HasMaxLength(500);
            Property(p => p.CreatedDate);
            Property(p => p.OrgID);
            Property(p => p.TemptId);
            ToTable("LinkedDocumentsTemporary");
        }
    }

    public class LinkedDocumentMap : EntityTypeConfiguration<LinkedDocument>
    {
        public LinkedDocumentMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();
            Property(p => p.DocumentPath)
                .HasMaxLength(500);
            Property(p => p.DocumentURL)
                .HasMaxLength(500);
            Property(p => p.DocumentDescription)
                 .HasMaxLength(500);
            Property(p => p.CreatedDate);
            Property(p => p.IsNew);
            Property(p => p.IsDeleted);

            Property(p => p.TemptId);

            HasRequired(p => p.Innovation);

            ToTable("LinkedDocuments");
        }
    }

    public class InnvationCurrentRefNumberMap : EntityTypeConfiguration<InnvationCurrentRefNumber>
    {
        public InnvationCurrentRefNumberMap()
            : base()
        {
            HasKey(p => p.ID);
            Property(p => p.ID)
                .HasColumnName("ID")
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)
                .IsRequired();

            Property(p => p.CurrentRefNumber);

            ToTable("InnvationRefNumber");
        }
    }
}