﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Xml.Serialization;
using System.Xml;
using System.IO;
using System.Configuration;
using System.Text.RegularExpressions;
using System.Globalization;
using System.Net.Mail;
using System.Net;
using log4net;
using System.Net.Mime;

namespace Crossrail.IMS.ImportInnovation.Util
{
    /// <summary>
    /// This class contains some common static method used to check or do somthing.
    /// </summary>
    public class CommonUtils
    {
        private static readonly ILog logEmail = LogManager.GetLogger(typeof(ImportInnovationService).Name);

        #region Common Method
        /// <summary>
        /// Deserialize XML file to object
        /// </summary>
        /// <typeparam name="T">object type</typeparam>
        /// <param name="xmlFileName">XML file name</param>
        /// <param name="xmlRootName">root name of xml file</param>
        /// <returns></returns>
        public static T DeserializeXml2Object<T>(string xmlFileName, string xmlRootName)
        {
            if (string.IsNullOrEmpty(xmlFileName))
            {
                return default(T);
            }
            Stream _stream = null;
            try
            {
                XmlSerializer _xmlSerializer = new XmlSerializer(typeof(T), new XmlRootAttribute(xmlRootName)); //object serializer used to deserialize xml file. 
                _stream = new FileStream(xmlFileName, FileMode.Open, FileAccess.Read); //stream contain data of xml file.
                var _result = (T)_xmlSerializer.Deserialize(_stream); //result after desrializing xml file
                _stream.Close();
                return _result;
            }
            catch (Exception ex)
            {
                if (_stream != null)
                {
                    _stream.Close();
                }
                throw ex;
            }
        }

        /// <summary>
        /// Transmit an email message to a recipient without any attachments
        /// </summary>
        /// <param name="sendTo">Recipient Email Address</param>
        /// <param name="sendCC">Recipient Email Address in CC list</param>
        /// <param name="sendFrom">Sender Email Address</param>
        /// <param name="sendSubject">Subject Line Describing Message</param>
        /// <param name="sendMessage">The Email Message Body</param>
        /// <returns>1: success; -1: failed</returns>
        public static int SendMessage(string sendTo, string sendCC, string sendFrom,
            string sendSubject, string sendMessage)
        {
            try
            {
                // create the email message 
                MailMessage _message = new MailMessage(); //object contains email message.   
                _message.From = new MailAddress(sendFrom);
                _message.Subject = sendSubject;
                _message.Body = sendMessage;

                RegexUtilities _regexUtilities = new RegexUtilities(); //regex utility used to validate email address
                //blnValidate = _regexUtilities.CheckValidEmail(sendTo);                

                string[] strCcMailTo = sendTo.Split(';');
                for (int i = 0; i < strCcMailTo.Length; i++)
                {
                    if (_regexUtilities.CheckValidEmail(strCcMailTo[i]))
                    {
                        _message.To.Add(strCcMailTo[i]);
                    }
                }


                // if the email address is bad, return message
                if (_message.To.Count == 0)
                {
                    return -1;
                }

                if (sendCC.Trim().Length > 0)
                {
                    string[] strCcMaillist = sendCC.Split(';');
                    for (int i = 0; i < strCcMaillist.Length; i++)
                    {
                        if (strCcMaillist[i].Trim() != string.Empty && _regexUtilities.CheckValidEmail(strCcMaillist[i].Trim()))
                        {
                            _message.CC.Add(strCcMaillist[i]);
                        }
                    }
                }

                var userName = GetUserName();
                var password = GetPassword();

                SmtpClient _client = null;
                if (userName.Trim() != string.Empty && password.Trim() != string.Empty)
                {
                    // create smtp client at mail server location
                    _client = new SmtpClient(GetSMTPServer(), int.Parse(GetSMTPPort()))
                    {
                        Credentials = new NetworkCredential(GetUserName(), GetPassword()),
                        EnableSsl = true
                    };
                }
                else
                {
                    _client = new SmtpClient(GetSMTPServer(), int.Parse(GetSMTPPort()))
                    {
                        Credentials = CredentialCache.DefaultNetworkCredentials,
                        DeliveryMethod = SmtpDeliveryMethod.Network
                    };
                }


                //// create smtp client at mail server location
                //var _client = new SmtpClient(GetSMTPServer(), int.Parse(GetSMTPPort()))
                //{
                //    Credentials = new NetworkCredential(GetUserName(), GetPassword()),
                //    EnableSsl = true
                //}; 
                // send message
                _client.Send(_message);

                logEmail.Info("Send mail:" + sendTo + "," + sendCC);
                return 1;
            }
            catch (Exception ex)
            {
                logEmail.Error(ex.InnerException.Message);
                return -1;
            }
        }

        public static int SendMessage(string sendTo, string sendCC, string sendFrom,
            string sendSubject, string sendMessage, bool isBodyHtml)
        {
            try
            {
                // create the email message 
                MailMessage _message = new MailMessage(); //object contains email message.   
                _message.From = new MailAddress(sendFrom);
                _message.Subject = sendSubject;
                _message.Body = sendMessage;
                _message.IsBodyHtml = isBodyHtml;

                RegexUtilities _regexUtilities = new RegexUtilities(); //regex utility used to validate email address
                //blnValidate = _regexUtilities.CheckValidEmail(sendTo);                

                string[] strCcMailTo = sendTo.Split(';');
                for (int i = 0; i < strCcMailTo.Length; i++)
                {
                    if (_regexUtilities.CheckValidEmail(strCcMailTo[i]))
                    {
                        _message.To.Add(strCcMailTo[i]);
                    }
                }


                // if the email address is bad, return message
                if (_message.To.Count == 0)
                {
                    return -1;
                }

                if (sendCC.Trim().Length > 0)
                {
                    string[] strCcMaillist = sendCC.Split(';');
                    for (int i = 0; i < strCcMaillist.Length; i++)
                    {
                        if (strCcMaillist[i].Trim() != string.Empty && _regexUtilities.CheckValidEmail(strCcMaillist[i].Trim()))
                        {
                            _message.CC.Add(strCcMaillist[i]);
                        }
                    }
                }

                var userName = GetUserName();
                var password = GetPassword();

                SmtpClient _client = null;
                if (userName.Trim() != string.Empty && password.Trim() != string.Empty)
                {
                    // create smtp client at mail server location
                    _client = new SmtpClient(GetSMTPServer(), int.Parse(GetSMTPPort()))
                    {
                        Credentials = new NetworkCredential(GetUserName(), GetPassword()),
                        EnableSsl = true
                    };
                }
                else
                {
                    _client = new SmtpClient(GetSMTPServer(), int.Parse(GetSMTPPort()))
                    {
                        Credentials = CredentialCache.DefaultNetworkCredentials,
                        DeliveryMethod = SmtpDeliveryMethod.Network
                    };
                }


                //// create smtp client at mail server location
                //var _client = new SmtpClient(GetSMTPServer(), int.Parse(GetSMTPPort()))
                //{
                //    Credentials = new NetworkCredential(GetUserName(), GetPassword()),
                //    EnableSsl = true
                //}; 
                // send message
                _client.Send(_message);

                logEmail.Info("Send mail:" + sendTo + "," + sendCC);
                return 1;
            }
            catch (Exception ex)
            {
                logEmail.Error(ex.InnerException.Message);
                return -1;
            }
        }


        public static int SendMessage(string sendTo, string sendCC, string sendFrom,
            string sendSubject, string sendMessage, bool isBodyHtml, string fileName)
        {
            var ms = new MemoryStream();
            var reader = new StreamReader(fileName);
            var writer = new StreamWriter(ms);
            try
            {
                // create the email message 
                MailMessage _message = new MailMessage(); //object contains email message.   
                _message.From = new MailAddress(sendFrom);
                _message.Subject = sendSubject;
                _message.Body = sendMessage;
                _message.IsBodyHtml = isBodyHtml;
                writer.Write(reader.ReadToEnd());
                writer.Flush();
                Attachment attachment = new Attachment(ms, Path.GetFileName(fileName));
                _message.Attachments.Add(attachment);

                RegexUtilities _regexUtilities = new RegexUtilities(); //regex utility used to validate email address

                string[] strCcMailTo = sendTo.Split(';');
                for (int i = 0; i < strCcMailTo.Length; i++)
                {
                    if (_regexUtilities.CheckValidEmail(strCcMailTo[i]))
                    {
                        _message.To.Add(strCcMailTo[i]);
                    }
                }


                // if the email address is bad, return message
                if (_message.To.Count == 0)
                {
                    return -1;
                }

                if (sendCC.Trim().Length > 0)
                {
                    string[] strCcMaillist = sendCC.Split(';');
                    for (int i = 0; i < strCcMaillist.Length; i++)
                    {
                        if (strCcMaillist[i].Trim() != string.Empty && _regexUtilities.CheckValidEmail(strCcMaillist[i].Trim()))
                        {
                            _message.CC.Add(strCcMaillist[i]);
                        }
                    }
                }

                var userName = GetUserName();
                var password = GetPassword();

                SmtpClient _client = null;
                if (userName.Trim() != string.Empty && password.Trim() != string.Empty)
                {
                    // create smtp client at mail server location
                    _client = new SmtpClient(GetSMTPServer(), int.Parse(GetSMTPPort()))
                    {
                        Credentials = new NetworkCredential(GetUserName(), GetPassword()),
                        EnableSsl = true
                    };
                }
                else
                {
                    _client = new SmtpClient(GetSMTPServer(), int.Parse(GetSMTPPort()))
                    {
                        Credentials = CredentialCache.DefaultNetworkCredentials,
                        DeliveryMethod = SmtpDeliveryMethod.Network
                    };
                }
                // send message
                _client.Send(_message);
                return 1;
            }
            catch (Exception ex)
            {
                logEmail.Error(ex.InnerException.Message);
                return -1;
            }
            finally
            {
                writer.Dispose();
                reader.Dispose();
                ms.Dispose();
            }
        }

        #endregion

        #region Read data from web config file
        /// <summary>
        /// Get SendFrom value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetSendFrom()
        {
            return ConfigurationManager.AppSettings["SendFrom"];
        }

        /// <summary>
        /// Get SendCC value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetSendCC()
        {
            return ConfigurationManager.AppSettings["SendCC"];
        }

        /// <summary>
        /// Get Server name value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetServerNameRunImport()
        {
            return ConfigurationManager.AppSettings["ServerNameRunImport"];
        }

        /// <summary>
        /// Get Contact Email value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetContactEmail()
        {
            return ConfigurationManager.AppSettings["ContactEmail"];
        }

        /// <summary>
        /// Get Error Email value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetErrorEmail()
        {
            return ConfigurationManager.AppSettings["ErrorEmail"];
        }

        /// <summary>
        /// Get SMTP Server value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetSMTPServer()
        {
            return ConfigurationManager.AppSettings["SMTPServer"];
        }

        /// <summary>
        /// Get SMTP Port value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetSMTPPort()
        {
            return ConfigurationManager.AppSettings["SMTPPort"];
        }

        /// <summary>
        /// Get UserName value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetUserName()
        {
            return ConfigurationManager.AppSettings["UserName"];
        }

        /// <summary>
        /// Get Password value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetPassword()
        {
            return ConfigurationManager.AppSettings["Password"];
        }

        /// <summary>
        /// Get Innovation Folder value from web config. This is share folder contain xml innovation files.
        /// </summary>
        /// <returns></returns>
        public static string GetInnovationFolder()
        {
            return ConfigurationManager.AppSettings["InnovationFolder"];
        }

        /// <summary>
        /// Get Innovation processed Folder value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetInnovationprocessedFolder()
        {
            return ConfigurationManager.AppSettings["InnovationProcessedFolder"];
        }

        /// <summary>
        /// Get InnovationFailedLoads Folder value from web config
        /// </summary>
        /// <returns></returns>
        public static string GetInnovationFailedLoadsFolder()
        {
            return ConfigurationManager.AppSettings["InnovationFailedLoadsFolder"];
        }

        #endregion

        #region Read data from resource file

        /// <summary>
        /// Get Notification Email Subject value from resource file
        /// </summary>
        /// <returns></returns>
        public static string GetNotificationEmailSubject()
        {
            return Crossrail.IMS.ImportInnovation.Resource.NotificationEmailSubject;
        }

        /// <summary>
        /// Get Notification Email Body value from resource file
        /// </summary>
        /// <returns></returns>
        public static string GetNotificationEmailBody()
        {
            return Crossrail.IMS.ImportInnovation.Resource.NotificationEmailBody;
        }

        /// <summary>
        /// Get Alert Email Subject value from resource file
        /// </summary>
        /// <returns></returns>
        public static string GetAlertEmailSubject()
        {
            return Crossrail.IMS.ImportInnovation.Resource.AlertEmailSubject;
        }

        /// <summary>
        /// Get Alert Email Body value from resource file
        /// </summary>
        /// <returns></returns>
        public static string GetAlertEmailBody()
        {
            return Crossrail.IMS.ImportInnovation.Resource.AlertEmailBody;
        }

        /// <summary>
        /// Get Error Message value from resource file
        /// </summary>
        /// <returns></returns>
        public static string GetErrorMessage()
        {
            return Crossrail.IMS.ImportInnovation.Resource.ErrorMessage;
        }

        /// <summary>
        /// Get Import Service Name value from resource file
        /// </summary>
        /// <returns></returns>
        public static string GetImportServiceName()
        {
            return Crossrail.IMS.ImportInnovation.Resource.ImportServiceName;
        }
        #endregion
    }
}