﻿using System;
using System.Globalization;
using System.Text.RegularExpressions;

namespace Crossrail.IMS.ImportInnovation.Util
{
    /// <summary>
    /// This class used to validate some regular expression.
    /// </summary>
    public class RegexUtilities
    {
        private bool blnInvalid = false; //flag is used to check domain name of email address

        /// <summary>
        /// Validate email address
        /// </summary>
        /// <param name="emailAddress">input email</param>
        /// <returns></returns>
        public bool CheckValidEmail(string emailAddress)
        {
            blnInvalid = false;
            if (String.IsNullOrEmpty(emailAddress))
            {
                return false;
            }
            // Use IdnMapping class to convert Unicode domain names. 
            try
            {
                emailAddress = Regex.Replace(emailAddress, @"(@)(.+)$", this.DoMapDomain, RegexOptions.None);
            }
            catch
            {
                return false;
            }
            if (blnInvalid)
            {
                return false;
            }
            // Return true if emailAddress is in valid e-mail format. 
            try
            {
                return Regex.IsMatch(emailAddress,
                      @"^(?("")(""[^""]+?""@)|(([0-9a-z]((\.(?!\.))|[-!#\$%&'\*\+/=\?\^`\{\}\|~\w])*)(?<=[0-9a-z])@))" +
                      @"(?(\[)(\[(\d{1,3}\.){3}\d{1,3}\])|(([0-9a-z][-\w]*[0-9a-z]*\.)+[a-z0-9]{2,17}))$",
                      RegexOptions.IgnoreCase);
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// DomainMapper
        /// </summary>
        /// <param name="match"></param>
        /// <returns></returns>
        private string DoMapDomain(Match match)
        {
            string strDomainResult = string.Empty; //contain domain name result.
            IdnMapping _idnMapping = new IdnMapping(); // IdnMapping class with default property values.
            string strDomainName = match.Groups[2].Value; //domain name
            try
            {
                strDomainName = _idnMapping.GetAscii(strDomainName);
                strDomainResult = match.Groups[1].Value + strDomainName;
            }
            catch (ArgumentException)
            {
                blnInvalid = true;                
            }           
            return strDomainResult;
        }
    }
}