﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data.SqlTypes;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Crossrail.IMS.Common;
using Crossrail.IMS.WebApp.DataGatewayService;
using Crossrail.IMS.WebApp.Models;
using Kendo.Mvc.Extensions;
using Kendo.Mvc.UI;
using Newtonsoft.Json;
using Crossrail.IMS.WebApp.Utilities;
using log4net;
using Crossrail.IMS.WebApp.Filters;
using System.Text.RegularExpressions;

namespace Crossrail.IMS.WebApp.Controllers
{
    public class InnovationDetailsController : BaseController
    {
        #region Property
        private IDataGatewayService _dataGatewayService;

        public IDataGatewayService DataGatewayService
        {
            get
            {
                return _dataGatewayService ?? (_dataGatewayService = new DataGatewayService.DataGatewayServiceClient());
            }
        }

        public string UserNameLog
        {
            get
            {
                return Request.ServerVariables["LOGON_USER"];
            }
        }

        public string Archive(DateTime? entryDate)
        {
            return "You wants Blog Entry on Date:=" + entryDate.ToString();
        }
        #endregion

        #region Action
        /// <summary>
        /// Action for create and edit the innovation
        /// </summary>
        /// <param name="id"></param>
        /// <param name="isFileSizeFailedToUpload"></param>
        /// <returns></returns>
        public ActionResult Index(int? id, bool? isFileSizeFailedToUpload, string selectedTab, string saveSuccess)
        {
            string displayName = string.Empty;
            string userRole = string.Empty;
            User user = null;
            Constant.EnumDetailMode detailMode = Constant.EnumDetailMode.CreateMode;

            if (HttpContext.Session["UserRole"] != null)
            {
                displayName = Session["UserDisplayName"].ToString();
                userRole = Session["UserRole"].ToString();
            }
            else
            {
                return RedirectToAction("NotAuthorized", "UserManagement");
            }

            if ((userRole == Common.Constant.Role.InnovationChampion || userRole == Common.Constant.Role.ReadOnly) && id.HasValue == false)
            {
                return RedirectToAction("NotAuthorized", "UserManagement");
            }

            bool isViewMode = false;
            if (userRole == Common.Constant.Role.ReadOnly)
            {
                isViewMode = true;
            }

            using (var client = new DataGatewayServiceClient())
            {
                user = client.GetUser(int.Parse(Session["UserId"].ToString()));

                var model = new InnovationDetailsModel();
                //Create new innovation
                var innovation = new Innovation()
                {
                    //InnovationRef = client.GetNextInnovationRef(),
                    InnovationTitle = String.Empty,
                    ContactEmail = String.Empty,
                    ContactFirstName = String.Empty,
                    ContactSurName = String.Empty,
                    ContactTelephone = String.Empty,
                    DateReceipt = SqlDateTime.MinValue.Value,
                    SubmittedForCompetitionDate = SqlDateTime.MinValue.Value

                };

                model.IsViewMode = isViewMode;
                model.UserRole = userRole;

                InnovationSummaryModel inovationSummaryModel = new InnovationSummaryModel();
                InnovationProgressModel innovationProgressModel = new InnovationProgressModel();
                InnovationFinancialsModel innovationFinancialsModel = new InnovationFinancialsModel();

                model.TemptId = Guid.NewGuid();
                if (id.HasValue)
                {
                    detailMode = Constant.EnumDetailMode.EditMode;
                    innovation = client.GetOnlineInnovationById(id.Value);
                    //Load existing innovation
                    inovationSummaryModel = InnovationSummaryModel.ConvertToInnovationSummaryModel(innovation);
                    innovationProgressModel = InnovationProgressModel.ConvertToInnovationProgressModel(client.GetInnovationProgress(id.Value));
                    var innovationFinancial = client.GetInnovationFinancialByInnovationId(id.Value);

                    //Handle the error when the innovation is created by import XML file
                    if (innovationFinancial == null)
                    {
                        client.AddInnovationFinancial(new InnovationFinancial()
                            {
                                Innovation = innovation,
                                InnovationRef = innovation.InnovationRef,
                                IncifDelegatedAuthorityDate = SqlDateTime.MinValue.Value,
                                DateofIA = SqlDateTime.MinValue.Value,
                                DateofICSC = SqlDateTime.MinValue.Value,
                                ExternalDelegatedAuthorityDate = SqlDateTime.MinValue.Value
                            }, UserNameLog);
                    }
                    innovationFinancialsModel = InnovationFinancialsModel.ConvertToInnovationFinancialsModel(client.GetInnovationFinancialByInnovationId(id.Value));

                    IEnumerable<InnovationData> proposingItems = user.ProposingItems.Where(p => p.ID == inovationSummaryModel.ProposingSiteProjectID);
                    if (proposingItems.Count() == 0 && (user.RoleName == Common.Constant.Role.ReadOnly || user.RoleName == Common.Constant.Role.InnovationChampion))
                    {
                        return RedirectToAction("NotAuthorized", "UserManagement");
                    }

                }
                else
                {
                    detailMode = Constant.EnumDetailMode.CreateMode;
                }

                model.InnovationModel = inovationSummaryModel;
                model.InnovationProgressModel = innovationProgressModel;
                model.InnovationFinancialsModel = innovationFinancialsModel;

                //Set field level permissions 
                if (model.UserRole != Common.Constant.Role.ReadOnly)
                {
                    inovationSummaryModel.HasWriteOnRelatedFiles = true;
                }
                SetFieldPermissionOnProgressModel(innovationProgressModel, model.UserRole);
                SetFieldPermissionOnFinancialModel(innovationFinancialsModel, model.UserRole);

                //Initialize the related data
                InitializeData(model, id);
                if (innovation != null)
                {
                    if (innovation.ProposingItems1 != null)
                    {
                        foreach (var item in model.InnovationModel.ProposingItems1)
                        {
                            item.IsSelected = innovation.ProposingItems1.Any(c => c.ProposingID == item.ID);
                        }
                    }
                    if (innovation.ProposingItems2 != null)
                    {
                        foreach (var item in model.InnovationModel.ProposingItems2)
                        {
                            item.IsSelected = innovation.ProposingItems2.Any(c => c.ProposingID == item.ID);
                        }
                    }
                }

                model.DetailMode = detailMode;

                if (isFileSizeFailedToUpload.HasValue) model.IsFileSizeFailedToUpload = isFileSizeFailedToUpload.Value;
                if (!String.IsNullOrEmpty(selectedTab)) model.SelectedTab = selectedTab;
                if (!String.IsNullOrEmpty(saveSuccess)) model.SaveSuccess = "1";

                if (id.HasValue)
                {
                    ResetFileUploadStatus(id);
                }

                ViewBag.InnovationRef = inovationSummaryModel.InnovationRef;

                return View("Index", model);
            }

        }

        private void ResetFileUploadStatus(int? id)
        {
            using (var client = new DataGatewayServiceClient())
            {
                client.ResetDocumentStatus(id.Value, UserNameLog);
            }
        }

        /// <summary>
        /// The View Mode
        /// </summary>
        /// <param name="id">The innovation id</param>
        public ActionResult ViewMode(int id)
        {
            using (var client = new DataGatewayServiceClient())
            {
                string userRole = string.Empty;
                if (HttpContext.Session["UserRole"] != null)
                {
                    userRole = Session["UserRole"].ToString();
                }
                else
                {
                    return RedirectToAction("NotAuthorized", "UserManagement");
                }

                var model = new InnovationDetailsModel();
                model.IsViewMode = true;
                model.DetailMode = Constant.EnumDetailMode.ViewMode;
                model.UserRole = userRole;

                //Load existing innovation
                var innovation = client.GetOnlineInnovationById(id);
                model.InnovationModel = InnovationSummaryModel.ConvertToInnovationSummaryModel(innovation);
                model.InnovationProgressModel = InnovationProgressModel.ConvertToInnovationProgressModel(client.GetInnovationProgress(id));
                var innovationFinancial = client.GetInnovationFinancialByInnovationId(id);

                if (innovationFinancial == null)
                {
                    client.AddInnovationFinancial(new InnovationFinancial()
                    {
                        Innovation = innovation,
                        InnovationRef = innovation.InnovationRef,
                        IncifDelegatedAuthorityDate = SqlDateTime.MinValue.Value,
                        DateofIA = SqlDateTime.MinValue.Value,
                        DateofICSC = SqlDateTime.MinValue.Value,
                        ExternalDelegatedAuthorityDate = SqlDateTime.MinValue.Value
                    }, UserNameLog);
                }

                model.InnovationFinancialsModel = InnovationFinancialsModel.ConvertToInnovationFinancialsModel(client.GetInnovationFinancialByInnovationId(id));

                InitializeData(model, id);
                if (innovation != null)
                {
                    if (innovation.ProposingItems1 != null)
                    {
                        foreach (var item in model.InnovationModel.ProposingItems1)
                        {
                            item.IsSelected = innovation.ProposingItems1.Any(c => c.ProposingID == item.ID);
                        }
                    }
                    if (innovation.ProposingItems2 != null)
                    {
                        foreach (var item in model.InnovationModel.ProposingItems2)
                        {
                            item.IsSelected = innovation.ProposingItems2.Any(c => c.ProposingID == item.ID);
                        }
                    }
                }

                ResetFileUploadStatus(id);

                return View("Index", model);
            }
        }

        public JsonResult Delete(int id)
        {
            using (var client = new DataGatewayServiceClient())
            {
                try
                {
                    return Json(client.DeleteOnlineInnovation(id, UserNameLog), JsonRequestBehavior.AllowGet);
                }
                catch
                {
                    return Json(false, JsonRequestBehavior.AllowGet);
                }
            }
        }

        [HttpPost]
        [ValidateInput(false)]
        public ActionResult Save(InnovationDetailsModel model, FormCollection form, string UploadFiles, string remitImp, string remitResponsible, string remitSuccessFailure, string remitAuthProceed, string remitSuppoter)
        {
            using (var client = new DataGatewayServiceClient())
            {
                var innovationId = model.InnovationModel.ID;

                var remitImps = JsonConvert.DeserializeObject<List<InnovationRemitImpModel>>(remitImp);

                #region validate InnovationRemitImpModel
                if (remitImps != null)
                {
                    foreach (var item in remitImps)
                    {
                        if (!string.IsNullOrEmpty(item.IR_ImpDescription) && item.IR_ImpDescription.Length > 256)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for What needs to be done to reach the desired outcome?(Implementation Description)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 256));
                            break;
                        }
                        DateTime _date = DateTime.MinValue;
                        if (!string.IsNullOrEmpty(item.IR_ImpStart) && !DateTime.TryParseExact(item.IR_ImpStart, "dd/MM/yyyy", CultureInfo.InvariantCulture, DateTimeStyles.None, out _date))
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for What needs to be done to reach the desired outcome?(Start). Data value must be valid.");
                            break;
                        }
                        if (!string.IsNullOrEmpty(item.IR_ImpFinish) && !DateTime.TryParseExact(item.IR_ImpFinish, "dd/MM/yyyy", CultureInfo.InvariantCulture, DateTimeStyles.None, out _date))
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for What needs to be done to reach the desired outcome?(Finish). Data value must be valid.");
                            break;
                        }
                        if (!string.IsNullOrEmpty(item.IR_ImpCost))
                        {
                            if (item.IR_ImpCost.Length > 10)
                            {
                                ModelState.AddModelError("InnovationRemit", "There is error validation for What needs to be done to reach the desired outcome?(Cost)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 10));
                                break;
                            }
                            else
                            {
                                Regex reg = new Regex(@"(|\d+|\d{1,8}\.\d{1,2})$");
                                if (!reg.IsMatch(item.IR_ImpCost))
                                {
                                    ModelState.AddModelError("InnovationRemit", "There is error validation for What needs to be done to reach the desired outcome?(Cost)." + Common.Constant.InnovationDetailErrorCode.ErrorMessageCurrency);
                                    break;
                                }
                            }
                        }
                    }

                }
                #endregion

                var remitResponsibles = JsonConvert.DeserializeObject<List<InnovationRemitResponsibleModel>>(remitResponsible);

                #region validate InnovationRemitResponsibleModel
                if (remitImps != null)
                {
                    foreach (var item in remitResponsibles)
                    {
                        if (!string.IsNullOrEmpty(item.IR_ResponsibleName) && item.IR_ResponsibleName.Length > 100)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for Who is responsible for implementing the innovation and what resources are required?(Name)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 100));
                            break;
                        }
                        if (!string.IsNullOrEmpty(item.IR_ResponsibleRole) && item.IR_ResponsibleRole.Length > 256)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for Who is responsible for implementing the innovation and what resources are required?(Role)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 256));
                            break;
                        }
                        if (!string.IsNullOrEmpty(item.IR_ResponsibleHours))
                        {
                            if (item.IR_ResponsibleHours.Length > 5)
                            {
                                ModelState.AddModelError("InnovationRemit", "There is error validation for Who is responsible for implementing the innovation and what resources are required?(Hour)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 5));
                                break;
                            }
                            else
                            {
                                Regex reg = new Regex(@"(|\d+|\d{1,3}\.\d{1,2})$");
                                if (!reg.IsMatch(item.IR_ResponsibleHours))
                                {
                                    ModelState.AddModelError("InnovationRemit", "There is error validation for Who is responsible for implementing the innovation and what resources are required?(Hour).");
                                    break;
                                }
                            }
                        }
                    }

                }
                #endregion

                var remitSuccessFailures = JsonConvert.DeserializeObject<List<InnovationRemitSuccessFailureModel>>(remitSuccessFailure);

                #region validate InnovationRemitSuccessFailureModel
                if (remitImps != null)
                {
                    foreach (var item in remitSuccessFailures)
                    {
                        if (!string.IsNullOrEmpty(item.IR_HowMeasured) && item.IR_HowMeasured.Length > 256)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for How is success and failure to be measured?(How will it be measured)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 256));
                            break;
                        }
                        if (!string.IsNullOrEmpty(item.IR_SuccessFailDescript) && item.IR_SuccessFailDescript.Length > 256)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for How is success and failure to be measured?(Description of success of failure)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 256));
                            break;
                        }
                        if (!string.IsNullOrEmpty(item.IR_Whenassessed) && item.IR_Whenassessed.Length > 100)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for How is success and failure to be measured?(When will it be assessed)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 100));
                            break;
                        }
                    }

                }
                #endregion

                var remitAuthProceeds = JsonConvert.DeserializeObject<List<InnovationRemitAuthProceedModel>>(remitAuthProceed);

                #region validate InnovationRemitAuthProceedModel
                if (remitImps != null)
                {
                    DateTime _date = DateTime.MinValue;
                    foreach (var item in remitAuthProceeds)
                    {
                        if (!string.IsNullOrEmpty(item.IR_AuthProceedDate) && !DateTime.TryParseExact(item.IR_AuthProceedDate, "dd/MM/yyyy", CultureInfo.InvariantCulture, DateTimeStyles.None, out _date))
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for Are there any hold points?(Date). Data value must be valid.");
                            break;
                        }
                        if (!string.IsNullOrEmpty(item.IR_AuthProceedName) && item.IR_AuthProceedName.Length > 100)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for Are there any hold points?(Name)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 100));
                            break;
                        }
                        if (!string.IsNullOrEmpty(item.IR_HoldPointDescript) && item.IR_HoldPointDescript.Length > 256)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for Are there any hold points?(Description of hold point)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 256));
                            break;
                        }
                    }

                }
                #endregion

                var remitSuppoters = JsonConvert.DeserializeObject<List<InnovationRemitSuppoterModel>>(remitSuppoter);

                #region validate InnovationRemitSuppoterModel
                if (remitImps != null)
                {
                    foreach (var item in remitSuppoters)
                    {
                        if (!string.IsNullOrEmpty(item.IR_SupporterName) && item.IR_SupporterName.Length > 100)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for List of those who support this innovation?(Name)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 100));
                            break;
                        }
                        if (!string.IsNullOrEmpty(item.IR_SupporterRole) && item.IR_SupporterRole.Length > 100)
                        {
                            ModelState.AddModelError("InnovationRemit", "There is error validation for List of those who support this innovation?(Role)." + string.Format(Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength, 100));
                            break;
                        }
                    }

                }
                #endregion

                var files = JsonConvert.DeserializeObject<List<LinkedDocumentModel>>(UploadFiles);

                if (ModelState.ContainsKey("InnovationRemit"))
                {
                    Session["InnovationImpsRemitError"] = remitImps;
                    Session["InnovationResponsiblesRemitError"] = remitResponsibles;
                    Session["InnovationSuccessFailuresRemitError"] = remitSuccessFailures;
                    Session["InnovationAuthProceedsRemitError"] = remitAuthProceeds;
                    Session["InnovationSuppotersRemitError"] = remitSuppoters;
                    if (files != null && files.Count > 0)
                    {
                        Session["InnovationRemitErrorUploadFile"] = files;
                    }
                    InitializeData(model, model.InnovationModel.ID);
                    ViewData["InnovationRemit"] = ModelState["InnovationRemit"].Errors.Select(c => c.ErrorMessage).FirstOrDefault();
                    model.SaveSuccess = string.Empty;
                    return View("Index", model);
                }

                var innovation = InnovationSummaryModel.ConvertToInnovationSummaryEntity(model.InnovationModel, model.InnovationProgressModel.CurrentStateId);

                innovation.InnovationRemitImps = InnovationRemitImpModel.ConvertToInnovationRemitImps(remitImps);
                innovation.InnovationRemitResponsibles = InnovationRemitResponsibleModel.ConvertToInnovationRemitResponsibles(remitResponsibles);
                innovation.InnovationRemitSuccessFailures = InnovationRemitSuccessFailureModel.ConvertToInnovationRemitSuccessFailures(remitSuccessFailures);
                innovation.InnovationRemitAuthProceeds = InnovationRemitAuthProceedModel.ConvertToInnovationRemitAuthProceeds(remitAuthProceeds);
                innovation.InnovationRemitSuppoters = InnovationRemitSuppoterModel.ConvertToInnovationRemitSuppoters(remitSuppoters);

                if (model.InnovationModel.ID > 0)
                {
                    client.UpdateOnlineInnovation(innovation, UserNameLog);
                }
                else
                {
                    innovationId = client.AddOnlineInnovation(innovation, UserNameLog);
                }

                Innovation currentInnovation = null;
                currentInnovation = client.GetOnlineInnovationById(innovationId);

                model.InnovationModel.ID = currentInnovation.ID;
                model.InnovationModel.InnovationRef = currentInnovation.InnovationRef;
                model.InnovationProgressModel.Innovation = currentInnovation;
                model.InnovationFinancialsModel.Innovation = currentInnovation;

                SaveInnovationProgress(model, form);

                SaveInnovationFinnancial(model);

                //Process Uploaded files
                var uploadFiles = JsonConvert.DeserializeObject<List<LinkedDocumentModel>>(UploadFiles);
                SaveLinkDocument(currentInnovation.ID, currentInnovation.InnovationRef, model.TemptId, uploadFiles);
            }

            if (form["hdnCloseAfterSave"] != null && form["hdnCloseAfterSave"] == "0")
            {
                var urlHelper = new UrlHelper(Request.RequestContext);
                var baseurl = urlHelper.Content("~");
                if (!String.IsNullOrEmpty(model.SelectedTab))
                {
                    return Redirect(baseurl + "InnovationDetails?id=" + model.InnovationModel.ID.ToString() + "&selectedTab=" + model.SelectedTab
                        + "&saveSuccess=1");
                }
                return Redirect(baseurl + "InnovationDetails?id=" + model.InnovationModel.ID.ToString());
            }

            return RedirectToAction("Index", "InnovationManagement");
        }

        public ActionResult DownloadFile(string fileName, string refNumber)
        {

            if (!String.IsNullOrEmpty(fileName) && !String.IsNullOrEmpty(refNumber))
            {
                var filePath = Path.Combine(ConfigurationManager.AppSettings["RelatedFilesPath"], refNumber, fileName);
                var cd = new System.Net.Mime.ContentDisposition
                {
                    FileName = Path.GetFileName(filePath),
                    Inline = false,
                };

                Response.AppendHeader("Content-Disposition", cd.ToString());
                return File(new FileStream(filePath, FileMode.Open), "application/octetstream");
            }

            return null;
        }

        [HttpPost]
        public void Cancel(int innovationId, string innovationRef)
        {
            using (var client = new DataGatewayServiceClient())
            {
                var innovation = client.GetOnlineInnovationById(innovationId);
                if (innovation != null && innovation.IsTemp)
                {
                    //Clean up temporary data
                    client.CancelinnovationDetail(innovationId, innovationRef, UserNameLog);
                    CommonUtil.DeleteTemporaryFiles(innovationId, innovationRef);
                }
            }
        }

        #endregion

        #region People Support
        public ActionResult CreatePeopleSupport([DataSourceRequest] DataSourceRequest request, PeopleSupportModel data, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                JsonResult result = null;
                if (data != null && ModelState.IsValid)
                {
                    try
                    {
                        data.TemptId = temporaryID;
                        int id = client.AddPeopleSupportTemporary(PeopleSupportModel.ConvertToPeopleSupportEntity(data), UserNameLog);

                        data.ID = id;
                        data.Name = data.Name.Trim();
                        if (data.Reason != null) data.Reason = data.Reason.Trim();

                        result = Json(new[] { data }.ToDataSourceResult(request, ModelState));
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exception", ex.Message);
                        result = Json(ModelState.ToDataSourceResult());
                    }
                }
                return result;
            }
        }
        public ActionResult GetPeopleSupport([DataSourceRequest] DataSourceRequest request, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                var listPeopleSupport =
                    client.GetAllPeopleSupportTemporaryByInnovationId(innovationId, temporaryID)
                                      .Select(
                                          p =>
                                          new PeopleSupportModel()
                                              {
                                                  ID = p.ID,
                                                  Innovation_ID = innovationId,
                                                  Name = p.Name,
                                                  Reason = p.Reason,
                                                  OrgID = p.OrgID
                                              });

                JsonResult result = Json(listPeopleSupport.ToDataSourceResult(request));

                return result;
            }
        }

        public ActionResult UpdatePeopleSupport([DataSourceRequest] DataSourceRequest request, PeopleSupportModel data, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                if (data != null)
                {
                    try
                    {
                        data.Innovation_ID = innovationId;
                        data.TemptId = temporaryID;
                        client.UpdatePeopleSupportTemporary(PeopleSupportModel.ConvertToPeopleSupportEntity(data), UserNameLog);
                        data.Name = data.Name.Trim();
                        if (data.Reason != null) data.Reason = data.Reason.Trim();
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exeption", ex.Message);
                    }
                }

                //return Json(new[] { data }.ToDataSourceResult(request, ModelState));
                return Json(ModelState.ToDataSourceResult());
            }
        }

        public ActionResult DeletePeopleSupport([DataSourceRequest] DataSourceRequest request, PeopleSupportModel data)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                if (data != null)
                {
                    try
                    {
                        DataGatewayService.DeletePeopleSupportTemporary(data.ID, UserNameLog);
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exeption", ex.Message);
                    }
                }
                return Json(ModelState.ToDataSourceResult());
            }
        }

        #endregion

        #region Actions Required

        public ActionResult CreateActionsRequired([DataSourceRequest] DataSourceRequest request, ActionsRequiredModel data, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                JsonResult result = null;

                if (data != null && ModelState.IsValid)
                {
                    try
                    {
                        data.TemptId = temporaryID;
                        int id = client.AddActionsRequiredTemporary(ActionsRequiredModel.ConvertToActionsRequiredEntity(data), UserNameLog);
                        data.ID = id;
                        data.Actions = data.Actions.Trim();
                        if (data.Owner != null) data.Owner = data.Owner.Trim();
                        result = Json(new[] { data }.ToDataSourceResult(request, ModelState));
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exception", ex.Message);
                        result = Json(ModelState.ToDataSourceResult());
                    }
                }

                return result;
            }
        }

        public ActionResult GetActionsRequired([DataSourceRequest] DataSourceRequest request, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                var listActionsRequired =
                    client.GetAllActionsRequiredTemporaryByInnovationId(innovationId, temporaryID)
                                      .Select(
                                          p =>
                                          new ActionsRequiredModel()
                                              {
                                                  ID = p.ID,
                                                  Innovation_ID = innovationId,
                                                  Actions = p.Actions,
                                                  Completed = p.Completed,
                                                  Owner = p.Owner,
                                                  OrgID = p.OrgID
                                              });

                JsonResult result = Json(listActionsRequired.ToDataSourceResult(request));

                return result;
            }
        }

        public ActionResult UpdateActionsRequired([DataSourceRequest] DataSourceRequest request, ActionsRequiredModel data, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                if (data != null)
                {
                    try
                    {
                        data.TemptId = temporaryID;
                        client.UpdateActionsRequiredTemporary(ActionsRequiredModel.ConvertToActionsRequiredEntity(data), UserNameLog);
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exeption", ex.Message);
                    }
                }

                return Json(ModelState.ToDataSourceResult());
            }
        }

        public ActionResult DeleteActionsRequired([DataSourceRequest] DataSourceRequest request, InnovationDetailsModel model, ActionsRequiredModel data)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                if (data != null)
                {
                    try
                    {
                        client.DeleteActionsRequiredTemporary(data.ID, UserNameLog);
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exeption", ex.Message);
                    }
                }
                return Json(ModelState.ToDataSourceResult());
            }
        }

        #endregion

        #region Resources Required
        public ActionResult CreateResourcesRequired([DataSourceRequest] DataSourceRequest request, ResourcesRequiredModel data, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                JsonResult result = null;

                if (data != null && ModelState.IsValid)
                {
                    try
                    {
                        data.TemptId = temporaryID;
                        int id = client.AddResourcesRequiredTemporary(ResourcesRequiredModel.ConvertToResourcesRequiredEntity(data), UserNameLog);
                        data.ID = id;
                        data.Name = data.Name.Trim();
                        data.Role = data.Role.Trim();
                        if (data.Output != null) data.Output = data.Output.Trim();

                        result = Json(new[] { data }.ToDataSourceResult(request, ModelState));
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exception", ex.Message);
                        result = Json(ModelState.ToDataSourceResult());
                    }
                }

                return result;
            }
        }

        public ActionResult GetResourcesRequired([DataSourceRequest] DataSourceRequest request, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                var listActionsRequired =
                    client.GetAllResourcesRequiredTemporaryByInnovationId(innovationId, temporaryID)
                                      .Select(
                                          p =>
                                          new ResourcesRequiredModel()
                                              {
                                                  ID = p.ID,
                                                  Innovation_ID = innovationId,
                                                  Name = p.Name,
                                                  Hours = p.Hours,
                                                  Output = p.Output,
                                                  Role = p.Role
                                              });

                JsonResult result = Json(listActionsRequired.ToDataSourceResult(request));

                return result;
            }
        }

        public ActionResult UpdateResourcesRequired([DataSourceRequest] DataSourceRequest request, ResourcesRequiredModel data, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                if (data != null)
                {
                    try
                    {
                        data.TemptId = temporaryID;
                        client.UpdateResourcesRequiredTemporary(ResourcesRequiredModel.ConvertToResourcesRequiredEntity(data), UserNameLog);
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exeption", ex.Message);
                    }
                }

                return Json(ModelState.ToDataSourceResult());
            }
        }

        public ActionResult DeleteResourcesRequired([DataSourceRequest] DataSourceRequest request, ResourcesRequiredModel data)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                if (data != null)
                {
                    try
                    {
                        client.DeleteResourcesRequiredTemporary(data.ID, UserNameLog);
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exeption", ex.Message);
                    }
                }
                return Json(ModelState.ToDataSourceResult());
            }
        }

        #endregion

        #region Related Files
        [DisableCache]
        public ActionResult GetLinkedDocuments([DataSourceRequest] DataSourceRequest request, int innovationId, Guid temporaryID)
        {
            if (Session["InnovationRemitErrorUploadFile"] != null)
            {
                var listLinkedDoc = (Session["InnovationRemitErrorUploadFile"] as List<LinkedDocumentModel>).ToList();
                Session.Remove("InnovationRemitErrorUploadFile");
                return Json(listLinkedDoc.ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
            using (var client = new DataGatewayServiceClient())
            {
                var listLinkedDoc = client.GetAllLinkedDocumentsTemporaryByInnovationId(innovationId, temporaryID).Select(
                                          p =>
                                          new LinkedDocumentModel()
                                          {
                                              ID = p.ID,
                                              Innovation_ID = innovationId,
                                              DocumentPath = p.DocumentPath,
                                              DocumentURL = p.DocumentURL,
                                              DocumentDescription = p.DocumentDescription
                                              //DownloadLink = !String.IsNullOrEmpty(p.DocumentPath)?  GenerateDownloadLink(p.DocumentPath) : ""
                                          });
                JsonResult result = Json(listLinkedDoc.ToDataSourceResult(request));

                return result;
            }
        }

        public ActionResult CreateLinkedDocument([DataSourceRequest] DataSourceRequest request, LinkedDocumentModel data, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                JsonResult result = null;

                if (data != null && ModelState.IsValid)
                {
                    if ((data.DocumentPath.HasValue() && !String.IsNullOrEmpty(data.DocumentPath.Trim()))
                        || (data.DocumentURL.HasValue() && !String.IsNullOrEmpty(data.DocumentURL.Trim())))
                    {
                        try
                        {
                            data.TemptId = temporaryID;
                            int id = client.AddLinkedDocumentsTemporary(LinkedDocumentModel.ConvertToLinkedDocumentEntity(data), UserNameLog);
                            data.ID = id;
                            if (data.DocumentPath != null) data.DocumentPath = data.DocumentPath.Trim();
                            if (data.DocumentURL != null) data.DocumentURL = data.DocumentURL.Trim();
                            if (data.DocumentDescription != null) data.DocumentDescription = data.DocumentDescription.Trim();
                            result = Json(new[] { data }.ToDataSourceResult(request, ModelState));
                        }
                        catch (Exception ex)
                        {
                            ModelState.AddModelError("Exception", ex.Message);
                            result = Json(ModelState.ToDataSourceResult());
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("DocumentPath", "Please enter document URL or select a file to upload.");
                        result = Json(ModelState.ToDataSourceResult());
                    }
                }

                return result;
            }
        }

        public ActionResult UpdateLinkedDocument([DataSourceRequest] DataSourceRequest request, LinkedDocumentModel data, int innovationId, Guid temporaryID)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                if (data != null)
                {
                    if ((data.DocumentPath.HasValue() && !String.IsNullOrEmpty(data.DocumentPath.Trim()))
                        || (data.DocumentURL.HasValue() && !String.IsNullOrEmpty(data.DocumentURL.Trim())))
                    {
                        try
                        {
                            if (data.DocumentPath != null) data.DocumentPath = data.DocumentPath.Trim();
                            if (data.DocumentURL != null) data.DocumentURL = data.DocumentURL.Trim();
                            if (data.DocumentDescription != null) data.DocumentDescription = data.DocumentDescription.Trim();
                            data.TemptId = temporaryID;
                            client.UpdateLinkedDocumentsTemporary(LinkedDocumentModel.ConvertToLinkedDocumentEntity(data), UserNameLog);
                        }
                        catch (Exception ex)
                        {
                            ModelState.AddModelError("Exeption", ex.Message);
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("DocumentPath", "Please enter document URL or select a file to upload.");
                    }
                }

                return Json(new[] { data }.ToDataSourceResult(request, ModelState));//Json(ModelState.ToDataSourceResult());
            }
        }

        public ActionResult DeleteLinkedDocument([DataSourceRequest] DataSourceRequest request, LinkedDocumentModel data)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                if (data != null)
                {
                    try
                    {
                        data.IsDeleted = true;
                        client.DeleteLinkedDocumentsTemporary(data.ID, UserNameLog);
                        //client.DeleteLinkedDocument(data.ID, UserNameLog);
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("Exeption", ex.Message);
                    }
                }
                return Json(ModelState.ToDataSourceResult());
            }
        }

        private string GenerateDownloadLink(string documentPath)
        {
            var link = Url.Action("DownloadFile", "InnovationDetails", new { @filePath = documentPath });
            return "<a href='#' link='" + link + "' class='download-file' onclick=\"DownloadFile('" + link + "')\">"
                + FileUtilities.GetOriginalFilename(documentPath) +
                   "</a>";
        }

        #region Upload File
        [HttpPost]
        public ActionResult UploadSave(HttpPostedFileBase attachment, string refNumber, bool? isEditing, Guid temptId)
        {
            var attachmentName = Path.GetFileName(attachment.FileName);
            string directory = string.Empty;

            if (!string.IsNullOrEmpty(refNumber))
            {
                directory = Path.Combine(ConfigurationManager.AppSettings["RelatedFilesPath"], refNumber);
            }
            else if (temptId != Guid.Empty)
            {
                directory = Path.Combine(ConfigurationManager.AppSettings["RelatedFilesPath"], temptId.ToString());
            }

            if (!Directory.Exists(directory))
            {
                Directory.CreateDirectory(directory);
            }
            attachment.SaveAs(Path.Combine(directory, attachmentName));

            return Json(new { fileName = attachmentName }, "text/plain");
        }

        [HttpPost]
        public ActionResult UploadRemove(string[] fileNames, string RefNumber, Guid temptId)
        {
            var directory = string.Empty;
            if (!string.IsNullOrEmpty(RefNumber))
            {
                directory = RefNumber;
            }
            else if (temptId != Guid.Empty)
            {
                directory = temptId.ToString();
            }
            // The parameter of the Remove action must be called "fileNames"
            if (fileNames != null)
            {
                foreach (var fullName in fileNames)
                {
                    CommonUtil.DeleteUploadFile(fullName, directory);
                }
            }

            // Return an empty string to signify success
            return Content("");
        }
        #endregion
        #endregion

        #region Private Methods

        private void InitializeData(InnovationDetailsModel model, int? innovationId)
        {
            model.InnovationProgressModel.Stages = GetStages();
            model.InnovationProgressModel.WhoBenefitsAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.WhoBenefits);
            model.InnovationProgressModel.WhatMagnitudeOfTheFundingAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.WhatMagnitudeRequired);
            model.InnovationProgressModel.WhatImpactToProductivityAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.WhatImpactToProductivity);
            model.InnovationProgressModel.HowIdeaImproveQualityAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.HowIdeaImproveQuality);
            model.InnovationProgressModel.HowStrongAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.HowStrongEvidenceImpact);
            model.InnovationProgressModel.WhatFlexibilityIdeaAffordAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.WhatFlexibilityIdeaAfford);
            model.InnovationProgressModel.WhatMagnitudeRiskAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.WhatMagnitudeIdeaImportstoCRL);
            model.InnovationProgressModel.CompetitiveSensitivityWithIdeaAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.CompetitiveSensitivityWithIdea);
            model.InnovationProgressModel.HowWellDevelopedIdeaAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.HowWellDevelopedIdea);
            model.InnovationProgressModel.WhatLevelofSupportAnswers =
                GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.WhatLevelofSupport);
            model.InnovationProgressModel.WhatLevelofEffortRequiredAnswers =
            GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId.WhatLevelofEffortRequired);

            using (var client = new DataGatewayServiceClient())
            {
                //Init data for tempory tables PeopleSupport
                if (innovationId.HasValue)
                {

                    client.CleanupPeopleSupportTemporaryByInnovationId(innovationId.Value);
                    client.FillPeopleSupportTemporaryByInnovationId(innovationId.Value);

                    client.CleanupActionsRequiredTemporaryByInnovationId(innovationId.Value);
                    client.FillActionsRequiredTemporaryByInnovationId(innovationId.Value);

                    client.CleanupResourceRequiredTemporaryByInnovationId(innovationId.Value);
                    client.FillResourceRequiredTemporaryByInnovationId(innovationId.Value);

                    client.CleanupLinkedDocumentsTemporaryByInnovationId(innovationId.Value);
                    client.FillLinkedDocumentsTemporaryByInnovationId(innovationId.Value);

                }

                //Set the data for Dropdown list
                ViewData[Common.Constant.DataManagementCategoryName.Area] = client.GetInnovationDataByCategory(InnovationDataCategory.Area).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.BenefitClearlyUnderstood] = client.GetInnovationDataByCategory(InnovationDataCategory.BenefitClearlyUnderstood).OrderBy(i => i.DataValue);
                var items = client.GetInnovationDataByCategory(InnovationDataCategory.SiteProjectDoctorate).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.Directorate] = items;
                ViewData[Common.Constant.DataManagementCategoryName.HowCanWeHelp] = client.GetInnovationDataByCategory(InnovationDataCategory.HowCanWeHelp).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.IndividualOrSharedInnovation] = client.GetInnovationDataByCategory(InnovationDataCategory.IndividualOrSharedInnovation).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.IRL] = client.GetInnovationDataByCategory(InnovationDataCategory.IRL).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.Stage] = client.GetInnovationDataByCategory(InnovationDataCategory.Stage).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.SuitablyDevelopedToShare] = client.GetInnovationDataByCategory(InnovationDataCategory.SuitablyDevelopedToShare).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.IdeaType] = client.GetInnovationDataByCategory(InnovationDataCategory.IdeaType).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.IdeaMaturity] = client.GetInnovationDataByCategory(InnovationDataCategory.IdeaMaturity).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.PrimaryBenefit] = client.GetInnovationDataByCategory(InnovationDataCategory.PrimaryBenefit).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.InnovationSource] = client.GetInnovationDataByCategory(InnovationDataCategory.InnovationSource).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.AssignedInnovationTeamMember] = client.GetInnovationDataByCategory(InnovationDataCategory.AssignedInnovationTeamMember).OrderBy(i => i.DataValue);

                if (innovationId.HasValue && model.InnovationModel.ThemeID != 0)
                {
                    var result = client.GetSubThemeByThemeID(model.InnovationModel.ThemeID).OrderBy(t => t.DataValue);
                    ViewData[Common.Constant.DataManagementCategoryName.SubTheme] = result;
                }
                else
                {
                    ViewData[Common.Constant.DataManagementCategoryName.SubTheme] = new List<SubTheme>();
                }
                ViewData[Common.Constant.DataManagementCategoryName.SuitablyDevelopedToShare] = client.GetInnovationDataByCategory(InnovationDataCategory.SuitablyDevelopedToShare).OrderBy(i => i.DataValue);
                ViewData[Common.Constant.DataManagementCategoryName.Theme] = client.GetInnovationDataByCategory(InnovationDataCategory.Theme).OrderBy(i => i.DataValue);
                model.InnovationModel.ProposingItems1 = items.Select(c => new ProposingItem() { ID = c.ID, Value = c.DataValue }).ToList();
                model.InnovationModel.ProposingItems2 = items.Select(c => new ProposingItem() { ID = c.ID, Value = c.DataValue }).ToList();
            }
        }

        private List<SelectListItem> GetStages()
        {
            var stages = DataGatewayService.GetInnovationDataByCategory(InnovationDataCategory.Stage).ToList().OrderBy(s => s.DataValue);
            var result = new List<SelectListItem>();

            foreach (var stage in stages)
            {
                var item = new SelectListItem() { Text = stage.DataValue, Value = stage.ID.ToString() };
                result.Add(item);
            }

            return result;
        }

        private List<AttractivenessComplexity> GetAttractivenessComplexityAnswers(AttractivenessComplexityQuestionId questionId)
        {
            return DataGatewayService.GetAttractivenessComplexityByQuestionId(questionId).ToList();
        }

        private void SaveInnovationFinnancial(InnovationDetailsModel model)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                //Save the Innovation Financial
                var innovationFinancial = InnovationFinancialsModel.ConvertToInnovationFinancialEntity(model.InnovationFinancialsModel);
                innovationFinancial.Innovation = model.InnovationFinancialsModel.Innovation;
                innovationFinancial.InnovationRef = model.InnovationFinancialsModel.Innovation.InnovationRef;

                client.UpdateInnovationFinancial(innovationFinancial, UserNameLog);
            }
        }

        private void SaveInnovationProgress(InnovationDetailsModel model, FormCollection form)
        {
            var innovationProgress = InnovationProgressModel.ConvertToInnovationProgressEntity(model.InnovationProgressModel);

            innovationProgress.Innovation = model.InnovationProgressModel.Innovation;
            innovationProgress.InnovationRef = model.InnovationProgressModel.Innovation.InnovationRef;

            //Manual parsing selected Date
            if (!String.IsNullOrEmpty(form["InnovationProgressModel.DatefirstInterviewConducted"]))
            {
                innovationProgress.DatefirstInterviewConducted = CommonUtil.ParseDate(form["InnovationProgressModel.DatefirstInterviewConducted"]);
            }
            else
            {
                innovationProgress.DatefirstInterviewConducted = SqlDateTime.MinValue.Value;
            }

            if (!String.IsNullOrEmpty(form["InnovationProgressModel.DateOfFirstResponse"]))
            {
                innovationProgress.DateOfFirstResponse = CommonUtil.ParseDate(form["InnovationProgressModel.DateOfFirstResponse"]);
            }
            else
            {
                innovationProgress.DateOfFirstResponse = SqlDateTime.MinValue.Value;
            }

            if (!String.IsNullOrEmpty(form["InnovationProgressModel.FollowupDeadline"]))
            {
                innovationProgress.FollowupDeadline = CommonUtil.ParseDate(form["InnovationProgressModel.FollowupDeadline"]);
            }
            else
            {
                innovationProgress.FollowupDeadline = SqlDateTime.MinValue.Value;
            }

            //Get Answers for Discory Evaluation
            GetAnswersForDiscoveryEvaluation(model, innovationProgress, form);

            if (model.InnovationProgressModel.ID > 0)
            {
                DataGatewayService.UpdateInnovationProgress(innovationProgress, UserNameLog);
            }
            else
            {
                DataGatewayService.CreateInnovationProgress(innovationProgress, UserNameLog);
            }
        }

        private void SaveResourceRequired(int innovationId, List<ResourcesRequiredModel> resourcesRequiredModels)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                var requiredTemporaries = new List<ResourcesRequiredTemporary>();
                resourcesRequiredModels.ForEach(m => requiredTemporaries.Add(ResourcesRequiredModel.ConvertToResourcesRequiredEntity(m)));
                client.SaveResourcesRequired(innovationId, UserNameLog, requiredTemporaries.ToArray());
            }
        }

        private void SaveActionRequired(int innovationId, List<ActionsRequiredModel> actionsRequiredModels)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                var actionsRequiredTemporaries = new List<ActionsRequiredTemporary>();
                actionsRequiredModels.ForEach(m => actionsRequiredTemporaries.Add(ActionsRequiredModel.ConvertToActionsRequiredEntity(m)));
                client.SaveActionsRequired(innovationId, UserNameLog, actionsRequiredTemporaries.ToArray());
            }
        }

        private void SavePeopleSupport(int innovationId, List<PeopleSupportModel> peopleSupportModels)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                List<PeopleSupportTemporary> peopleSupportTemporaries = new List<PeopleSupportTemporary>();
                peopleSupportModels.ForEach(m => peopleSupportTemporaries.Add(PeopleSupportModel.ConvertToPeopleSupportEntity(m)));
                client.SavePeopleSupport(innovationId, UserNameLog, peopleSupportTemporaries.ToArray());
            }
        }

        private void SaveLinkDocument(int innovationId, string innovationRef, Guid temptId, List<LinkedDocumentModel> linkedDocumentModels)
        {
            using (var client = new DataGatewayService.DataGatewayServiceClient())
            {
                try
                {
                    if (linkedDocumentModels.Count != 0)
                    {
                        var linkedDocumentsTemporaries = new List<LinkedDocumentsTemporary>();
                        linkedDocumentModels.ForEach(m => linkedDocumentsTemporaries.Add(LinkedDocumentModel.ConvertToLinkedDocumentEntity(m)));
                        client.SaveLinkedDocuments(innovationId, UserNameLog, linkedDocumentsTemporaries.ToArray());

                        //Move file to tempt folder to ref folder
                        CommonUtil.CopyAllUploadFile(temptId.ToString(), innovationRef);
                    }
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
        }

        private void GetAnswersForDiscoveryEvaluation(InnovationDetailsModel model, InnovationProgress innovationProgress, FormCollection form)
        {
            int selectedAnswer = 0;

            if (String.IsNullOrEmpty(form["selectWhoBenefits"]))
            {
                selectedAnswer = model.InnovationProgressModel.WhoBenefits.HasValue
                                     ? model.InnovationProgressModel.WhoBenefits.Value
                                     : 0;
            }
            else
            {
                selectedAnswer = int.Parse(form["selectWhoBenefits"]);
            }
            if (selectedAnswer > 0) innovationProgress.WhoBenefits = selectedAnswer;


            selectedAnswer = (String.IsNullOrEmpty(form["selectFundingRequired"]) && model.InnovationProgressModel.WhatMagnitudeRequired != 0) ?
                model.InnovationProgressModel.WhatMagnitudeRequired :
                String.IsNullOrEmpty(form["selectFundingRequired"]) ? 0 : int.Parse(form["selectFundingRequired"]);
            if (selectedAnswer > 0) innovationProgress.WhatMagnitudeRequired = selectedAnswer;

            selectedAnswer = (String.IsNullOrEmpty(form["selectImpact"]) && model.InnovationProgressModel.WhatImpactToProductivity != 0) ?
                model.InnovationProgressModel.WhatImpactToProductivity :
                String.IsNullOrEmpty(form["selectImpact"]) ? 0 : int.Parse(form["selectImpact"]);
            if (selectedAnswer > 0) innovationProgress.WhatImpactToProductivity = selectedAnswer;

            selectedAnswer = String.IsNullOrEmpty(form["selectQuality"]) && model.InnovationProgressModel.HowIdeaImproveQuality != 0 ?
                model.InnovationProgressModel.HowIdeaImproveQuality :
                String.IsNullOrEmpty(form["selectQuality"]) ? 0 : int.Parse(form["selectQuality"]);
            if (selectedAnswer > 0) innovationProgress.HowIdeaImproveQuality = selectedAnswer;

            selectedAnswer = String.IsNullOrEmpty(form["selectEvidence"]) && model.InnovationProgressModel.HowStrongEvidenceImpact != 0 ?
                model.InnovationProgressModel.HowStrongEvidenceImpact :
                String.IsNullOrEmpty(form["selectEvidence"]) ? 0 : int.Parse(form["selectEvidence"]);
            if (selectedAnswer > 0) innovationProgress.HowStrongEvidenceImpact = selectedAnswer;

            selectedAnswer = String.IsNullOrEmpty(form["selectFlexibility"]) && model.InnovationProgressModel.WhatFlexibilityIdeaAfford != 0 ?
                model.InnovationProgressModel.WhatFlexibilityIdeaAfford :
                String.IsNullOrEmpty(form["selectFlexibility"]) ? 0 : int.Parse(form["selectFlexibility"]);
            if (selectedAnswer > 0) innovationProgress.WhatFlexibilityIdeaAfford = selectedAnswer;

            selectedAnswer = String.IsNullOrEmpty(form["selectRisk"]) && model.InnovationProgressModel.WhatMagnitudeIdeaImportstoCRL != 0 ?
                model.InnovationProgressModel.WhatMagnitudeIdeaImportstoCRL :
                String.IsNullOrEmpty(form["selectRisk"]) ? 0 : int.Parse(form["selectRisk"]);
            if (selectedAnswer > 0) innovationProgress.WhatMagnitudeIdeaImportstoCRL = selectedAnswer;

            selectedAnswer = String.IsNullOrEmpty(form["selectCompetitive"]) && model.InnovationProgressModel.CompetitiveSensitivityWithIdea != 0 ?
                model.InnovationProgressModel.CompetitiveSensitivityWithIdea :
                String.IsNullOrEmpty(form["selectCompetitive"]) ? 0 : int.Parse(form["selectCompetitive"]);
            if (selectedAnswer > 0) innovationProgress.CompetitiveSensitivityWithIdea = selectedAnswer;

            selectedAnswer = String.IsNullOrEmpty(form["selectWellDeveloped"]) && model.InnovationProgressModel.HowWellDevelopedIdea != 0 ?
                model.InnovationProgressModel.HowWellDevelopedIdea :
                String.IsNullOrEmpty(form["selectWellDeveloped"]) ? 0 : int.Parse(form["selectWellDeveloped"]);
            if (selectedAnswer > 0) innovationProgress.HowWellDevelopedIdea = selectedAnswer;

            selectedAnswer = String.IsNullOrEmpty(form["selectLevelSupport"]) && model.InnovationProgressModel.WhatLevelofSupport != 0 ?
                model.InnovationProgressModel.WhatLevelofSupport :
                String.IsNullOrEmpty(form["selectLevelSupport"]) ? 0 : int.Parse(form["selectLevelSupport"]);
            if (selectedAnswer > 0) innovationProgress.WhatLevelofSupport = selectedAnswer;

            selectedAnswer = String.IsNullOrEmpty(form["selectLevelEffort"]) && model.InnovationProgressModel.WhatLevelofEffortRequired != 0 ?
                model.InnovationProgressModel.WhatLevelofEffortRequired :
                String.IsNullOrEmpty(form["selectLevelEffort"]) ? 0 : int.Parse(form["selectLevelEffort"]);
            if (selectedAnswer > 0) innovationProgress.WhatLevelofEffortRequired = selectedAnswer;

        }

        private void SetFieldPermissionOnProgressModel(InnovationProgressModel innovationProgressModel, string userRole)
        {
            innovationProgressModel.HasWriteOnAttractiveness = "disabled";
            innovationProgressModel.HasWriteOnComplexity = "disabled";

            switch (userRole)
            {
                case Common.Constant.Role.InnovationChampion:
                    innovationProgressModel.HasWriteOnLatestUpateField = true;
                    innovationProgressModel.HasWriteOnCurrentStage = true;
                    innovationProgressModel.HasWriteOnCurrentStageCompleted = true;
                    innovationProgressModel.HasWriteOnInnovatorContactDetailsChecked = true;
                    innovationProgressModel.HasWriteOnDateFirstInterview = true;
                    innovationProgressModel.HasWriteOnDateFirstResponse = true;
                    innovationProgressModel.HasWriteOnFollowupDeadline = true;
                    innovationProgressModel.HasWriteOnNextSteps = true;
                    innovationProgressModel.HasWriteOnWhyIdealFailed = true;
                    innovationProgressModel.HasWriteOnPeopleSupport = true;
                    innovationProgressModel.HasWriteOnHowInnovationTeamIsToHelp = true;
                    innovationProgressModel.HasWriteOnActionsRequired = true;
                    innovationProgressModel.HasWriteOnNovelIdea = true;
                    innovationProgressModel.HasWriteOnIdeaUnderstood = true;
                    innovationProgressModel.HasWriteOnReadyDiscoveryEvaluation = true;
                    innovationProgressModel.HasWriteOnResourcesRequired = true;
                    break;
                case Common.Constant.Role.InnovationTeam:
                case Common.Constant.Role.SystemAdministrator:
                    innovationProgressModel.HasWriteOnLatestUpateField = true;
                    innovationProgressModel.HasWriteOnCurrentStage = true;
                    innovationProgressModel.HasWriteOnCurrentStageCompleted = true;
                    innovationProgressModel.HasWriteOnInnovatorContactDetailsChecked = true;
                    innovationProgressModel.HasWriteOnDateFirstInterview = true;
                    innovationProgressModel.HasWriteOnDateFirstResponse = true;
                    innovationProgressModel.HasWriteOnFollowupDeadline = true;
                    innovationProgressModel.HasWriteOnNextSteps = true;
                    innovationProgressModel.HasWriteOnWhyIdealFailed = true;
                    innovationProgressModel.HasWriteOnPeopleSupport = true;
                    innovationProgressModel.HasWriteOnHowInnovationTeamIsToHelp = true;
                    innovationProgressModel.HasWriteOnActionsRequired = true;
                    innovationProgressModel.HasWriteOnNovelIdea = true;
                    innovationProgressModel.HasWriteOnIdeaUnderstood = true;
                    innovationProgressModel.HasWriteOnReadyDiscoveryEvaluation = true;
                    innovationProgressModel.HasWriteOnResourcesRequired = true;
                    innovationProgressModel.HasWriteOnAttractiveness = string.Empty;
                    innovationProgressModel.HasWriteOnComplexity = string.Empty;
                    break;
            }
        }

        private void SetFieldPermissionOnFinancialModel(InnovationFinancialsModel innovationFinancialModel, string userRole)
        {
            innovationFinancialModel.HasWriteOnIncifAuthoritytoProceed = "disabled";
            innovationFinancialModel.HasWriteOnICSCAuthoritytoproceed = "disabled";
            innovationFinancialModel.HasWriteOnIAAuthoritytoProceed = "disabled";
            innovationFinancialModel.HasWriteOnExternalAuthoritytoProceed = "disabled";
            switch (userRole)
            {
                case Common.Constant.Role.ReadOnly:
                case Common.Constant.Role.InnovationChampion:
                    break;
                case Common.Constant.Role.InnovationTeam:
                case Common.Constant.Role.SystemAdministrator:
                    innovationFinancialModel.HasWriteOnExpenditureToDate = true;
                    innovationFinancialModel.HasWriteOnTotalAuthorisedInvestment = true;
                    innovationFinancialModel.HasWriteOnCRLAuthorisedInvestment = true;
                    innovationFinancialModel.HasWriteOnTSLGFundsSecured = true;
                    innovationFinancialModel.HasWriteOnTfLFundsSecured = true;
                    innovationFinancialModel.HasWriteOnRIAFundsSecured = true;
                    innovationFinancialModel.HasWriteOnBISFundsSecured = true;
                    innovationFinancialModel.HasWriteOnFundingComments = true;
                    innovationFinancialModel.HasWriteOnIncifAuthoritytoProceed = "";
                    innovationFinancialModel.HasWriteOnIncifPurchaseOrderNumber = true;
                    innovationFinancialModel.HasWriteOnIncifMaximumValueofWork = true;
                    innovationFinancialModel.HasWriteOnIncifDelegatedAuthorityName = true;
                    innovationFinancialModel.HasWriteOnIncifDelegatedAuthorityTitle = true;
                    innovationFinancialModel.HasWriteOnIncifDelegatedAuthorityContactNumber = true;
                    innovationFinancialModel.HasWriteOnIncifDelegatedAuthorityDate = true;
                    innovationFinancialModel.HasWriteOnICSCRef = true;
                    innovationFinancialModel.HasWriteOnDateofICSC = true;
                    innovationFinancialModel.HasWriteOnICSCAuthoritytoproceed = "";
                    innovationFinancialModel.HasWriteOnIARef = true;
                    innovationFinancialModel.HasWriteOnDateofIA = true;
                    innovationFinancialModel.HasWriteOnIAAuthoritytoProceed = "";
                    innovationFinancialModel.HasWriteOnIAMaximumValueofWork = true;
                    innovationFinancialModel.HasWriteOnExternalAuthoritytoProceed = "";
                    innovationFinancialModel.HasWriteOnExternalRef = true;
                    innovationFinancialModel.HasWriteOnExternalMaximumValueofWork = true;
                    innovationFinancialModel.HasWriteOnExternalDelegatedAuthorityName = true;
                    innovationFinancialModel.HasWriteOnExternalDelegatedAuthorityTitle = true;
                    innovationFinancialModel.HasWriteOnExternalDelegatedAuthorityContactNumber = true;
                    innovationFinancialModel.HasWriteOnExternalDelegatedAuthorityDate = true;
                    innovationFinancialModel.HasWriteOnExternalFundingComments = true;
                    break;

            }
        }
        #endregion

        #region GetSubThemeByTheme
        /// <summary>
        /// Get sub theme based on selected theme
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public ActionResult GetSubThemeByThemeId(int id)
        {
            using (var client = new DataGatewayServiceClient())
            {
                var result = client.GetSubThemeByThemeID(id).OrderBy(t => t.DataValue);
                var itemContent = "<option value='{0}' style='width: 120px;' title='{1}'>-{1}</option>";

                System.Text.StringBuilder html = new System.Text.StringBuilder();
                html.AppendLine(string.Format(itemContent, "0", "--Select--"));
                foreach (var item in result)
                {
                    html.AppendLine(string.Format(itemContent, item.ID, item.DataValue));
                }


                return new JsonResult() { Data = html.ToString() };
            }

        }
        #endregion

        #region Remit
        [DisableCache]
        public ActionResult InnovationRemitImp_Read([DataSourceRequest] DataSourceRequest request, int innovationId)
        {
            if (Session["InnovationImpsRemitError"] != null)
            {
                var list = (Session["InnovationImpsRemitError"] as List<InnovationRemitImpModel>).ToList();
                Session.Remove("InnovationImpsRemitError");
                return Json(list.ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
            using (var client = new DataGatewayServiceClient())
            {
                return Json(InnovationRemitImpModel.ConvertToInnovationRemitImpModels(client.GetInnovationRemitImps(innovationId)).ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitImp_Create([DataSourceRequest] DataSourceRequest request, InnovationRemitImpModel item)
        {
            ModelState.Clear();
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitImp_Update([DataSourceRequest] DataSourceRequest request, InnovationRemitImpModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitImp_Destroy([DataSourceRequest] DataSourceRequest request, InnovationRemitImpModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        public ActionResult InnovationRemitResponsible_Read([DataSourceRequest] DataSourceRequest request, int innovationId)
        {
            if (Session["InnovationResponsiblesRemitError"] != null)
            {
                var list = (Session["InnovationResponsiblesRemitError"] as List<InnovationRemitResponsibleModel>).ToList();
                Session.Remove("InnovationResponsiblesRemitError");
                return Json(list.ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
            using (var client = new DataGatewayServiceClient())
            {
                return Json(InnovationRemitResponsibleModel.ConvertToInnovationRemitResponsibleModels(client.GetInnovationRemitResponsibles(innovationId)).ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitResponsible_Create([DataSourceRequest] DataSourceRequest request, InnovationRemitResponsibleModel item)
        {
            ModelState.Clear();
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitResponsible_Update([DataSourceRequest] DataSourceRequest request, InnovationRemitResponsibleModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitResponsible_Destroy([DataSourceRequest] DataSourceRequest request, InnovationRemitResponsibleModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        public ActionResult InnovationRemitSuccessFailure_Read([DataSourceRequest] DataSourceRequest request, int innovationId)
        {
            if (Session["InnovationSuccessFailuresRemitError"] != null)
            {
                var list = (Session["InnovationSuccessFailuresRemitError"] as List<InnovationRemitSuccessFailureModel>).ToList();
                Session.Remove("InnovationSuccessFailuresRemitError");
                return Json(list.ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
            using (var client = new DataGatewayServiceClient())
            {
                return Json(InnovationRemitSuccessFailureModel.ConvertToInnovationRemitSuccessFailureModels(client.GetInnovationRemitSuccessFailures(innovationId)).ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitSuccessFailure_Create([DataSourceRequest] DataSourceRequest request, InnovationRemitSuccessFailureModel item)
        {
            ModelState.Clear();
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitSuccessFailure_Update([DataSourceRequest] DataSourceRequest request, InnovationRemitSuccessFailureModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitSuccessFailure_Destroy([DataSourceRequest] DataSourceRequest request, InnovationRemitSuccessFailureModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        public ActionResult InnovationRemitAuthProceed_Read([DataSourceRequest] DataSourceRequest request, int innovationId)
        {
            if (Session["InnovationAuthProceedsRemitError"] != null)
            {
                var list = (Session["InnovationAuthProceedsRemitError"] as List<InnovationRemitAuthProceedModel>).ToList();
                Session.Remove("InnovationAuthProceedsRemitError");
                return Json(list.ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
            using (var client = new DataGatewayServiceClient())
            {
                return Json(InnovationRemitAuthProceedModel.ConvertToInnovationRemitAuthProceedModels(client.GetInnovationRemitAuthProceeds(innovationId)).ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitAuthProceed_Create([DataSourceRequest] DataSourceRequest request, InnovationRemitAuthProceedModel item)
        {
            ModelState.Clear();
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitAuthProceed_Update([DataSourceRequest] DataSourceRequest request, InnovationRemitAuthProceedModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitAuthProceed_Destroy([DataSourceRequest] DataSourceRequest request, InnovationRemitAuthProceedModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        public ActionResult InnovationRemitSuppoter_Read([DataSourceRequest] DataSourceRequest request, int innovationId)
        {
            if (Session["InnovationSuppotersRemitError"] != null)
            {
                var list = (Session["InnovationSuppotersRemitError"] as List<InnovationRemitSuppoterModel>).ToList();
                Session.Remove("InnovationSuppotersRemitError");
                return Json(list.ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
            using (var client = new DataGatewayServiceClient())
            {
                return Json(InnovationRemitSuppoterModel.ConvertToInnovationRemitSuppoterModels(client.GetInnovationRemitSuppoters(innovationId)).ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
            }
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitSuppoter_Create([DataSourceRequest] DataSourceRequest request, InnovationRemitSuppoterModel item)
        {
            ModelState.Clear();
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitSuppoter_Update([DataSourceRequest] DataSourceRequest request, InnovationRemitSuppoterModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        [DisableCache]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InnovationRemitv_Destroy([DataSourceRequest] DataSourceRequest request, InnovationRemitSuppoterModel item)
        {
            return Json(new[] { item }.ToDataSourceResult(request, ModelState), JsonRequestBehavior.AllowGet);
        }

        #endregion
    }
}
