﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Crossrail.IMS.Common;
using Crossrail.IMS.WebApp.DataGatewayService;
using Crossrail.IMS.WebApp.Models;
using Kendo.Mvc.UI;
using Kendo.Mvc.Extensions;
using Crossrail.IMS.WebApp.Filters;

namespace Crossrail.IMS.WebApp.Controllers
{
    public class InnovationManagementController : BaseController
    {
        public ActionResult Index()
        {
            return RedirectToAction("InnovationList");
        }

        public ActionResult InnovationList()
        {
            return View();
        }

        public ActionResult CreateInnovation()
        {
            return View();
        }

        [DisableCache]
        public ActionResult GetAllInnovationsForAjaxCallback([DataSourceRequest] DataSourceRequest request)
        {
            int total = 0;
            string sortCol = null;
            string sortDir = null;
            if (request.Sorts != null && request.Sorts.Count > 0)
            {
                sortCol = request.Sorts.Select(c => c.Member).FirstOrDefault();
                var dir = request.Sorts.Select(c => c.SortDirection).FirstOrDefault();
                sortDir = dir == System.ComponentModel.ListSortDirection.Ascending ? "asc" : "desc";
            }
            var datas = GetAllInnovations(request.Page, request.PageSize, sortCol, sortDir, out total);
            var result = datas.ToDataSourceResult(request);
            result.Data = datas;
            result.Total = total;
            return Json(result);
        }

        public IEnumerable<InnovationModel> GetAllInnovations(int pageNumber, int pageSize, string sortColumn, string sortDir, out int total)
        {
            var client = new DataGatewayServiceClient();
            var innovations = client.GetAllInnovations(out total, User.Identity.Name, pageNumber, pageSize, sortColumn, sortDir);

            return innovations.ToModelList();
        }

        public ActionResult EditInnovation(int innovationID)
        {
            return null;
        }

        [HttpGet]
        public ActionResult SearchInnovation()
        {
            var model = new SearchInnovationModel();
            model.ProposingItems = GetInnovationData(InnovationDataCategory.SiteProjectDoctorate);
            model.Themes = GetInnovationData(InnovationDataCategory.Theme);
            model.SubThemes = new List<SelectListItem>();
            model.CurrentIRLs = GetInnovationData(InnovationDataCategory.IRL);
            model.CurrentStages = GetInnovationData(InnovationDataCategory.Stage);
            model.SearchMode = SearchInnovationMode.Restricted;

            return View(model);
        }

        public ActionResult GetSearchInnovationResultForAjaxBinding([DataSourceRequest] DataSourceRequest request, SearchInnovationModel model)
        {
            var criteria = new SearchInnovationCriteria();
            criteria.InnovationRef = string.IsNullOrEmpty(model.InnovationReference)
                                         ? string.Empty
                                         : model.InnovationReference.Trim();
            criteria.InnovationTitle = string.IsNullOrEmpty(model.InnovationTitle)
                                           ? string.Empty
                                           : model.InnovationTitle.Trim();
            criteria.ProposingItemID = model.ProposingItem;
            criteria.ThemeID = model.Theme;
            criteria.SubThemeID = model.SubTheme;
            criteria.CurrentIRLID = model.CurrentIRL;
            criteria.CurrentStageID = model.CurrentStage;
            criteria.CompetitionDateFrom = model.SubmittedForCompetitionDateFrom;
            criteria.CompetitionDateTo = model.SubmittedForCompetitionDateTo;
            criteria.SearchMode = model.SearchMode;
            criteria.SearchText = string.IsNullOrEmpty(model.SearchText)
                                      ? string.Empty
                                      : model.SearchText.Trim();
            int total = 0;
            string sortCol = null;
            string sortDir = null;
            if (request.Sorts != null && request.Sorts.Count > 0)
            {
                sortCol = request.Sorts.Select(c => c.Member).FirstOrDefault();
                var dir = request.Sorts.Select(c => c.SortDirection).FirstOrDefault();
                sortDir = dir == System.ComponentModel.ListSortDirection.Ascending ? "asc" : "desc";
            }
            var client = new DataGatewayServiceClient();
            var result = client.SearchInnovation(out total, criteria, User.Identity.Name, request.Page, request.PageSize, sortCol, sortDir).ToModelList();
            var datas = result.ToDataSourceResult(request);
            datas.Data = result;
            datas.Total = total;
            return Json(datas);
        }

        [HttpPost]
        public ActionResult GetSubThemesByThemeID(int themeID)
        {
            var client = new DataGatewayServiceClient();
            var result = client.GetSubThemeByThemeID(themeID).OrderBy(x => x.DataValue);

            return new JsonResult() { Data = result };
        }

        public ActionResult InnovationNextSteps(int innovationID)
        {
            var client = new DataGatewayServiceClient();
            var innovationProgress = client.GetInnovationProgress(innovationID);
            if (innovationProgress != null)
            {
                var model = new InnovationNextStepsModel();
                model.InnovationRef = innovationProgress.Innovation.InnovationRef;
                model.InnovationTitle = innovationProgress.Innovation.InnovationTitle;
                model.NextSteps = innovationProgress.NextSteps;

                return View(model);
            }

            return null;
        }

        #region Private Methods
        private List<SelectListItem> GetInnovationData(InnovationDataCategory category)
        {
            var client = new DataGatewayServiceClient();
            return client.GetInnovationDataByCategory(category).OrderBy(x => x.DataValue).ToMvcSelectList();
        }

        private List<SelectListItem> GetSubThemes()
        {
            var client = new DataGatewayServiceClient();
            return client.GetSubThemes().ToMvcSelectList();
        }
        #endregion
    }
}
