﻿using System;
using System.Collections.Generic;
using System.Data.SqlTypes;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Crossrail.IMS.Common;
using Crossrail.IMS.WebApp.DataGatewayService;
using Crossrail.IMS.WebApp.Models;
using ICSharpCode.SharpZipLib.Zip;
using Infragistics.Documents.Excel;
namespace Crossrail.IMS.WebApp.Controllers
{
    public class ReportController : BaseController
    {
        public IDataGatewayService DataService
        {
            get
            {
                return new DataGatewayService.DataGatewayServiceClient();
            }
            set { }
        }
        public ActionResult Index()
        {
            var model = new ReportInnovationModel();
            var periods = DataService.GetInnovationPeriods(string.Empty);
            model.Periods = (from d in periods
                             select new KeyValuePair<string, string>(d.Code, d.Name)).ToList();
            return View(model);
        }
        [OutputCache(Duration = 0, VaryByParam = "none")]
        public FileResult ExportTheReport(ReportInnovationModel model)
        {
            if (!model.IsInnovationExtract && !model.IsNewSubmission && model.IsStageSubmission)
            {
                var mStream = ExportStageInnovation(model.PeriodFrom, model.PeriodTo, new List<RangePeriod>().ToArray());
                return File(mStream, "application/vnd.ms-excel", "Stage of Innovations by period.xls");
            }
            else if (!model.IsInnovationExtract && model.IsNewSubmission && !model.IsStageSubmission)
            {
                var mStream = ExportNewSubmissionInnovation(model.PeriodFrom, model.PeriodTo, new List<RangePeriod>().ToArray());
                return File(mStream, "application/vnd.ms-excel", "New submission by period.xls");
            }
            else if (model.IsInnovationExtract && !model.IsNewSubmission && !model.IsStageSubmission)
            {
                var mSteam = ExportExtractInnovation(model.PeriodFrom, model.PeriodTo, new List<RangePeriod>().ToArray());
                return File(mSteam, "application/vnd.ms-excel", "Innovation Extract.xls");
            }
            else
            {
                var newSubmission = model.IsNewSubmission != false
                                        ? ExportNewSubmissionInnovation(model.PeriodFrom, model.PeriodTo,
                                                                    new List<RangePeriod>().ToArray())
                                        : null;
                var stageInnovation = model.IsStageSubmission != false
                                        ? ExportStageInnovation(model.PeriodFrom, model.PeriodTo,
                                                                    new List<RangePeriod>().ToArray())
                                        : null;
                var exstractInnovation = model.IsInnovationExtract != false
                                        ? ExportExtractInnovation(model.PeriodFrom, model.PeriodTo,
                                                                    new List<RangePeriod>().ToArray())
                                        : null;



                List<NamingReport> zips = new List<NamingReport>();
                if (newSubmission != null)
                {
                    zips.Add(new NamingReport
                    {
                        Stream = newSubmission,
                        FileName = "New submission by period",
                    });
                }
                if (stageInnovation != null)
                {
                    zips.Add(new NamingReport
                    {
                        Stream = stageInnovation,
                        FileName = "Stage of Innovations by period",
                    });
                }
                if (exstractInnovation != null)
                {
                    zips.Add(new NamingReport
                    {
                        Stream = exstractInnovation,
                        FileName = "Innovation Extract",
                    });
                }

                FileStreamResult file = File(ZipFilesFromStreams(zips), "zip - application/x-zip-compressed", "TheReport.zip");
                HttpContext.Response.Buffer = true;
                HttpContext.Response.Clear();
                HttpContext.Response.ClearContent();
                HttpContext.Response.ClearHeaders();
                HttpContext.Response.ContentType = "zip - application/x-zip-compressed";
                HttpContext.Response.AddHeader("content-disposition", "attachment; filename=TheReport.zip");

                return file;

            }
            return null;
        }
        public byte[] ReadToEnd(System.IO.Stream stream)
        {
            long originalPosition = 0;

            if (stream.CanSeek)
            {
                originalPosition = stream.Position;
                stream.Position = 0;
            }

            try
            {
                byte[] readBuffer = new byte[4096];

                int totalBytesRead = 0;
                int bytesRead;

                while ((bytesRead = stream.Read(readBuffer, totalBytesRead, readBuffer.Length - totalBytesRead)) > 0)
                {
                    totalBytesRead += bytesRead;

                    if (totalBytesRead == readBuffer.Length)
                    {
                        int nextByte = stream.ReadByte();
                        if (nextByte != -1)
                        {
                            byte[] temp = new byte[readBuffer.Length * 2];
                            Buffer.BlockCopy(readBuffer, 0, temp, 0, readBuffer.Length);
                            Buffer.SetByte(temp, totalBytesRead, (byte)nextByte);
                            readBuffer = temp;
                            totalBytesRead++;
                        }
                    }
                }

                byte[] buffer = readBuffer;
                if (readBuffer.Length != totalBytesRead)
                {
                    buffer = new byte[totalBytesRead];
                    Buffer.BlockCopy(readBuffer, 0, buffer, 0, totalBytesRead);
                }
                return buffer;
            }
            finally
            {
                if (stream.CanSeek)
                {
                    stream.Position = originalPosition;
                }
            }
        }
        private Stream ZipFilesFromStreams(List<NamingReport> lstExportedFile)
        {
            var outputMemStream = new MemoryStream();
            var oZipStream = GetZipStreamObject(outputMemStream);
            int fileCount = 0;
            foreach (var exportedFile in lstExportedFile)
            {
                var temp = new MemoryStream(ToByte(exportedFile.Stream));
                CreateZipEntryByStream(oZipStream, temp, exportedFile.FileName + ".xls");
            }
            oZipStream.IsStreamOwner = false;
            oZipStream.Close();
            outputMemStream.Position = 0;
            return outputMemStream;
        }
        private void CreateZipEntryByStream(ZipOutputStream zipStream, MemoryStream stream, string fileName)
        {
            var zipEntry = new ZipEntry(fileName) { DateTime = DateTime.Now, Size = stream.Length }; ZipEntry.CleanName(zipEntry.Name);
            zipEntry.Size = stream.Length;
            var crc = new ICSharpCode.SharpZipLib.Checksums.Crc32();
            crc.Reset();
            crc.Update(stream.ToArray());
            zipEntry.Crc = crc.Value;
            zipStream.PutNextEntry(zipEntry);
            zipStream.Write(stream.ToArray(), 0, (int)stream.Length);
            zipStream.CloseEntry();
        }
        private ZipOutputStream GetZipStreamObject(Stream outputMemStream)
        {
            var zipStream = new ZipOutputStream(outputMemStream) //Create a memory Zip
            {
                IsStreamOwner = false// False stops the Close also Closing the underlying stream.
            };
            zipStream.SetLevel(0); //0-9, 9 being the highest level of compression
            zipStream.Password = null; //optional. Nothing is the same as not setting.
            zipStream.UseZip64 = UseZip64.Off; //Make zipfile WinXP-compability. Max 4GB.
            return zipStream;
        }
        private int GetAttractivenessScore(string innovationRef)
        {
            try
            {
                var innovationProgress = DataService.GetInnovationProgressByInRef(innovationRef);

                var whoBenefits = DataService.GetAnswersById(innovationProgress.WhoBenefits);
                var whatMagnitudeRequired = DataService.GetAnswersById(innovationProgress.WhatMagnitudeRequired);
                var whatImpactToProductivity = DataService.GetAnswersById(innovationProgress.WhatImpactToProductivity);
                var howIdeaImproveQuality = DataService.GetAnswersById(innovationProgress.HowIdeaImproveQuality);
                var howStrongEvidenceImpact = DataService.GetAnswersById(innovationProgress.HowStrongEvidenceImpact);
                var whatFlexibilityIdeaAfford = DataService.GetAnswersById(innovationProgress.WhatFlexibilityIdeaAfford);
                var whatMagnitudeIdeaImportstoCRL = DataService.GetAnswersById(innovationProgress.WhatMagnitudeIdeaImportstoCRL);

                var result = 0;
                if (whoBenefits != null) result = result + whoBenefits.AnswerScore;
                if (whatMagnitudeRequired != null) result = result + whatMagnitudeRequired.AnswerScore;
                if (whatMagnitudeIdeaImportstoCRL != null) result = result + whatMagnitudeIdeaImportstoCRL.AnswerScore;
                if (whatFlexibilityIdeaAfford != null) result = result + whatFlexibilityIdeaAfford.AnswerScore;
                if (whatImpactToProductivity != null) result = result + whatImpactToProductivity.AnswerScore;
                if (howIdeaImproveQuality != null) result = result + howIdeaImproveQuality.AnswerScore;
                if (howStrongEvidenceImpact != null) result = result + howStrongEvidenceImpact.AnswerScore;

                return result;
            }

            catch (Exception)
            {

                return 0;
            }

        }
        private int GetComplexityScore(string innovationRef)
        {
            try
            {
                var innovationProgress = DataService.GetInnovationProgressByInRef(innovationRef);
                var competitiveSensitivityWithIdea = DataService.GetAnswersById(innovationProgress.CompetitiveSensitivityWithIdea);
                var howWellDevelopedIdea = DataService.GetAnswersById(innovationProgress.HowWellDevelopedIdea);
                var whatLevelofSupport = DataService.GetAnswersById(innovationProgress.WhatLevelofSupport);
                var whatLevelofEffortRequired = DataService.GetAnswersById(innovationProgress.WhatLevelofEffortRequired);

                var result = 0;
                if (competitiveSensitivityWithIdea != null) result = result + competitiveSensitivityWithIdea.AnswerScore;
                if (howWellDevelopedIdea != null) result = result + howWellDevelopedIdea.AnswerScore;
                if (whatLevelofSupport != null) result = result + whatLevelofSupport.AnswerScore;
                if (whatLevelofEffortRequired != null) result = result + whatLevelofEffortRequired.AnswerScore;
                return result;
            }
            catch (Exception)
            {

                return 0;
            }

        }
        public MemoryStream ExportNewSubmissionInnovation(string periodFrom, string periodTo, IList<RangePeriod> periodsss)
        {
            List<InnovationPeriod> periodsList;
            using (var DataService1 = new DataGatewayService.DataGatewayServiceClient())
            {
                periodsList = DataService1.GetInnovationPeriods(string.Empty).ToList();
            }
            periodsList.RemoveAt(0);
            var startPeriod = periodFrom != null ? periodsList.Where(x => x.Code == periodFrom).FirstOrDefault() : null;
            var endPeriod = periodTo != null ? periodsList.Where(x => x.Code == periodTo).FirstOrDefault() : null;
            var periods = new List<InnovationPeriod>();

            if (endPeriod != null)
            {
                periods.AddRange(periodsList.Where(x => int.Parse(x.Code) >= int.Parse(startPeriod.Code) && int.Parse(x.Code) <= int.Parse(endPeriod.Code)).ToList());
            }
            else
            {
                periods.Add(startPeriod);
            }

            DataNewSubmissionReport[] datas;
            using (var DataService1 = new DataGatewayService.DataGatewayServiceClient())
            {
                datas = DataService1.exportNewSubmissionReport(periods.ToArray());
            }

            Workbook workbook = new Workbook(WorkbookFormat.Excel97To2003);
            Worksheet sheet = workbook.Worksheets.Add("NewSubmissionInnovation");

            Infragistics.Documents.Excel.IWorkbookFont normalFont = workbook.Styles.NormalStyle.StyleFormat.Font;
            normalFont.Name = "Arial";
            normalFont.Height = 11 * 20;
            sheet.Rows[0].Cells[0].Value = "Periods: " + startPeriod.Name + (endPeriod != null ? string.Format(" to {0}", endPeriod.Name) : string.Empty);
            var colNo = 2;
            var themesNo = 0;
            if (datas.Count() > 0)
            {
                var test = datas.FirstOrDefault().Innovations.Where(p => p.ID >= 576 && p.ID <= 589 && p.ThemeID != null);
                //where id >= 576 and id <= 589 and themeid is not null
                colNo += datas.FirstOrDefault().Themes.Count() + datas.FirstOrDefault().SubThemeList.Count();
                themesNo = datas.FirstOrDefault().Themes.Count();
                var subThemesNo = datas.FirstOrDefault().SubThemeList.Count();
                for (int k = 0; k < colNo + 2; k++)
                {
                    sheet.Columns[k].Width = 32 * 256;
                }
                if (themesNo > 0)
                {
                    Infragistics.Documents.Excel.WorksheetMergedCellsRegion mergedRegion1 =
                        sheet.MergedCellsRegions.Add(2, 2, 2, 1 + themesNo);
                    mergedRegion1.CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);//Color.FromArgb(217, 217, 217);
                    mergedRegion1.CellFormat.TopBorderColor = Color.Black;
                    mergedRegion1.Value = "Submissions by Theme";
                }
                else
                {
                    sheet.Rows[2].Cells[2].Value = "Submissions by Theme";
                    sheet.Rows[2].Cells[2].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[2].Cells[2].CellFormat.BottomBorderColor =
                      sheet.Rows[2].Cells[2].CellFormat.LeftBorderColor =
                      sheet.Rows[2].Cells[2].CellFormat.RightBorderColor =
                      sheet.Rows[2].Cells[2].CellFormat.TopBorderColor = Color.Black;
                    sheet.Rows[3].Cells[2].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[3].Cells[2].CellFormat.BottomBorderColor =
                      sheet.Rows[3].Cells[2].CellFormat.LeftBorderColor =
                      sheet.Rows[3].Cells[2].CellFormat.RightBorderColor =
                      sheet.Rows[3].Cells[2].CellFormat.TopBorderColor = Color.Black;
                }
                if (subThemesNo > 0 && themesNo > 0)
                {
                    sheet.Rows[2].Cells[2 + themesNo].Value = "Submissions by Sub Theme";
                    Infragistics.Documents.Excel.WorksheetMergedCellsRegion mergedRegion2 =
                        sheet.MergedCellsRegions.Add(2, 2 + themesNo, 2,
                                                     2 + themesNo + datas.FirstOrDefault().SubThemeList.Count() - 1);
                    mergedRegion2.CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    mergedRegion2.CellFormat.BottomBorderColor =
                         mergedRegion2.CellFormat.LeftBorderColor =
                         mergedRegion2.CellFormat.RightBorderColor =
                        mergedRegion2.CellFormat.TopBorderColor = Color.Black;
                    sheet.Rows[2].Cells[3].CellFormat.Alignment =
                        Infragistics.Documents.Excel.HorizontalCellAlignment.Center;
                    sheet.Rows[2].Cells[4].CellFormat.Alignment =
                        Infragistics.Documents.Excel.HorizontalCellAlignment.Center;
                    mergedRegion2.Value = "Submissions by Sub Theme";
                }
                else
                {
                    sheet.Rows[2].Cells[3].Value = "Submissions by Sub Theme";
                    sheet.Rows[2].Cells[3].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[2].Cells[3].CellFormat.BottomBorderColor =
                        sheet.Rows[2].Cells[3].CellFormat.LeftBorderColor =
                        sheet.Rows[2].Cells[3].CellFormat.RightBorderColor =
                        sheet.Rows[2].Cells[3].CellFormat.TopBorderColor = Color.Black;
                    sheet.Rows[3].Cells[3].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[3].Cells[3].CellFormat.BottomBorderColor =
                        sheet.Rows[3].Cells[3].CellFormat.LeftBorderColor =
                        sheet.Rows[3].Cells[3].CellFormat.RightBorderColor =
                        sheet.Rows[3].Cells[3].CellFormat.TopBorderColor = Color.Black;
                }
                sheet.Rows[2].Cells[0].Value = "Period";
                sheet.Rows[2].Cells[0].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                sheet.Rows[2].Cells[0].CellFormat.BottomBorderColor =
                       sheet.Rows[2].Cells[0].CellFormat.LeftBorderColor =
                       sheet.Rows[2].Cells[0].CellFormat.RightBorderColor =
                       sheet.Rows[2].Cells[0].CellFormat.TopBorderColor = Color.Black;
                sheet.Rows[2].Cells[1].Value = "Total";
                sheet.Rows[2].Cells[1].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                sheet.Rows[2].Cells[1].CellFormat.BottomBorderColor =
                       sheet.Rows[2].Cells[1].CellFormat.LeftBorderColor =
                       sheet.Rows[2].Cells[1].CellFormat.RightBorderColor =
                       sheet.Rows[2].Cells[1].CellFormat.TopBorderColor = Color.Black;
                sheet.Rows[3].Cells[0].Value = string.Empty;
                sheet.Rows[3].Cells[0].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                sheet.Rows[3].Cells[0].CellFormat.BottomBorderColor =
                       sheet.Rows[3].Cells[0].CellFormat.LeftBorderColor =
                       sheet.Rows[3].Cells[0].CellFormat.RightBorderColor =
                       sheet.Rows[3].Cells[0].CellFormat.TopBorderColor = Color.Black;
                sheet.Rows[3].Cells[1].Value = "Total Submissions";
                sheet.Rows[3].Cells[1].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                sheet.Rows[3].Cells[1].CellFormat.BottomBorderColor =
                       sheet.Rows[3].Cells[1].CellFormat.LeftBorderColor =
                       sheet.Rows[3].Cells[1].CellFormat.RightBorderColor =
                       sheet.Rows[3].Cells[1].CellFormat.TopBorderColor = Color.Black;
                for (int l = 1; l <= datas.FirstOrDefault().Themes.Count(); l++)
                {
                    sheet.Rows[3].Cells[1 + l].Value = datas.FirstOrDefault().Themes[l - 1].DataValue;
                    sheet.Rows[3].Cells[1 + l].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[3].Cells[1 + l].CellFormat.BottomBorderColor =
                        sheet.Rows[3].Cells[1 + l].CellFormat.LeftBorderColor =
                        sheet.Rows[3].Cells[1 + l].CellFormat.RightBorderColor =
                        sheet.Rows[3].Cells[1 + l].CellFormat.TopBorderColor = Color.Black;
                }
                if (datas.FirstOrDefault().SubThemeList.Count() > 0)
                {

                    int n = 0;
                    while (n < subThemesNo)
                    {

                        var tempItem = string.Empty;
                        for (int l = 0; l < datas.FirstOrDefault().SubThemeList.Count(); l++)
                        {
                            tempItem = datas.FirstOrDefault().SubThemeList[l].InnovationData.DataValue +
                                       " - " + datas.FirstOrDefault().SubThemeList[l].DataValue;
                            sheet.Rows[3].Cells[2 + themesNo + n].Value = tempItem;
                            sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                            sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.BottomBorderColor =
                                sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.LeftBorderColor =
                                sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.RightBorderColor =
                                sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.TopBorderColor = Color.Black;
                            n++;
                        }

                    }
                }

                var step = 1;
                foreach (var data in datas)
                {
                    sheet.Rows[3 + step].Cells[0].Value = data.Period;
                    sheet.Rows[3 + step].Cells[0].CellFormat.BottomBorderColor =
                                      sheet.Rows[3 + step].Cells[0].CellFormat.LeftBorderColor =
                                       sheet.Rows[3 + step].Cells[0].CellFormat.RightBorderColor =
                                      sheet.Rows[3 + step].Cells[0].CellFormat.TopBorderColor = Color.Black;

                    sheet.Rows[3 + step].Cells[1].Value = data.Total;
                    sheet.Rows[3 + step].Cells[1].CellFormat.BottomBorderColor =
                                       sheet.Rows[3 + step].Cells[1].CellFormat.LeftBorderColor =
                                       sheet.Rows[3 + step].Cells[1].CellFormat.RightBorderColor =
                                       sheet.Rows[3 + step].Cells[1].CellFormat.TopBorderColor = Color.Black;

                    for (int k = 0; k < data.Themes.Count(); k++)
                    {
                        sheet.Rows[3 + step].Cells[1 + 1 + k].Value = data.Innovations.Where(x => x.ThemeID == data.Themes[k].ID).Count();
                        sheet.Rows[3 + step].Cells[1 + 1 + k].CellFormat.BottomBorderColor =
                       sheet.Rows[3 + step].Cells[1 + 1 + k].CellFormat.LeftBorderColor =
                      sheet.Rows[3 + step].Cells[1 + 1 + k].CellFormat.RightBorderColor =
                       sheet.Rows[3 + step].Cells[1 + 1 + k].CellFormat.TopBorderColor = Color.Black;
                    }
                    if (datas.FirstOrDefault().SubThemeList.Count() > 0)
                    {
                        foreach (var dt in datas)
                        {
                            var n = 0;
                            while (n < subThemesNo)
                            {

                                for (int l = 0; l < dt.SubThemeList.Count(); l++)
                                {
                                    int tempItem = data.Innovations.Where(x => x.SubThemeID == dt.SubThemeList[l].ID
                                                                            && x.ThemeID == datas.FirstOrDefault().SubThemeList[l].InnovationData.ID).Count();
                                    sheet.Rows[3 + step].Cells[2 + themesNo + n].Value = tempItem;
                                    sheet.Rows[3 + step].Cells[2 + themesNo + n].CellFormat.BottomBorderColor =
                                        sheet.Rows[3 + step].Cells[2 + themesNo + n].CellFormat.LeftBorderColor =
                                        sheet.Rows[3 + step].Cells[2 + themesNo + n].CellFormat.RightBorderColor =
                                        sheet.Rows[3 + step].Cells[2 + themesNo + n].CellFormat.TopBorderColor = Color.Black;
                                    n++;
                                }

                            }
                        }
                    }
                    step++;
                }
            }
            var result = Save(workbook, "NewSubmissionInnovation.xls");
            return result;
        }
        public MemoryStream ExportStageInnovation(string periodFrom, string periodTo, IList<RangePeriod> periodsss)
        {
            var periodsList = DataService.GetInnovationPeriods(string.Empty).ToList();
            periodsList.RemoveAt(0);
            var startPeriod = periodFrom != null ? periodsList.Where(x => x.Code == periodFrom).FirstOrDefault() : null;
            var periods = new List<InnovationPeriod>();


            periods.Add(startPeriod);


            var datas = DataService.exportStageSubmissionReport(periods.ToArray());

            Workbook workbook = new Workbook(WorkbookFormat.Excel97To2003);
            Worksheet sheet = workbook.Worksheets.Add("StageInnovation");
            Infragistics.Documents.Excel.IWorkbookFont normalFont = workbook.Styles.NormalStyle.StyleFormat.Font;
            normalFont.Name = "Arial";
            normalFont.Height = 11 * 20;
            sheet.Rows[0].Cells[0].Value = "Selected Period: " + startPeriod.Name;
            var colNo = 2;
            var themesNo = 0;
            if (datas.Count() > 0)
            {

                colNo += datas.FirstOrDefault().Themes.Count() +
                         datas.FirstOrDefault().SubThemeList.Count();
                themesNo = datas.FirstOrDefault().Themes.Count();
                var subThemesNo = datas.FirstOrDefault().SubThemeList.Count();
                for (int k = 0; k < colNo + 2; k++)
                {
                    sheet.Columns[k].Width = 32 * 256;
                }
                if (themesNo > 0)
                {
                    Infragistics.Documents.Excel.WorksheetMergedCellsRegion mergedRegion1 =
                        sheet.MergedCellsRegions.Add(2, 2, 2, 1 + themesNo);
                    mergedRegion1.CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    mergedRegion1.Value = "Innovations by Theme";

                    mergedRegion1.CellFormat.BottomBorderColor =
                      mergedRegion1.CellFormat.LeftBorderColor =
                      mergedRegion1.CellFormat.RightBorderColor =
                      mergedRegion1.CellFormat.TopBorderColor = Color.Black;
                }
                else
                {
                    sheet.Rows[2].Cells[2].Value = "Innovations by Theme";
                    sheet.Rows[2].Cells[2].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[2].Cells[2].CellFormat.BottomBorderColor =
                      sheet.Rows[2].Cells[2].CellFormat.LeftBorderColor =
                      sheet.Rows[2].Cells[2].CellFormat.RightBorderColor =
                      sheet.Rows[2].Cells[2].CellFormat.TopBorderColor = Color.Black;
                    sheet.Rows[3].Cells[2].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[3].Cells[2].CellFormat.BottomBorderColor =
                    sheet.Rows[3].Cells[2].CellFormat.LeftBorderColor =
                    sheet.Rows[3].Cells[2].CellFormat.RightBorderColor =
                    sheet.Rows[3].Cells[2].CellFormat.TopBorderColor = Color.Black;
                }
                if (subThemesNo > 0 && themesNo > 0)
                {
                    sheet.Rows[2].Cells[2 + themesNo].Value = "Innovations by Sub Theme";
                    Infragistics.Documents.Excel.WorksheetMergedCellsRegion mergedRegion2 =
                        sheet.MergedCellsRegions.Add(2, 2 + themesNo, 2, 2 + themesNo + datas.FirstOrDefault().SubThemeList.Count());
                    mergedRegion2.CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    mergedRegion2.CellFormat.BottomBorderColor =
                      mergedRegion2.CellFormat.LeftBorderColor =
                      mergedRegion2.CellFormat.RightBorderColor =
                      mergedRegion2.CellFormat.TopBorderColor = Color.Black;
                    sheet.Rows[2].Cells[3].CellFormat.Alignment =
                        Infragistics.Documents.Excel.HorizontalCellAlignment.Center;
                    sheet.Rows[2].Cells[4].CellFormat.Alignment =
                        Infragistics.Documents.Excel.HorizontalCellAlignment.Center;
                    mergedRegion2.Value = "Innovations by Sub Theme";
                }
                else
                {
                    sheet.Rows[2].Cells[3].Value = "Innovations by Sub Theme";
                    sheet.Rows[2].Cells[3].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[2].Cells[3].CellFormat.BottomBorderColor =
                      sheet.Rows[2].Cells[3].CellFormat.LeftBorderColor =
                      sheet.Rows[2].Cells[3].CellFormat.RightBorderColor =
                      sheet.Rows[2].Cells[3].CellFormat.TopBorderColor = Color.Black;
                    sheet.Rows[3].Cells[3].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[3].Cells[3].CellFormat.BottomBorderColor =
                      sheet.Rows[3].Cells[3].CellFormat.LeftBorderColor =
                      sheet.Rows[3].Cells[3].CellFormat.RightBorderColor =
                      sheet.Rows[3].Cells[3].CellFormat.TopBorderColor = Color.Black;
                }
                sheet.Rows[2].Cells[0].Value = "Stage";
                sheet.Rows[2].Cells[0].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                sheet.Rows[2].Cells[0].CellFormat.BottomBorderColor =
                      sheet.Rows[2].Cells[0].CellFormat.LeftBorderColor =
                      sheet.Rows[2].Cells[0].CellFormat.RightBorderColor =
                      sheet.Rows[2].Cells[0].CellFormat.TopBorderColor = Color.Black;
                sheet.Rows[2].Cells[1].Value = "Total";
                sheet.Rows[2].Cells[1].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                sheet.Rows[2].Cells[1].CellFormat.BottomBorderColor =
                      sheet.Rows[2].Cells[1].CellFormat.LeftBorderColor =
                      sheet.Rows[2].Cells[1].CellFormat.RightBorderColor =
                      sheet.Rows[2].Cells[1].CellFormat.TopBorderColor = Color.Black;
                sheet.Rows[3].Cells[0].Value = string.Empty;
                sheet.Rows[3].Cells[0].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                sheet.Rows[3].Cells[0].CellFormat.BottomBorderColor =
                      sheet.Rows[3].Cells[0].CellFormat.LeftBorderColor =
                      sheet.Rows[3].Cells[0].CellFormat.RightBorderColor =
                      sheet.Rows[3].Cells[0].CellFormat.TopBorderColor = Color.Black;
                sheet.Rows[3].Cells[1].Value = "Total Submissions";
                sheet.Rows[3].Cells[1].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                sheet.Rows[3].Cells[1].CellFormat.BottomBorderColor =
                      sheet.Rows[3].Cells[1].CellFormat.LeftBorderColor =
                      sheet.Rows[3].Cells[1].CellFormat.RightBorderColor =
                      sheet.Rows[3].Cells[1].CellFormat.TopBorderColor = Color.Black;
                for (int l = 1; l <= datas.FirstOrDefault().Themes.Count(); l++)
                {
                    sheet.Rows[3].Cells[1 + l].Value = datas.FirstOrDefault().Themes[l - 1].DataValue;
                    sheet.Rows[3].Cells[1 + l].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                    sheet.Rows[3].Cells[1 + l].CellFormat.BottomBorderColor =
                      sheet.Rows[3].Cells[1 + l].CellFormat.LeftBorderColor =
                      sheet.Rows[3].Cells[1 + l].CellFormat.RightBorderColor =
                      sheet.Rows[3].Cells[1 + l].CellFormat.TopBorderColor = Color.Black;
                }
                if (datas.FirstOrDefault().SubThemeList.Count() > 0)
                {
                    int n = 0;
                    while (n < subThemesNo)
                    {

                        var tempItem = string.Empty;
                        for (int l = 0; l < datas.FirstOrDefault().SubThemes.Count(); l++)
                        {
                            tempItem = datas.FirstOrDefault().SubThemeList[l].InnovationData.DataValue +
                                   " - " + datas.FirstOrDefault().SubThemeList[l].DataValue;
                            sheet.Rows[3].Cells[2 + themesNo + n].Value = tempItem;
                            sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.FillPatternForegroundColor = Color.FromArgb(217, 217, 217);
                            sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.BottomBorderColor =
                      sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.LeftBorderColor =
                      sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.RightBorderColor =
                      sheet.Rows[3].Cells[2 + themesNo + n].CellFormat.TopBorderColor = Color.Black;
                            n++;
                        }

                    }
                }
                var step = 1;
                foreach (var data in datas)
                {
                    sheet.Rows[3 + step].Cells[0].Value = data.Stage;
                    sheet.Rows[3 + step].Cells[0].CellFormat.BottomBorderColor =
                     sheet.Rows[3 + step].Cells[0].CellFormat.LeftBorderColor =
                     sheet.Rows[3 + step].Cells[0].CellFormat.RightBorderColor =
                     sheet.Rows[3 + step].Cells[0].CellFormat.TopBorderColor = Color.Black;

                    sheet.Rows[3 + step].Cells[1].Value = data.Total;
                    sheet.Rows[3 + step].Cells[1].CellFormat.BottomBorderColor =
                     sheet.Rows[3 + step].Cells[1].CellFormat.LeftBorderColor =
                     sheet.Rows[3 + step].Cells[1].CellFormat.RightBorderColor =
                     sheet.Rows[3 + step].Cells[1].CellFormat.TopBorderColor = Color.Black;
                    for (int k = 0; k < data.Themes.Count(); k++)
                    {
                        sheet.Rows[3 + step].Cells[1 + 1 + k].Value = data.Innovations.Where(x => x.ThemeID == data.Themes[k].ID).Count();
                        sheet.Rows[3 + step].Cells[1 + 1 + k].CellFormat.BottomBorderColor =
                     sheet.Rows[3 + step].Cells[1 + 1 + k].CellFormat.LeftBorderColor =
                     sheet.Rows[3 + step].Cells[1 + 1 + k].CellFormat.RightBorderColor =
                      sheet.Rows[3 + step].Cells[1 + 1 + k].CellFormat.TopBorderColor = Color.Black;
                    }
                    if (datas.FirstOrDefault().SubThemeList.Count() > 0)
                    {
                        foreach (var dt in datas)
                        {


                            var n = 0;
                            while (n < subThemesNo)
                            {

                                var tempItem = 0;

                                for (int l = 0; l < dt.SubThemes.Count(); l++)
                                {
                                    tempItem = data.Innovations.Where(x => x.SubThemeID == dt.SubThemeList[l].ID
                                                                      && x.ThemeID == datas.FirstOrDefault().SubThemeList[l].InnovationData.ID).Count();
                                    sheet.Rows[3 + step].Cells[2 + themesNo + n].Value = tempItem;
                                    sheet.Rows[3 + step].Cells[2 + themesNo + n].CellFormat.BottomBorderColor =
                      sheet.Rows[3 + step].Cells[2 + themesNo + n].CellFormat.LeftBorderColor =
                      sheet.Rows[3 + step].Cells[2 + themesNo + n].CellFormat.RightBorderColor =
                      sheet.Rows[3 + step].Cells[2 + themesNo + n].CellFormat.TopBorderColor = Color.Black;
                                    n++;
                                }


                            }
                        }
                    }
                    step++;
                }
            }
            var result = Save(workbook, "StageInnovation.xls");
            return result;
        }
        public MemoryStream ExportExtractInnovation(string periodFrom, string periodTo, IList<RangePeriod> periodsss)
        {

            var periodsList = DataService.GetInnovationPeriods(string.Empty).ToList();
            periodsList.RemoveAt(0);
            var startPeriod = periodFrom != null ? periodsList.Where(x => x.Code == periodFrom).FirstOrDefault() : null;
            var endPeriod = periodTo != null ? periodsList.Where(x => x.Code == periodTo).FirstOrDefault() : null;
            var periods = new List<InnovationPeriod>();

            if (endPeriod != null)
            {
                periods.AddRange(periodsList.Where(x => int.Parse(x.Code) >= int.Parse(startPeriod.Code) && int.Parse(x.Code) <= int.Parse(endPeriod.Code)).ToList());
            }
            else
            {
                periods.Add(startPeriod);
            }

            var datas = DataService.exportExtractSubmissionReport(periods.ToArray());

            Workbook workbook = new Workbook(WorkbookFormat.Excel97To2003);
            Infragistics.Documents.Excel.IWorkbookFont normalFont = workbook.Styles.NormalStyle.StyleFormat.Font;
            normalFont.Name = "Arial";
            normalFont.Height = 11 * 20;
            Worksheet sheet = workbook.Worksheets.Add("Innovation Details");
            Worksheet sheet1 = workbook.Worksheets.Add("Suporters and Rejecters");
            Worksheet sheet2 = workbook.Worksheets.Add("Actions Required");
            Worksheet sheet3 = workbook.Worksheets.Add("Resources Required");
            var listInnovations = new List<Innovation>();
            if (datas.Count() > 0)
            {
                sheet.Rows[0].Cells[0].Value = "Periods: " + startPeriod.Name + (endPeriod != null ? string.Format(" to {0}", endPeriod.Name) : string.Empty);
                // binding data for header table.
                var colNo = 81;
                for (int k = 0; k < colNo; k++)
                {
                    sheet.Columns[k].Width = 16 * 256;
                }
                #region report header

                sheet.Rows[2].Cells[0].Value = "Innovation Ref";
                sheet.Rows[2].Cells[1].Value = "Innovation Title";
                sheet.Rows[2].Cells[2].Value = "Description of Innovation";
                sheet.Rows[2].Cells[3].Value = "Challenge that motivated idea";//Challenge that motivated idea
                sheet.Rows[2].Cells[4].Value = "Individual or shared innovation";
                sheet.Rows[2].Cells[5].Value = "Who is / are involved in generating the idea";
                sheet.Rows[2].Cells[6].Value = "Date of Receipt";
                sheet.Rows[2].Cells[7].Value = "Parent Company";
                sheet.Rows[2].Cells[8].Value = "Contact First Name";
                sheet.Rows[2].Cells[9].Value = "Contact Surname Name";
                sheet.Rows[2].Cells[10].Value = "Contact Telephone";
                sheet.Rows[2].Cells[11].Value = "Contact Email";
                sheet.Rows[2].Cells[12].Value = "Proposing Site / Project / Directorate";// add new here
                sheet.Rows[2].Cells[13].Value = "Idea Type";
                sheet.Rows[2].Cells[14].Value = "Primary Benefit";
                sheet.Rows[2].Cells[15].Value = "Idea Maturity";
                sheet.Rows[2].Cells[16].Value = "Innovation Source";
                sheet.Rows[2].Cells[17].Value = "Hours developing before submission";// to here
                sheet.Rows[2].Cells[18].Value = "Proposing Site / Project / Directorate";
                sheet.Rows[2].Cells[19].Value = "Sponsoring Champion";
                sheet.Rows[2].Cells[20].Value = "Area";
                sheet.Rows[2].Cells[21].Value = "Detailed description of innovation";
                sheet.Rows[2].Cells[22].Value = "Keywords / Metadata";
                sheet.Rows[2].Cells[23].Value = "Theme";
                sheet.Rows[2].Cells[24].Value = "Sub Theme";
                sheet.Rows[2].Cells[25].Value = "What is the benefit / Why should Crossrail do this?";
                sheet.Rows[2].Cells[26].Value = "Benefit Clearly understood?";
                sheet.Rows[2].Cells[27].Value = "Who will Benefit?";
                sheet.Rows[2].Cells[28].Value = "Functional Sponsor";
                sheet.Rows[2].Cells[29].Value = "Suitably Developed to Share?";
                sheet.Rows[2].Cells[30].Value = "Current IRL";
                sheet.Rows[2].Cells[31].Value = "Target IRL";
                sheet.Rows[2].Cells[32].Value = "Desired outcome";
                sheet.Rows[2].Cells[33].Value = "Actions needed to reach desired outcome";
                sheet.Rows[2].Cells[34].Value = "Submitted to competition?";
                sheet.Rows[2].Cells[35].Value = "Submitted for competition date";
                sheet.Rows[2].Cells[36].Value = "Safety";
                sheet.Rows[2].Cells[37].Value = "Inspiration";
                sheet.Rows[2].Cells[38].Value = "Collaboration";
                sheet.Rows[2].Cells[39].Value = "Integrity";
                sheet.Rows[2].Cells[40].Value = "Respect";
                sheet.Rows[2].Cells[41].Value = "Last Edit";// LastEdit Here
                sheet.Rows[2].Cells[42].Value = "Current Stage";
                sheet.Rows[2].Cells[43].Value = "Current Stage % Compete";
                sheet.Rows[2].Cells[44].Value = "Innovator Contact Details Checked?";
                sheet.Rows[2].Cells[45].Value = "Date first Interview conducted";
                sheet.Rows[2].Cells[46].Value = "Date of first response?";
                sheet.Rows[2].Cells[47].Value = "Follow-up Deadline";
                sheet.Rows[2].Cells[48].Value = "Next Steps";
                sheet.Rows[2].Cells[49].Value = "(if applicable) Why has idea failed to gain support?";
                sheet.Rows[2].Cells[50].Value = "How innovation Team is to help";
                sheet.Rows[2].Cells[51].Value = "Novel Idea?";
                sheet.Rows[2].Cells[52].Value = "Innovation Idea understood?";
                sheet.Rows[2].Cells[53].Value = "Ready for Discovery Evaluation?";
                sheet.Rows[2].Cells[53].Value = "Ready for Evaluation";
                Infragistics.Documents.Excel.WorksheetMergedCellsRegion mergedRegion1 =
                      sheet.MergedCellsRegions.Add(2, 54, 2, 60);
                mergedRegion1.Value = "Attractiveness";
                sheet.Rows[3].Cells[54].Value = "Who benefits from the application of the idea?";
                sheet.Rows[3].Cells[55].Value = "What is the magnitude of the funding required?";
                sheet.Rows[3].Cells[56].Value = "What is the impact to productivity or schedule certainty?";
                sheet.Rows[3].Cells[57].Value = "How does the idea improve quality?";
                sheet.Rows[3].Cells[58].Value = "How strong is the evidence of the impact this idea will have?";
                sheet.Rows[3].Cells[59].Value = "What if any flexibility does the idea afford?";
                sheet.Rows[3].Cells[60].Value = "What is the magnitude of the risk that this idea imports to CRL?";
                Infragistics.Documents.Excel.WorksheetMergedCellsRegion mergedRegion2 = sheet.MergedCellsRegions.Add(2, 61, 2, 64);
                mergedRegion2.Value = "Complexity";

                sheet.Rows[3].Cells[61].Value = "Is there any competitive sensitivity with the idea";
                sheet.Rows[3].Cells[62].Value = "How well developed is the idea?";
                sheet.Rows[3].Cells[63].Value = "What is the level of support?";
                sheet.Rows[3].Cells[64].Value = "What is the level of effort required to develop this idea";


                sheet.Rows[2].Cells[65].Value = "Attractiveness Score";
                sheet.Rows[2].Cells[66].Value = "Complexity Score";
                sheet.Rows[2].Cells[67].Value = "Expenditure to date";
                sheet.Rows[2].Cells[68].Value = "Total Authorised Investment";
                sheet.Rows[2].Cells[69].Value = "CRL Authorised Investment";
                sheet.Rows[2].Cells[70].Value = "TSLG Funds Secured";
                sheet.Rows[2].Cells[71].Value = "TfL Funds Secured";
                sheet.Rows[2].Cells[72].Value = "RIA Funds Secured";
                sheet.Rows[2].Cells[73].Value = "BIS Funds Secured";
                sheet.Rows[2].Cells[74].Value = "Funding Comments";
                sheet.Rows[2].Cells[75].Value = "Authority to Proceed";
                sheet.Rows[2].Cells[76].Value = "Purchase Order Number";
                sheet.Rows[2].Cells[77].Value = "Maximum Value of Work";
                sheet.Rows[2].Cells[78].Value = "Delegated Authority Name";
                sheet.Rows[2].Cells[79].Value = "Delegated Authority title";
                sheet.Rows[2].Cells[80].Value = "Delegated Authority Contact Number";
                sheet.Rows[2].Cells[81].Value = "Delegated Authority Date";
                sheet.Rows[2].Cells[82].Value = "ICSC Ref";
                sheet.Rows[2].Cells[83].Value = "Date of ICSC";
                sheet.Rows[2].Cells[84].Value = "ICSC Authority to proceed";
                sheet.Rows[2].Cells[85].Value = "IA Ref";
                sheet.Rows[2].Cells[86].Value = "Date of IA";
                sheet.Rows[2].Cells[87].Value = "IA Authority to Proceed";
                sheet.Rows[2].Cells[88].Value = "IA Maximum Value of Work";
                sheet.Rows[2].Cells[89].Value = "External Authority to Proceed";
                sheet.Rows[2].Cells[90].Value = "External Ref";
                sheet.Rows[2].Cells[91].Value = "External Maximum Value of Work";
                sheet.Rows[2].Cells[92].Value = "External Delegated Authority Name";
                sheet.Rows[2].Cells[93].Value = "External Delegated Authority title";
                sheet.Rows[2].Cells[94].Value = "External Delegated Authority Contact Number";
                sheet.Rows[2].Cells[95].Value = "External Delegated Authority Date";
                sheet.Rows[2].Cells[96].Value = "External Funding Comments";

                # endregion

                foreach (var data in datas)
                {
                    listInnovations.AddRange(data.Innovations);
                }
                int i = 0;
                #region binding
                foreach (var item in listInnovations)
                {
                    var innovationProgress = DataService.GetInnovationProgressByInRef(item.InnovationRef);
                    var innovationFinance = DataService.GetInnovationFinancialByInRef(item.InnovationRef);

                    sheet.Rows[3 + i + 1].Cells[0].Value = item.InnovationRef;
                    sheet.Rows[3 + i + 1].Cells[1].Value = item.InnovationTitle;
                    sheet.Rows[3 + i + 1].Cells[2].Value = item.Description;
                    sheet.Rows[3 + i + 1].Cells[3].Value = item.InnovationContext;
                    sheet.Rows[3 + i + 1].Cells[4].Value = item.IndividualOrSharedInnovationID != null ? (item.IndividualOrSharedInnovationID.Value > 0
                                                       ? DataService.GetInnovationDataById(
                                                           item.IndividualOrSharedInnovationID.Value).DataValue
                                                           : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[5].Value = item.Who;
                    sheet.Rows[3 + i + 1].Cells[6].Value = item.DateReceipt.Value.ToString("dd/MM/yyyy");
                    sheet.Rows[3 + i + 1].Cells[7].Value = item.ParentCompany;
                    sheet.Rows[3 + i + 1].Cells[8].Value = item.ContactFirstName;
                    sheet.Rows[3 + i + 1].Cells[9].Value = item.ContactSurName;
                    sheet.Rows[3 + i + 1].Cells[10].Value = item.ContactTelephone;
                    sheet.Rows[3 + i + 1].Cells[11].Value = item.ContactEmail;

                    sheet.Rows[3 + i + 1].Cells[12].Value = item.ProposingSiteProjectID != null ? (item.ProposingSiteProjectID.Value > 0
                                                        ? DataService.GetInnovationDataById(
                                                            item.ProposingSiteProjectID.Value).DataValue
                                                            : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[13].Value = item.IdeaType != null ? item.IdeaType.DataValue : string.Empty;// hrere
                    sheet.Rows[3 + i + 1].Cells[14].Value = item.PrimaryBenefitID != null ? (item.PrimaryBenefitID.Value > 0 ? DataService.GetInnovationDataById(item.PrimaryBenefitID.Value).DataValue : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[15].Value = item.IdeaMaturityID != null ? (item.IdeaMaturityID.Value > 0 ? DataService.GetInnovationDataById(item.IdeaMaturityID.Value).DataValue : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[16].Value = item.InnovationSourceID != null ? (item.InnovationSourceID.Value > 0 ? DataService.GetInnovationDataById(item.InnovationSourceID.Value).DataValue : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[17].Value = item.HoursDeveloping != null ? item.HoursDeveloping.Value.ToString() : string.Empty;//end here
                    sheet.Rows[3 + i + 1].Cells[18].Value = item.SponsoringChampion;// confirm
                    sheet.Rows[3 + i + 1].Cells[19].Value = item.SponsoringChampion;// confirm
                    sheet.Rows[3 + i + 1].Cells[20].Value = item.Area;
                    sheet.Rows[3 + i + 1].Cells[21].Value = item.DetailedDescription;
                    sheet.Rows[3 + i + 1].Cells[22].Value = item.Keywords;
                    sheet.Rows[3 + i + 1].Cells[23].Value = item.ThemeID != null ? (item.ThemeID.Value > 0
                                                        ? DataService.GetInnovationDataById(item.ThemeID.Value).DataValue : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[24].Value = item.SubThemeID != null ? (item.SubThemeID.Value > 0
                                                        ? DataService.FindSubTheme(item.SubThemeID.Value).DataValue : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[25].Value = item.Benefit;
                    sheet.Rows[3 + i + 1].Cells[26].Value = item.BenefitClearlyUnderstoodId != null ? (item.BenefitClearlyUnderstoodId.Value > 0
                                                        ? DataService.GetInnovationDataById(
                                                            item.BenefitClearlyUnderstoodId.Value).DataValue
                                                            : string.Empty) : string.Empty;


                    sheet.Rows[3 + i + 1].Cells[27].Value = item.WhoWillBenefit;
                    sheet.Rows[3 + i + 1].Cells[28].Value = item.FunctionalSponsor;
                    sheet.Rows[3 + i + 1].Cells[29].Value = item.SuitablyDevelopedToShareID != null ? (item.SuitablyDevelopedToShareID.Value > 0
                                                        ? DataService.GetInnovationDataById(
                                                            item.SuitablyDevelopedToShareID.Value).DataValue
                                                            : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[30].Value = item.CurrentIRLID != null ? (item.CurrentIRLID.Value > 0
                                                        ? DataService.GetInnovationDataById(item.CurrentIRLID.Value).
                                                              DataValue
                                                              : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[31].Value = item.TargetIRLID != null ? (item.TargetIRLID.Value > 0
                                                        ? DataService.GetInnovationDataById(item.TargetIRLID.Value).
                                                              DataValue
                                                              : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[32].Value = item.DesiredOutcome;
                    sheet.Rows[3 + i + 1].Cells[33].Value = item.Actions;
                    sheet.Rows[3 + i + 1].Cells[34].Value = item.IsSubmitted == true ? "Yes" : "No";
                    sheet.Rows[3 + i + 1].Cells[35].Value = item.SubmittedForCompetitionDate != SqlDateTime.MinValue.Value ? item.SubmittedForCompetitionDate.ToString("dd/MM/yyyy") : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[36].Value = item.IsSafety != null ? (item.IsSafety == true ? "Yes" : "No") : "No";
                    sheet.Rows[3 + i + 1].Cells[37].Value = item.IsInspiration != null ? (item.IsInspiration == true ? "Yes" : "No") : "No";
                    sheet.Rows[3 + i + 1].Cells[38].Value = item.IsCollaboration != null ? (item.IsCollaboration == true ? "Yes" : "No") : "No";
                    sheet.Rows[3 + i + 1].Cells[39].Value = item.IsIntegrity != null ? (item.IsIntegrity == true ? "Yes" : "No") : "No";
                    sheet.Rows[3 + i + 1].Cells[40].Value = item.IsRespect != null ? (item.IsRespect == true ? "Yes" : "No") : "No";
                    sheet.Rows[3 + i + 1].Cells[41].Value = innovationProgress != null ? (innovationProgress.LastEdit.HasValue && innovationProgress.LastEdit.Value != SqlDateTime.MinValue.Value ? innovationProgress.LastEdit.Value.ToString("dd/MM/yyyy") : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[42].Value = innovationProgress != null ? (innovationProgress.CurrentStage != null ? innovationProgress.CurrentStage.DataValue : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[43].Value = innovationProgress != null ? innovationProgress.CurrentStageComplete + "%" : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[44].Value = innovationProgress != null ? (innovationProgress.IsInnovatorContactDetailsChecked == true ? "Yes" : "No") : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[45].Value = innovationProgress != null ? (innovationProgress.DatefirstInterviewConducted != SqlDateTime.MinValue.Value ? innovationProgress.DatefirstInterviewConducted.ToString("dd/MM/yyyy") : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[46].Value = innovationProgress != null ? (innovationProgress.DateOfFirstResponse != SqlDateTime.MinValue.Value ? innovationProgress.DateOfFirstResponse.ToString("dd/MM/yyyy") : string.Empty) : string.Empty; //innovationProgress.DateOfFirstResponse.ToString("dd/MM/yyyy") : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[47].Value = innovationProgress != null ? (innovationProgress.FollowupDeadline != SqlDateTime.MinValue.Value ? innovationProgress.FollowupDeadline.ToString("dd/MM/yyyy") : string.Empty) : string.Empty;//innovationProgress.FollowupDeadline.ToString("dd/MM/yyyy") : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[48].Value = innovationProgress != null ? innovationProgress.NextSteps : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[49].Value = innovationProgress != null ? innovationProgress.WhyFailedSupport : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[50].Value = innovationProgress != null ? innovationProgress.HowToHelp : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[51].Value = innovationProgress != null ? (innovationProgress.IsNovelIdea == true ? "Yes" : "No") : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[52].Value = innovationProgress != null ? (innovationProgress.IsUnderstood == true ? "Yes" : "No") : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[53].Value = innovationProgress != null ? (innovationProgress.IsReadyForDiscoveryEvaluation == true ? "Yes" : "No") : string.Empty;
                    // Attractiveness
                    sheet.Rows[3 + i + 1].Cells[54].Value = innovationProgress != null ? (innovationProgress.WhoBenefits > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.WhoBenefits).AnswerContent
                                                                    : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[55].Value = innovationProgress != null ? (innovationProgress.WhatMagnitudeRequired > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.WhatMagnitudeRequired).AnswerContent
                                                                    : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[56].Value = innovationProgress != null ? (innovationProgress.WhatImpactToProductivity > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.WhatImpactToProductivity).AnswerContent
                                                                : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[57].Value = innovationProgress != null ? (innovationProgress.HowIdeaImproveQuality > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.HowIdeaImproveQuality).AnswerContent
                                                                : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[58].Value = innovationProgress != null ? (innovationProgress.HowStrongEvidenceImpact > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.HowStrongEvidenceImpact).AnswerContent
                                                                : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[59].Value = innovationProgress != null ? (innovationProgress.WhatFlexibilityIdeaAfford > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.WhatFlexibilityIdeaAfford).AnswerContent
                                                                : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[60].Value = innovationProgress != null ? (innovationProgress.WhatMagnitudeIdeaImportstoCRL > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.WhatMagnitudeIdeaImportstoCRL).AnswerContent
                                                                : string.Empty) : string.Empty;
                    //Complexity
                    sheet.Rows[3 + i + 1].Cells[61].Value = innovationProgress != null ? (innovationProgress.CompetitiveSensitivityWithIdea > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.CompetitiveSensitivityWithIdea).AnswerContent
                                                                : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[62].Value = innovationProgress != null ? (innovationProgress.HowWellDevelopedIdea > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.HowWellDevelopedIdea).AnswerContent
                                                                : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[63].Value = innovationProgress != null ? (innovationProgress.WhatLevelofSupport > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.WhatLevelofSupport).AnswerContent
                                                                : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[64].Value = innovationProgress != null ? (innovationProgress.WhatLevelofEffortRequired > 0
                                                                ? DataService.GetAnswersById(
                                                                    innovationProgress.WhatLevelofEffortRequired).AnswerContent
                                                                : string.Empty) : string.Empty;

                    sheet.Rows[3 + i + 1].Cells[65].Value = GetAttractivenessScore(item.InnovationRef);
                    sheet.Rows[3 + i + 1].Cells[66].Value = GetComplexityScore(item.InnovationRef);
                    sheet.Rows[3 + i + 1].Cells[67].Value = innovationFinance != null ? "£" + innovationFinance.ExpenditureToDate : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[68].Value = innovationFinance != null ? "£" + innovationFinance.TotalAuthorisedInvestment : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[69].Value = innovationFinance != null ? "£" + innovationFinance.CRLAuthorisedInvestment : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[70].Value = innovationFinance != null ? "£" + innovationFinance.TSLGFundsSecured : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[71].Value = innovationFinance != null ? "£" + innovationFinance.TfLFundsSecured : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[72].Value = innovationFinance != null ? "£" + innovationFinance.RIAFundsSecured : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[73].Value = innovationFinance != null ? "£" + innovationFinance.BISFundsSecured : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[74].Value = innovationFinance != null ? "£" + innovationFinance.FundingComments : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[75].Value = innovationFinance != null ? (innovationFinance.IsIncifAuthoritytoProceed == true ? "Yes" : "No") : string.Empty; ;
                    sheet.Rows[3 + i + 1].Cells[76].Value = innovationFinance != null ? "£" + innovationFinance.IncifPurchaseOrderNumber : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[77].Value = innovationFinance != null ? "£" + innovationFinance.IncifMaximumValueofWork : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[78].Value = innovationFinance != null ? innovationFinance.IncifDelegatedAuthorityName : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[79].Value = innovationFinance != null ? innovationFinance.IncifDelegatedAuthorityTitle : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[80].Value = innovationFinance != null ? innovationFinance.IncifDelegatedAuthorityContactNumber : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[81].Value = innovationFinance != null ? (innovationFinance.IncifDelegatedAuthorityDate != SqlDateTime.MinValue.Value ? innovationFinance.IncifDelegatedAuthorityDate.ToString("dd/MM/yyyy") : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[82].Value = innovationFinance != null ? innovationFinance.ICSCRef : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[83].Value = innovationFinance != null ? (innovationFinance.DateofICSC != SqlDateTime.MinValue.Value ? innovationFinance.DateofICSC.ToString("dd/MM/yyyy") : string.Empty) : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[84].Value = innovationFinance != null ? (innovationFinance.IsICSCAuthoritytoProceed == true ? "Yes" : "No") : string.Empty; ;
                    sheet.Rows[3 + i + 1].Cells[85].Value = innovationFinance != null ? innovationFinance.IARef : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[86].Value = innovationFinance != null ? (innovationFinance.DateofIA != SqlDateTime.MinValue.Value ? innovationFinance.DateofIA.ToString("dd/MM/yyyy") : string.Empty) : string.Empty;//innovationFinance.DateofIA.ToString("dd/MM/yyyy") : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[87].Value = innovationFinance != null ? (innovationFinance.IsIAAuthoritytoProceed == true ? "Yes" : "No") : string.Empty; ;
                    sheet.Rows[3 + i + 1].Cells[88].Value = innovationFinance != null ? "£" + innovationFinance.IAMaximumValueofWork : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[89].Value = innovationFinance != null ? (innovationFinance.IsExternalAuthoritytoProceed == true ? "Yes" : "No") : string.Empty; ;
                    sheet.Rows[3 + i + 1].Cells[90].Value = innovationFinance != null ? innovationFinance.ExternalRef : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[91].Value = innovationFinance != null ? "£" + innovationFinance.ExternalMaximumValueofWork : "£" + 0;
                    sheet.Rows[3 + i + 1].Cells[92].Value = innovationFinance != null ? innovationFinance.ExternalDelegatedAuthorityName : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[93].Value = innovationFinance != null ? innovationFinance.ExternalDelegatedAuthorityTitle : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[94].Value = innovationFinance != null ? innovationFinance.ExternalDelegatedAuthorityContactNumber : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[95].Value = innovationFinance != null ? (innovationFinance.ExternalDelegatedAuthorityDate != SqlDateTime.MinValue.Value ? innovationFinance.ExternalDelegatedAuthorityDate.ToString("dd/MM/yyyy") : string.Empty) : string.Empty;//innovationFinance.ExternalDelegatedAuthorityDate.ToString("dd/MM/yyyy") : string.Empty;
                    sheet.Rows[3 + i + 1].Cells[96].Value = innovationFinance != null ? innovationFinance.ExternalFundingComments : string.Empty;


                    i++;
                }

                #endregion
            }
            for (int k = 0; k < 3; k++)
            {
                sheet1.Columns[k].Width = 16 * 256;
            }
            sheet1.Rows[0].Cells[0].Value = "Innovatin Ref";
            sheet1.Rows[0].Cells[1].Value = "Name";
            sheet1.Rows[0].Cells[2].Value = "Reason why";

            List<SuporterOrRejecter> listSheet1 = new List<SuporterOrRejecter>();
            foreach (var row in listInnovations)
            {
                var p = from d in DataService.GetAllPeopleSupportByInnovationId(row.ID)
                        select new SuporterOrRejecter
                        {
                            InnovationRef = row.InnovationRef,
                            Name = d.Name,
                            ReasonWhy = d.Reason
                        };
                listSheet1.AddRange(p.ToList());

            }
            int rowCount = 0;
            foreach (var suporterOrRejecter in listSheet1)
            {
                rowCount++;
                sheet1.Rows[rowCount].Cells[0].Value = suporterOrRejecter.InnovationRef;
                sheet1.Rows[rowCount].Cells[1].Value = suporterOrRejecter.Name;
                sheet1.Rows[rowCount].Cells[2].Value = suporterOrRejecter.ReasonWhy;
            }
            for (int k = 0; k < 4; k++)
            {
                sheet1.Columns[k].Width = 16 * 256;
            }

            sheet2.Rows[0].Cells[0].Value = "Innovatin Ref";
            sheet2.Rows[0].Cells[1].Value = "Action";
            sheet2.Rows[0].Cells[2].Value = "Owner";
            sheet2.Rows[0].Cells[3].Value = "Completed";

            List<ActionRequiredReport> listSheet2 = new List<ActionRequiredReport>();
            foreach (var row in listInnovations)
            {
                var p = from d in DataService.GetAllActionsRequiredByInnovationId(row.ID)
                        select new ActionRequiredReport
                        {
                            InnovationRef = row.InnovationRef,
                            Action = d.Actions,
                            Completed = d.Completed
                        };
                listSheet2.AddRange(p.ToList());

            }
            rowCount = 0;
            foreach (var actionRequiredReport in listSheet2)
            {
                rowCount++;
                sheet2.Rows[rowCount].Cells[0].Value = actionRequiredReport.InnovationRef;
                sheet2.Rows[rowCount].Cells[1].Value = actionRequiredReport.Action;
                sheet2.Rows[rowCount].Cells[2].Value = actionRequiredReport.Owner;
                sheet2.Rows[rowCount].Cells[3].Value = actionRequiredReport.Completed == true ? "Yes" : "No";
            }
            for (int k = 0; k < 4; k++)
            {
                sheet3.Columns[k].Width = 16 * 256;
            }
            sheet3.Rows[0].Cells[0].Value = "Innovatin Ref";
            sheet3.Rows[0].Cells[1].Value = "Name";
            sheet3.Rows[0].Cells[2].Value = "Role";
            sheet3.Rows[0].Cells[3].Value = "Hours";
            sheet3.Rows[0].Cells[4].Value = "Output";
            rowCount = 0;
            List<ResourceRequiredReport> listSheet3 = new List<ResourceRequiredReport>();
            foreach (var row in listInnovations)
            {
                var p = from d in DataService.GetAllResourcesRequiredByInnovationId(row.ID)
                        select new ResourceRequiredReport
                        {
                            InnovationRef = row.InnovationRef,
                            Name = d.Name,
                            Hours = d.Hours,
                            Output = d.Output,
                            Role = d.Role
                        };
                listSheet3.AddRange(p.ToList());

            }
            foreach (var resourceRequiredReport in listSheet3)
            {
                rowCount++;
                sheet3.Rows[rowCount].Cells[0].Value = resourceRequiredReport.InnovationRef;
                sheet3.Rows[rowCount].Cells[1].Value = resourceRequiredReport.Name;
                sheet3.Rows[rowCount].Cells[2].Value = resourceRequiredReport.Role;
                sheet3.Rows[rowCount].Cells[3].Value = resourceRequiredReport.Hours;
                sheet3.Rows[rowCount].Cells[4].Value = resourceRequiredReport.Output;

            }
            var result = Save(workbook, "extractInnovation.xls");
            return result;
        }
        private MemoryStream Save(Workbook document, string fileName)
        {
            MemoryStream temp = new MemoryStream();
            document.SetCurrentFormat(WorkbookFormat.Excel97To2003);
            document.Save(Response.OutputStream);
            document.Save(temp);
            return temp;
        }
        public byte[] ToByte(Stream stream)
        {
            long originalPosition = 0;

            if (stream.CanSeek)
            {
                originalPosition = stream.Position;
                stream.Position = 0;
            }

            try
            {
                var readBuffer = new byte[4096];

                var totalBytesRead = 0;
                int bytesRead;

                while ((bytesRead = stream.Read(readBuffer, totalBytesRead, readBuffer.Length - totalBytesRead)) > 0)
                {
                    totalBytesRead += bytesRead;

                    if (totalBytesRead == readBuffer.Length)
                    {
                        var nextByte = stream.ReadByte();
                        if (nextByte != -1)
                        {
                            var temp = new byte[readBuffer.Length * 2];
                            Buffer.BlockCopy(readBuffer, 0, temp, 0, readBuffer.Length);
                            Buffer.SetByte(temp, totalBytesRead, (byte)nextByte);
                            readBuffer = temp;
                            totalBytesRead++;
                        }
                    }
                }

                var buffer = readBuffer;
                if (readBuffer.Length != totalBytesRead)
                {
                    buffer = new byte[totalBytesRead];
                    Buffer.BlockCopy(readBuffer, 0, buffer, 0, totalBytesRead);
                }
                return buffer;
            }
            finally
            {
                if (stream.CanSeek)
                {
                    stream.Position = originalPosition;
                }
            }
        }
    }

}
