﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ComponentModel.DataAnnotations;

namespace Crossrail.IMS.WebApp.Models
{
    public class InnovationSummaryModel
    {
        /// <summary>
        /// The id of the Innovation
        /// </summary>
        public int ID { get; set; }

        /// <summary>
        ///Read-only
        ///Format: INVXXXXX 
        ///XXXXX is a numerical identified assigned concurrently from 00050 to 99999
        /// </summary>
        [StringLength(8)]
        public string InnovationRef { get; set; }

        [StringLength(256)]
        public string InnovationTitle { get; set; }

        /// <summary>
        /// Description of Innovation
        /// </summary>
        [StringLength(2000)]
        public string Description { get; set; }

        /// <summary>
        /// Context of Innovation or Problems addressed
        /// </summary>
        [StringLength(2000)]
        public string InnovationContext { get; set; }

        /// <summary>
        /// How can we help?
        /// The dropdown list
        /// </summary>
        public int? HowCanWeHelpID { get; set; }

        /// <summary>
        /// Individual or shared innovation
        /// The dropdown list
        /// </summary>
        public int? IndividualOrSharedInnovationID { get; set; }

        /// <summary>
        /// Who is/are involved in generating the idea
        /// </summary>
        [StringLength(1000)]
        public string Who { get; set; }

        /// <summary>
        /// Format:dd/mm/yyyyhh:ss
        ///This value cannot be in the future.
        /// </summary>
        public DateTime DateReceipt { get; set; }

        [StringLength(256)]
        public string ParentCompany { get; set; }

        [StringLength(32)]
        public string ContactFirstName { get; set; }

        [StringLength(32)]
        public string ContactSurName { get; set; }

        [StringLength(128)]
        public string ContactTelephone { get; set; }

        [StringLength(32)]
        [DataType(DataType.EmailAddress)]
        public string ContactEmail { get; set; }

        /// <summary>
        /// Proposing Site/ Project/Directorate
        /// </summary>
        public int? ProposingSiteProjectID { get; set; }

        /// <summary>
        /// Sponsoring Champion
        /// </summary>
        [StringLength(256)]
        public string SponsoringChampion { get; set; }


        public int? Area { get; set; }

        /// <summary>
        /// Detailed description of innovation
        /// </summary>
        [StringLength(2000)]
        public string DetailedDescription { get; set; }

        /// <summary>
        /// Keywords / Metadata
        /// </summary>
        [StringLength(2000)]
        public string Keywords { get; set; }


        public int? ThemeID { get; set; }


        public int? SubThemeID { get; set; }

        /// <summary>
        /// What is the benefit / why should Crossrail do this?
        /// </summary>
        [StringLength(2000)]
        public string Benefit { get; set; }


        public int? BenefitClearlyUnderstoodId { get; set; }

        /// <summary>
        /// Who will benefit?
        /// </summary>
        [StringLength(2000)]
        public string WhoWillBenefit { get; set; }

        /// <summary>
        /// Validated against AD
        ///This searches all users within our Active Directory (not Innovation System Users) and validates against this.
        ///Ideally it would display the names in AD that match / are close to the users input allowing them 
        ///to select the correct person.
        /// </summary>
        public string FunctionalSponsor { get; set; }

        /// <summary>
        /// Suitably Developed to share?
        /// </summary>
        public int? SuitablyDevelopedToShareID { get; set; }


        public int? CurrentIRLID { get; set; }


        public int? TargetIRLID { get; set; }

        /// <summary>
        /// Desired outcome
        /// </summary>
        [StringLength(2000)]
        public string DesiredOutcome { get; set; }

        /// <summary>
        /// Actions needed to reach desired outcome
        /// </summary>
        [StringLength(2000)]
        public string Actions { get; set; }

        /// <summary>
        /// Values:
        /// Yes : true
        ///No :false
        /// </summary>
        public bool IsSubmitted { get; set; }

        /// <summary>
        /// Submitted for Competition Date
        /// Dropdown list
        /// If “Submitted” field is “Yes”, it will be auto-populated. Otherwise, it is blank and disabled
        /// </summary>
        public DateTime SubmittedForCompetitionDate { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsSafety { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsInspiration { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsCollaboration { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsIntegrity { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsRespect { get; set; }

        /// <summary>
        /// The description of the link document
        /// </summary>
        [StringLength(245)]
        public string LinkedDocumentDescription { get; set; }


        public string LinkedDocument { get; set; }


        public string DocumentURL { get; set; }


        public string Notes { get; set; }
    }
}