﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;
using Crossrail.IMS.WebApp.DataGatewayService;

namespace Crossrail.IMS.WebApp.Models
{
	public class ActionsRequiredModel
	{
		public int ID { get; set; }

		/// <summary>
		/// Gets or sets the actions.
		/// </summary>
		/// <value>
		/// The actions.
		/// </value>
		[Required(ErrorMessage = "Action is required"), MaxLength(256)]
		[RegularExpression(@"^(?!\s*$).+", ErrorMessage = "Action cannot be empty")]
		public string Actions { get; set; }

		/// <summary>
		/// Gets or sets the owner.
		/// </summary>
		/// <value>
		/// The owner.
		/// </value>
		[Required(ErrorMessage = "Owner is required"), MaxLength(128)]
		[RegularExpression(@"^(?!\s*$).+", ErrorMessage = "Owner cannot be empty")]
		public string Owner { get; set; }

		/// <summary>
		/// Gets or sets a value indicating whether this is completed.
		/// </summary>
		/// <value>
		///   <c>true</c> if completed; otherwise, <c>false</c>.
		/// </value>
		public bool Completed { get; set; }

		/// <summary>
		/// Gets or sets the innovation progress.
		/// </summary>
		/// <value>
		/// The innovation progress.
		/// </value>
		public int Innovation_ID { get; set; }

		/// <summary>
		/// Gets or sets the org ID.
		/// </summary>
		/// <value>
		/// The org ID.
		/// </value>
		public int OrgID { get; set; }

        public Guid TemptId { get; set; }

        public static ActionsRequiredTemporary ConvertToActionsRequiredEntity(ActionsRequiredModel data)
        {
            return new ActionsRequiredTemporary()
            {
                ID = data.ID,
                Actions = data.Actions.Trim(),
                Completed = data.Completed,
                Owner = data.Owner != null ? data.Owner.Trim() : null,
                TemptId = data.TemptId,
                OrgID = data.OrgID
            };
        }
	}
}