﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Crossrail.IMS.WebApp.DataGatewayService;
using Crossrail.IMS.WebApp.Utilities;
using System.Data.SqlTypes;

namespace Crossrail.IMS.WebApp.Models
{
    /// <summary>
    /// InnovationProgressModel
    /// </summary>
    public class InnovationProgressModel
    {
        #region Innovation Progress Details

        public int ID { get; set; }

        /// <summary>
        /// Gets or sets the latest update.
        /// </summary>
        /// <value>
        /// The latest update.
        /// </value>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [AllowHtml]
        public string LatestUpdate { get; set; }

        /// <summary>
        /// Gets or sets the current stage complete.
        /// </summary>
        /// <value>
        /// The current stage complete.
        /// </value>
        public decimal CurrentStageComplete { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is innovator contact details checked.
        /// </summary>
        /// <value>
        /// <c>true</c> if this instance is innovator contact details checked; otherwise, <c>false</c>.
        /// </value>
        public bool? IsInnovatorContactDetailsChecked { get; set; }

        /// <summary>
        /// Gets or sets the datefirst interview conducted.
        /// </summary>
        /// <value>
        /// The datefirst interview conducted.
        /// </value>
        /// 
        [DisplayName("Date first Interview conducted")]
        //[DisplayFormat(ApplyFormatInEditMode = true, DataFormatString = "{0:dd/mm/yyyy}")]
        //public DateTime? DatefirstInterviewConducted { get; set; }
        public string DatefirstInterviewConducted { get; set; }

        /// <summary>
        /// Gets or sets the date of first response.
        /// </summary>
        /// <value>
        /// The date of first response.
        /// </value>
        [DisplayName("Date of first response?")]
        //[DisplayFormat(ApplyFormatInEditMode = true, DataFormatString = "{0:dd/mm/yyyy}")]
        //public DateTime? DateOfFirstResponse { get; set; }
        public string DateOfFirstResponse { get; set; }

        /// <summary>
        /// Gets or sets the followup deadline.
        /// </summary>
        /// <value>
        /// The followup deadline.
        /// </value>
        [DisplayName("Follow-up Deadline")]
        //[DisplayFormat(ApplyFormatInEditMode = true, DataFormatString = "{0:dd/mm/yyyy}")]
        //public DateTime? FollowupDeadline { get; set; }
        public string FollowupDeadline { get; set; }

        /// <summary>
        /// Gets or sets the next steps.
        /// </summary>
        /// <value>
        /// The next steps.
        /// </value>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string NextSteps { get; set; }

        /// <summary>
        /// Gets or sets the why failed support.
        /// </summary>
        /// <value>
        /// The why failed support.
        /// </value>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string WhyFailedSupport { get; set; }

        /// <summary>
        /// Gets or sets the how to help.
        /// </summary>
        /// <value>
        /// The how to help.
        /// </value>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string HowToHelp { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is novel idea.
        /// </summary>
        /// <value>
        /// <c>true</c> if this instance is novel idea; otherwise, <c>false</c>.
        /// </value>
        public bool? IsNovelIdea { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is understood.
        /// </summary>
        /// <value>
        /// <c>true</c> if this instance is understood; otherwise, <c>false</c>.
        /// </value>
        public bool? IsUnderstood { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is ready for discovery evaluation.
        /// </summary>
        /// <value>
        /// <c>true</c> if this instance is ready for discovery evaluation; otherwise, <c>false</c>.
        /// </value>
        public bool? IsReadyForDiscoveryEvaluation { get; set; }

        /// <summary>
        /// Gets or sets the who benefits.
        /// </summary>
        /// <value>
        /// The who benefits.
        /// </value>
        public int? WhoBenefits { get; set; }

        /// <summary>
        /// Gets or sets the what magnitude required.
        /// </summary>
        /// <value>
        /// The what magnitude required.
        /// </value>
        public int WhatMagnitudeRequired { get; set; }

        /// <summary>
        /// Gets or sets the what impact to productivity.
        /// </summary>
        /// <value>
        /// The what impact to productivity.
        /// </value>
        public int WhatImpactToProductivity { get; set; }

        /// <summary>
        /// Gets or sets the how idea improve quality.
        /// </summary>
        /// <value>
        /// The how idea improve quality.
        /// </value>
        public int HowIdeaImproveQuality { get; set; }

        /// <summary>
        /// Gets or sets the how strong evidence impact.
        /// </summary>
        /// <value>
        /// The how strong evidence impact.
        /// </value>
        public int HowStrongEvidenceImpact { get; set; }

        /// <summary>
        /// Gets or sets the what flexibility idea afford.
        /// </summary>
        /// <value>
        /// The what flexibility idea afford.
        /// </value>
        public int WhatFlexibilityIdeaAfford { get; set; }

        /// <summary>
        /// Gets or sets the what magnitude idea importsto CRL.
        /// </summary>
        /// <value>
        /// The what magnitude idea importsto CRL.
        /// </value>
        public int WhatMagnitudeIdeaImportstoCRL { get; set; }

        /// <summary>
        /// Gets or sets the competitive sensitivity with idea.
        /// </summary>
        /// <value>
        /// The competitive sensitivity with idea.
        /// </value>
        public int CompetitiveSensitivityWithIdea { get; set; }

        /// <summary>
        /// Gets or sets the how well developed idea.
        /// </summary>
        /// <value>
        /// The how well developed idea.
        /// </value>
        public int HowWellDevelopedIdea { get; set; }

        /// <summary>
        /// Gets or sets the what levelof support.
        /// </summary>
        /// <value>
        /// The what levelof support.
        /// </value>
        public int WhatLevelofSupport { get; set; }

        /// <summary>
        /// Gets or sets the what levelof effort required.
        /// </summary>
        /// <value>
        /// The what levelof effort required.
        /// </value>
        public int WhatLevelofEffortRequired { get; set; }

        /// <summary>
        /// Gets or sets the innovation.
        /// </summary>
        /// <value>
        /// The innovation.
        /// </value>
        public Innovation Innovation { get; set; }

        /// <summary>
        /// Gets or sets the current stage.
        /// </summary>
        /// <value>
        /// The current stage.
        /// </value>
        public InnovationData CurrentStage { get; set; }

        public int? CurrentStateId { get; set; }

        public string LastEdit { get; set; }

        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string ProgrammeManagersRecommendation { get; set; }
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string WorkingGroupRecommendations { get; set; }
        [StringLength(500, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string ForumDecision { get; set; }

        #endregion

        #region Child Models

        /// <summary>
        /// The actions required model
        /// </summary>
        public IEnumerable<ActionsRequiredModel> ActionsRequiredModel { get; set; }

        /// <summary>
        /// The people support model
        /// </summary>
        public IEnumerable<PeopleSupportModel> PeopleSupportModel { get; set; }

        /// <summary>
        /// The resources required model
        /// </summary>
        public IEnumerable<ResourcesRequiredModel> ResourcesRequiredModel { get; set; }

        #endregion

        #region Collections

        public List<SelectListItem> Stages { get; set; }

        public List<AttractivenessComplexity> WhoBenefitsAnswers { get; set; }

        public List<AttractivenessComplexity> WhatMagnitudeOfTheFundingAnswers { get; set; }

        public List<AttractivenessComplexity> WhatImpactToProductivityAnswers { get; set; }

        public List<AttractivenessComplexity> HowIdeaImproveQualityAnswers { get; set; }

        public List<AttractivenessComplexity> HowStrongAnswers { get; set; }

        public List<AttractivenessComplexity> WhatFlexibilityIdeaAffordAnswers { get; set; }

        public List<AttractivenessComplexity> WhatMagnitudeRiskAnswers { get; set; }

        public List<AttractivenessComplexity> CompetitiveSensitivityWithIdeaAnswers { get; set; }

        public List<AttractivenessComplexity> HowWellDevelopedIdeaAnswers { get; set; }

        public List<AttractivenessComplexity> WhatLevelofSupportAnswers { get; set; }

        public List<AttractivenessComplexity> WhatLevelofEffortRequiredAnswers { get; set; }

        public List<SelectListItem> IsInnovatorContactDetailsCheckedOptions { get; set; }

        public List<SelectListItem> IsNovelIdeaOptions { get; set; }

        public List<SelectListItem> IsUnderstoodOptions { get; set; }

        public List<SelectListItem> IsReadyForDiscoveryEvaluationOptions { get; set; }
        #endregion

        public InnovationProgressModel()
        {
            ActionsRequiredModel = new List<ActionsRequiredModel>();
            PeopleSupportModel = new List<PeopleSupportModel>();
            ResourcesRequiredModel = new List<ResourcesRequiredModel>();
        }

        #region Fields Permission

        public bool HasWriteOnLatestUpateField { get; set; }

        public bool HasWriteOnCurrentStage { get; set; }

        public bool HasWriteOnCurrentStageCompleted { get; set; }

        public bool HasWriteOnInnovatorContactDetailsChecked { get; set; }

        public bool HasWriteOnDateFirstInterview { get; set; }

        public bool HasWriteOnDateFirstResponse { get; set; }

        public bool HasWriteOnFollowupDeadline { get; set; }

        public bool HasWriteOnNextSteps { get; set; }

        public bool HasWriteOnWhyIdealFailed { get; set; }

        public bool HasWriteOnPeopleSupport { get; set; }

        public bool HasWriteOnHowInnovationTeamIsToHelp { get; set; }

        public bool HasWriteOnActionsRequired { get; set; }

        public bool HasWriteOnNovelIdea { get; set; }

        public bool HasWriteOnIdeaUnderstood { get; set; }

        public bool HasWriteOnReadyDiscoveryEvaluation { get; set; }

        public bool HasWriteOnResourcesRequired { get; set; }

        public string HasWriteOnAttractiveness { get; set; }

        public string HasWriteOnComplexity { get; set; }



        #endregion

        #region Convert Method

        /// <summary>
        /// Convert the innovation progress model to innovation progress entity
        /// </summary>
        public static InnovationProgress ConvertToInnovationProgressEntity(InnovationProgressModel model)
        {
            InnovationProgress entity = new InnovationProgress()
            {
                ID = model.ID,
                LatestUpdate = model.LatestUpdate,
                CurrentStage = model.CurrentStateId.HasValue ? new InnovationData { ID = model.CurrentStateId.Value } : null,
                CurrentStageComplete = model.CurrentStageComplete,
                IsInnovatorContactDetailsChecked = model.IsInnovatorContactDetailsChecked,
                NextSteps = model.NextSteps,
                WhyFailedSupport = model.WhyFailedSupport,
                HowToHelp = model.HowToHelp,
                IsNovelIdea = model.IsNovelIdea,
                IsUnderstood = model.IsUnderstood,
                IsReadyForDiscoveryEvaluation = model.IsReadyForDiscoveryEvaluation,
                ProgrammeManagersRecommendation = model.ProgrammeManagersRecommendation,
                WorkingGroupRecommendations = model.WorkingGroupRecommendations,
                ForumDecision = model.ForumDecision
            };

            entity.DatefirstInterviewConducted = CommonUtil.ParseDate(model.DatefirstInterviewConducted);
            entity.DateOfFirstResponse = CommonUtil.ParseDate(model.DateOfFirstResponse);
            entity.FollowupDeadline = CommonUtil.ParseDate(model.FollowupDeadline);

            return entity;
        }

        /// <summary>
        /// Convert the innovation progress entity to innovation progress model
        /// </summary>
        public static InnovationProgressModel ConvertToInnovationProgressModel(InnovationProgress innovationProgress)
        {
            if (innovationProgress == null)
            {
                return new InnovationProgressModel();
            }
            var progressModel = new InnovationProgressModel
            {
                ID = innovationProgress.ID,
                LatestUpdate = innovationProgress.LatestUpdate,
                CurrentStage = innovationProgress.CurrentStage,
                CurrentStageComplete = innovationProgress.CurrentStageComplete,
                IsInnovatorContactDetailsChecked = innovationProgress.IsInnovatorContactDetailsChecked,
                NextSteps = innovationProgress.NextSteps,
                WhyFailedSupport = innovationProgress.WhyFailedSupport,
                HowToHelp = innovationProgress.HowToHelp,
                IsNovelIdea = innovationProgress.IsNovelIdea,
                IsUnderstood = innovationProgress.IsUnderstood,
                IsReadyForDiscoveryEvaluation = innovationProgress.IsReadyForDiscoveryEvaluation,
                WhoBenefits = innovationProgress.WhoBenefits,
                WhatMagnitudeRequired = innovationProgress.WhatMagnitudeRequired,
                WhatImpactToProductivity = innovationProgress.WhatImpactToProductivity,
                HowIdeaImproveQuality = innovationProgress.HowIdeaImproveQuality,
                HowStrongEvidenceImpact = innovationProgress.HowStrongEvidenceImpact,
                WhatFlexibilityIdeaAfford = innovationProgress.WhatFlexibilityIdeaAfford,
                WhatMagnitudeIdeaImportstoCRL = innovationProgress.WhatMagnitudeIdeaImportstoCRL,
                CompetitiveSensitivityWithIdea = innovationProgress.CompetitiveSensitivityWithIdea,
                HowWellDevelopedIdea = innovationProgress.HowWellDevelopedIdea,
                WhatLevelofSupport = innovationProgress.WhatLevelofSupport,
                WhatLevelofEffortRequired = innovationProgress.WhatLevelofEffortRequired,
                ProgrammeManagersRecommendation = innovationProgress.ProgrammeManagersRecommendation,
                WorkingGroupRecommendations = innovationProgress.WorkingGroupRecommendations,
                ForumDecision = innovationProgress.ForumDecision
            };

            if (innovationProgress.DatefirstInterviewConducted != SqlDateTime.MinValue.Value)
                progressModel.DatefirstInterviewConducted = CommonUtil.ParseDateToString(innovationProgress.DatefirstInterviewConducted);

            if (innovationProgress.DateOfFirstResponse != SqlDateTime.MinValue.Value)
                progressModel.DateOfFirstResponse = CommonUtil.ParseDateToString(innovationProgress.DateOfFirstResponse);

            if (innovationProgress.FollowupDeadline != SqlDateTime.MinValue.Value)
                progressModel.FollowupDeadline = CommonUtil.ParseDateToString(innovationProgress.FollowupDeadline);

            if (innovationProgress.LastEdit != null && innovationProgress.LastEdit != SqlDateTime.MinValue.Value)
                progressModel.LastEdit = CommonUtil.ParseDateToString(innovationProgress.LastEdit);

            if (progressModel.CurrentStage != null)
                progressModel.CurrentStateId = progressModel.CurrentStage.ID;

            return progressModel;
        }
        #endregion
    }
}