﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Crossrail.IMS.WebApp.DataGatewayService;
using System.ComponentModel.DataAnnotations;
using Crossrail.IMS.WebApp.Utilities;

namespace Crossrail.IMS.WebApp.Models
{
    public abstract class InnovationRemitModel
    {
        public string ID { get; set; }
    }
    public class InnovationRemitImpModel : InnovationRemitModel
    {
        [DataType(DataType.MultilineText)]
        [StringLength(256, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string IR_ImpDescription { get; set; }
        public string IR_ImpStart { get; set; }
        public string IR_ImpFinish { get; set; }
        [RegularExpression(@"(|\d+|\d{1,8}\.\d{1,2})$", ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageCurrency)]
        [StringLength(10, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string IR_ImpCost { get; set; }

        public static InnovationRemitImpModel ConvertToInnovationRemitImpModel(InnovationRemitImp data)
        {
            if (data == null)
                return null;
            return new InnovationRemitImpModel
            {
                ID = data.ID.ToString(),
                IR_ImpCost = data.IR_ImpCost != null ? data.IR_ImpCost.Trim() : string.Empty,
                IR_ImpDescription = data.IR_ImpDescription,
                IR_ImpFinish = CommonUtil.ParseDateToString(data.IR_ImpFinish),
                IR_ImpStart = CommonUtil.ParseDateToString(data.IR_ImpStart)
            };
        }

        public static InnovationRemitImp ConvertToInnovationRemitImp(InnovationRemitImpModel data)
        {
            if (data == null)
                return null;
            float temp;
            if (float.TryParse(data.IR_ImpCost, out temp))
            {
                return new InnovationRemitImp
                {
                    ID = string.IsNullOrEmpty(data.ID) ? Guid.NewGuid() : new Guid(data.ID),
                    IR_ImpCost = data.IR_ImpCost,
                    IR_ImpDescription = data.IR_ImpDescription,
                    IR_ImpFinish = CommonUtil.ParseDate(data.IR_ImpFinish),
                    IR_ImpStart = CommonUtil.ParseDate(data.IR_ImpStart)
                };
            }
            return new InnovationRemitImp
            {
                ID = string.IsNullOrEmpty(data.ID) ? Guid.NewGuid() : new Guid(data.ID),
                IR_ImpCost = null,
                IR_ImpDescription = data.IR_ImpDescription,
                IR_ImpFinish = CommonUtil.ParseDate(data.IR_ImpFinish),
                IR_ImpStart = CommonUtil.ParseDate(data.IR_ImpStart)
            };
        }

        public static List<InnovationRemitImpModel> ConvertToInnovationRemitImpModels(InnovationRemitImp[] data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitImpModel>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitImpModel(item));
            }
            return result;
        }

        public static InnovationRemitImp[] ConvertToInnovationRemitImps(List<InnovationRemitImpModel> data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitImp>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitImp(item));
            }
            return result.ToArray();
        }
    }
    public class InnovationRemitResponsibleModel : InnovationRemitModel
    {
        [StringLength(100, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_ResponsibleName { get; set; }
        [StringLength(256, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_ResponsibleRole { get; set; }
        [StringLength(5, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [RegularExpression(@"(|\d+|\d{1,3}\.\d{1,2})$", ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageNumber)]
        public string IR_ResponsibleHours { get; set; }

        public static InnovationRemitResponsibleModel ConvertToInnovationRemitResponsibleModel(InnovationRemitResponsible data)
        {
            if (data == null)
                return null;
            return new InnovationRemitResponsibleModel
            {
                ID = data.ID.ToString(),
                IR_ResponsibleHours = data.IR_ResponsibleHours != null ? data.IR_ResponsibleHours.Trim() : string.Empty,
                IR_ResponsibleName = data.IR_ResponsibleName,
                IR_ResponsibleRole = data.IR_ResponsibleRole
            };
        }

        public static InnovationRemitResponsible ConvertToInnovationRemitResponsible(InnovationRemitResponsibleModel data)
        {
            if (data == null)
                return null;
            float temp;
            if (float.TryParse(data.IR_ResponsibleHours, out temp))
            {
                return new InnovationRemitResponsible
                {
                    ID = string.IsNullOrEmpty(data.ID) ? Guid.NewGuid() : new Guid(data.ID),
                    IR_ResponsibleHours = data.IR_ResponsibleHours,
                    IR_ResponsibleName = data.IR_ResponsibleName,
                    IR_ResponsibleRole = data.IR_ResponsibleRole
                };
            }
            return new InnovationRemitResponsible
            {
                ID = string.IsNullOrEmpty(data.ID) ? Guid.NewGuid() : new Guid(data.ID),
                IR_ResponsibleHours = null,
                IR_ResponsibleName = data.IR_ResponsibleName,
                IR_ResponsibleRole = data.IR_ResponsibleRole
            };
        }

        public static List<InnovationRemitResponsibleModel> ConvertToInnovationRemitResponsibleModels(InnovationRemitResponsible[] data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitResponsibleModel>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitResponsibleModel(item));
            }
            return result;
        }

        public static InnovationRemitResponsible[] ConvertToInnovationRemitResponsibles(List<InnovationRemitResponsibleModel> data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitResponsible>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitResponsible(item));
            }
            return result.ToArray();
        }
    }
    public class InnovationRemitSuccessFailureModel : InnovationRemitModel
    {
        [DataType(DataType.MultilineText)]
        [StringLength(256, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string IR_SuccessFailDescript { get; set; }
        [StringLength(100, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_Whenassessed { get; set; }
        [DataType(DataType.MultilineText)]
        [StringLength(256, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string IR_HowMeasured { get; set; }

        public static InnovationRemitSuccessFailureModel ConvertToInnovationRemitSuccessFailureModel(InnovationRemitSuccessFailure data)
        {
            if (data == null)
                return null;
            return new InnovationRemitSuccessFailureModel
            {
                ID = data.ID.ToString(),
                IR_HowMeasured = data.IR_HowMeasured,
                IR_SuccessFailDescript = data.IR_SuccessFailDescript,
                IR_Whenassessed = data.IR_Whenassessed
            };
        }

        public static InnovationRemitSuccessFailure ConvertToInnovationRemitSuccessFailure(InnovationRemitSuccessFailureModel data)
        {
            if (data == null)
                return null;
            return new InnovationRemitSuccessFailure
            {
                ID = string.IsNullOrEmpty(data.ID) ? Guid.NewGuid() : new Guid(data.ID),
                IR_HowMeasured = data.IR_HowMeasured,
                IR_SuccessFailDescript = data.IR_SuccessFailDescript,
                IR_Whenassessed = data.IR_Whenassessed
            };
        }

        public static List<InnovationRemitSuccessFailureModel> ConvertToInnovationRemitSuccessFailureModels(InnovationRemitSuccessFailure[] data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitSuccessFailureModel>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitSuccessFailureModel(item));
            }
            return result;
        }

        public static InnovationRemitSuccessFailure[] ConvertToInnovationRemitSuccessFailures(List<InnovationRemitSuccessFailureModel> data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitSuccessFailure>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitSuccessFailure(item));
            }
            return result.ToArray();
        }
    }
    public class InnovationRemitAuthProceedModel : InnovationRemitModel
    {
        [DataType(DataType.MultilineText)]
        [StringLength(256, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string IR_HoldPointDescript { get; set; }
        [StringLength(100, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_AuthProceedName { get; set; }
        [StringLength(100, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string IR_AuthProceedDate { get; set; }

        public static InnovationRemitAuthProceedModel ConvertToInnovationRemitAuthProceedModel(InnovationRemitAuthProceed data)
        {
            if (data == null)
                return null;
            return new InnovationRemitAuthProceedModel
            {
                ID = data.ID.ToString(),
                IR_AuthProceedDate = data.IR_AuthProceedDate,
                IR_AuthProceedName = data.IR_AuthProceedName,
                IR_HoldPointDescript = data.IR_HoldPointDescript
            };
        }

        public static InnovationRemitAuthProceed ConvertToInnovationRemitAuthProceed(InnovationRemitAuthProceedModel data)
        {
            if (data == null)
                return null;
            return new InnovationRemitAuthProceed
            {
                ID = string.IsNullOrEmpty(data.ID) ? Guid.NewGuid() : new Guid(data.ID),
                IR_AuthProceedDate = data.IR_AuthProceedDate,
                IR_AuthProceedName = data.IR_AuthProceedName,
                IR_HoldPointDescript = data.IR_HoldPointDescript
            };
        }

        public static List<InnovationRemitAuthProceedModel> ConvertToInnovationRemitAuthProceedModels(InnovationRemitAuthProceed[] data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitAuthProceedModel>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitAuthProceedModel(item));
            }
            return result;
        }

        public static InnovationRemitAuthProceed[] ConvertToInnovationRemitAuthProceeds(List<InnovationRemitAuthProceedModel> data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitAuthProceed>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitAuthProceed(item));
            }
            return result.ToArray();
        }
    }
    public class InnovationRemitSuppoterModel : InnovationRemitModel
    {
        [DataType(DataType.MultilineText)]
        [StringLength(100, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string IR_SupporterName { get; set; }
        [StringLength(100, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_SupporterRole { get; set; }

        public static InnovationRemitSuppoterModel ConvertToInnovationRemitSuppoterModel(InnovationRemitSuppoter data)
        {
            if (data == null)
                return null;
            return new InnovationRemitSuppoterModel
            {
                ID = data.ID.ToString(),
                IR_SupporterName = data.IR_SupporterName,
                IR_SupporterRole = data.IR_SupporterRole
            };
        }

        public static InnovationRemitSuppoter ConvertToInnovationRemitSuppoter(InnovationRemitSuppoterModel data)
        {
            if (data == null)
                return null;
            return new InnovationRemitSuppoter
            {
                ID = string.IsNullOrEmpty(data.ID) ? Guid.NewGuid() : new Guid(data.ID),
                IR_SupporterName = data.IR_SupporterName,
                IR_SupporterRole = data.IR_SupporterRole
            };
        }

        public static List<InnovationRemitSuppoterModel> ConvertToInnovationRemitSuppoterModels(InnovationRemitSuppoter[] data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitSuppoterModel>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitSuppoterModel(item));
            }
            return result;
        }

        public static InnovationRemitSuppoter[] ConvertToInnovationRemitSuppoters(List<InnovationRemitSuppoterModel> data)
        {
            if (data == null)
                return null;
            var result = new List<InnovationRemitSuppoter>();
            foreach (var item in data)
            {
                result.Add(ConvertToInnovationRemitSuppoter(item));
            }
            return result.ToArray();
        }
    }
}