﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ComponentModel.DataAnnotations;
using Crossrail.IMS.WebApp.DataGatewayService;
using System.Web.Mvc;
using Crossrail.IMS.WebApp.Utilities;

namespace Crossrail.IMS.WebApp.Models
{
    public class InnovationSummaryModel
    {
        public InnovationSummaryModel()
        {

        }

        #region Properties
        /// <summary>
        /// The id of the Innovation
        /// </summary>
        public int ID { get; set; }

        /// <summary>
        ///Read-only
        ///Format: INVXXXXX 
        ///XXXXX is a numerical identified assigned concurrently from 00050 to 99999
        /// </summary>
        [StringLength(8, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string InnovationRef { get; set; }

        [StringLength(256, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [Required(ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageRequired)]
        [RegularExpression(@"^(?!\s*$).+", ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageRequired)]
        public string InnovationTitle { get; set; }

        /// <summary>
        /// Description of Innovation
        /// </summary>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string Description { get; set; }

        /// <summary>
        /// Context of Innovation or Problems addressed
        /// </summary>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string InnovationContext { get; set; }

        /// <summary>
        /// How can we help?
        /// The dropdown list
        /// </summary>
        public int HowCanWeHelpID { get; set; }

        /// <summary>
        /// Individual or shared innovation
        /// The dropdown list
        /// </summary>
        public int IndividualOrSharedInnovationID { get; set; }

        /// <summary>
        /// Who is/are involved in generating the idea
        /// </summary>
        [StringLength(1000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string Who { get; set; }

        /// <summary>
        /// Format:dd/mm/yyyyhh:ss
        ///This value cannot be in the future.
        /// </summary>
        [Required(ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageRequired)]
        public string DateReceipt { get; set; }

        [StringLength(256, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string ParentCompany { get; set; }

        [StringLength(32, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [Required(ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageRequired)]
        [RegularExpression(@"^(?!\s*$).+", ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageRequired)]
        public string ContactFirstName { get; set; }

        [StringLength(32, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [Required(ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageRequired)]
        [RegularExpression(@"^(?!\s*$).+", ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageRequired)]
        public string ContactSurName { get; set; }

        [StringLength(128, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [Required(ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageRequired)]
        [RegularExpression(@"^[\d ]*[\d][\d ]*$", ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageNumber)]
        public string ContactTelephone { get; set; }

        //[RegularExpression(@"\b[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}\b", ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageEmailInvalid)]
        //[DataType(DataType.EmailAddress,ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageEmailInvalid)]
        [StringLength(128, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [Required(ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageRequired)]
        [RegularExpression(@"^([\w\.\-]+)@([\w\-]+)((\.(\w){2,3})+)$", ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageEmailInvalid)]
        public string ContactEmail { get; set; }

        /// <summary>
        /// Proposing Site/ Project/Directorate
        /// </summary>
        public int ProposingSiteProjectID { get; set; }

        /// <summary>
        /// Sponsoring Champion
        /// </summary>
        [StringLength(256, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string SponsoringChampion { get; set; }


        public int Area { get; set; }

        /// <summary>
        /// Detailed description of innovation
        /// </summary>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string DetailedDescription { get; set; }

        /// <summary>
        /// Keywords / Metadata
        /// </summary>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string Keywords { get; set; }


        public int ThemeID { get; set; }


        public int SubThemeID { get; set; }

        /// <summary>
        /// What is the benefit / why should Crossrail do this?
        /// </summary>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string Benefit { get; set; }


        public int BenefitClearlyUnderstoodId { get; set; }

        /// <summary>
        /// Who will benefit?
        /// </summary>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string WhoWillBenefit { get; set; }

        /// <summary>
        /// Validated against AD
        ///This searches all users within our Active Directory (not Innovation System Users) and validates against this.
        ///Ideally it would display the names in AD that match / are close to the users input allowing them 
        ///to select the correct person.
        /// </summary>
        [StringLength(256, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string FunctionalSponsor { get; set; }

        /// <summary>
        /// Suitably Developed to share?
        /// </summary>
        public int SuitablyDevelopedToShareID { get; set; }

        public int CurrentIRLID { get; set; }

        public int TargetIRLID { get; set; }

        /// <summary>
        /// Desired outcome
        /// </summary>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string DesiredOutcome { get; set; }

        /// <summary>
        /// Actions needed to reach desired outcome
        /// </summary>
        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string Actions { get; set; }

        /// <summary>
        /// Values:
        /// Yes : true
        ///No :false
        /// </summary>
        public bool? IsSubmitted { get; set; }

        /// <summary>
        /// Submitted for Competition Date
        /// Dropdown list
        /// If “Submitted” field is “Yes”, it will be auto-populated. Otherwise, it is blank and disabled
        /// </summary
        public string SubmittedForCompetitionDate { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsSafety { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsInspiration { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsCollaboration { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsIntegrity { get; set; }

        /// <summary>
        /// Check box
        /// </summary>
        public bool IsRespect { get; set; }

        [StringLength(5000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string Notes { get; set; }

        public int IdeaTypeID { get; set; }

        public int IdeaMaturityID { get; set; }

        public int PrimaryBenefitID { get; set; }

        public int InnovationSourceID { get; set; }

        [CustomAttribute.Decimal(ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageNumber)]
        public decimal? HoursDeveloping { get; set; }

        public IEnumerable<LinkedDocumentModel> LinkedDocuments { get; set; }

        [StringLength(1000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        public string TaskGroupMembers { get; set; }

        public int AssignedInnovationTeamMemberID { get; set; }

        public List<ProposingItem> ProposingItems1 { get; set; }
        public List<ProposingItem> ProposingItems2 { get; set; }

        [StringLength(1000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_IssueOpportunity { get; set; }

        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_InnovativeSol { get; set; }

        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_PrimaryBenefit { get; set; }

        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_DesiredOutcome { get; set; }

        [StringLength(10, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [RegularExpression(@"(|\d+|\d{1,8}\.\d{1,2})$", ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageNumber)]
        public string IR_FundingSought { get; set; }

        [StringLength(2000, ErrorMessage = Common.Constant.InnovationDetailErrorCode.ErrorMessageMaximumLength)]
        [DataType(DataType.MultilineText)]
        public string IR_Assumptions { get; set; }

        #endregion

        #region Field Permission
        public bool HasWriteOnRelatedFiles { get; set; }
        #endregion

        #region Convert Method
        /// <summary>
        /// Convert the innovation model to innovation entity
        /// </summary>
        public static Innovation ConvertToInnovationSummaryEntity(InnovationSummaryModel model, int? currentStage)
        {
            Innovation entity = new Innovation()
            {
                ID = model.ID,
                InnovationRef = model.InnovationRef,
                InnovationTitle = CommonUtil.TrimString(model.InnovationTitle),
                Description = CommonUtil.TrimString(model.Description),
                InnovationContext = CommonUtil.TrimString(model.InnovationContext),
                HowCanWeHelpID = null,
                IndividualOrSharedInnovationID = null,
                Who = CommonUtil.TrimString(model.Who),
                DateReceipt = CommonUtil.ParseDate(model.DateReceipt),
                ParentCompany = CommonUtil.TrimString(model.ParentCompany),
                ContactFirstName = CommonUtil.TrimString(model.ContactFirstName),
                ContactSurName = CommonUtil.TrimString(model.ContactSurName),
                ContactTelephone = CommonUtil.TrimString(model.ContactTelephone),
                ContactEmail = CommonUtil.TrimString(model.ContactEmail),
                ProposingSiteProjectID = null,
                SponsoringChampion = CommonUtil.TrimString(model.SponsoringChampion),
                DetailedDescription = CommonUtil.TrimString(model.DetailedDescription),
                Keywords = CommonUtil.TrimString(model.Keywords),
                ThemeID = null,
                SubThemeID = null,
                Benefit = CommonUtil.TrimString(model.Benefit),
                BenefitClearlyUnderstoodId = null,
                WhoWillBenefit = CommonUtil.TrimString(model.WhoWillBenefit),
                FunctionalSponsor = CommonUtil.TrimString(model.FunctionalSponsor),
                CurrentIRLID = null,
                TargetIRLID = null,
                DesiredOutcome = CommonUtil.TrimString(model.DesiredOutcome),
                Actions = CommonUtil.TrimString(model.Actions),
                IsSubmitted = model.IsSubmitted,
                SubmittedForCompetitionDate = CommonUtil.ParseDate(model.SubmittedForCompetitionDate),
                IsSafety = model.IsSafety,
                IsInspiration = model.IsInspiration,
                IsCollaboration = model.IsCollaboration,
                IsIntegrity = model.IsIntegrity,
                IsRespect = model.IsRespect,
                Notes = CommonUtil.TrimString(model.Notes),
                IdeaTypeID = null,
                IdeaMaturityID = null,
                PrimaryBenefitID = null,
                InnovationSourceID = null,
                HoursDeveloping = model.HoursDeveloping,
                TaskGroupMembers = model.TaskGroupMembers,
                AssignedInnovationTeamMemberID = model.AssignedInnovationTeamMemberID,
                ProposingItems1 = model.ProposingItems1.Where(c => c.IsSelected).Select(c => new InnovationProposingSiteProject1 { InnovationID = model.ID,ProposingID = c.ID }).ToArray(),
                ProposingItems2 = model.ProposingItems2.Where(c => c.IsSelected).Select(c => new InnovationProposingSiteProject2 { InnovationID = model.ID, ProposingID = c.ID }).ToArray(),


                IR_Assumptions = model.IR_Assumptions,
                IR_DesiredOutcome = model.IR_DesiredOutcome,
                IR_FundingSought = model.IR_FundingSought,
                IR_InnovativeSol = model.IR_InnovativeSol,
                IR_IssueOpportunity = model.IR_IssueOpportunity,
                IR_PrimaryBenefit = model.IR_PrimaryBenefit,

                CurrentStage = currentStage.HasValue ? new InnovationData { ID = currentStage.Value } : null
            };

            if (model.HowCanWeHelpID != 0)
            {
                entity.HowCanWeHelpID = model.HowCanWeHelpID;
            }

            if (model.IndividualOrSharedInnovationID != 0)
            {
                entity.IndividualOrSharedInnovationID = model.IndividualOrSharedInnovationID;
            }

            if (model.ProposingSiteProjectID != 0)
            {
                entity.ProposingSiteProjectID = model.ProposingSiteProjectID;
            }

            if (model.ThemeID != 0)
            {
                entity.ThemeID = model.ThemeID;
            }

            if (model.SubThemeID != 0)
            {
                entity.SubThemeID = model.SubThemeID;
            }

            if (model.BenefitClearlyUnderstoodId != 0)
            {
                entity.BenefitClearlyUnderstoodId = model.BenefitClearlyUnderstoodId;
            }

            if (model.SuitablyDevelopedToShareID != 0)
            {
                entity.SuitablyDevelopedToShareID = model.SuitablyDevelopedToShareID;
            }

            if (model.CurrentIRLID != 0)
            {
                entity.CurrentIRLID = model.CurrentIRLID;
            }

            if (model.TargetIRLID != 0)
            {
                entity.TargetIRLID = model.TargetIRLID;
            }

            if (model.Area != 0)
            {
                entity.Area = model.Area;
            }

            if (model.IdeaTypeID != 0)
            {
                entity.IdeaTypeID = model.IdeaTypeID;
            }

            if (model.IdeaMaturityID != 0)
            {
                entity.IdeaMaturityID = model.IdeaMaturityID;
            }

            if (model.PrimaryBenefitID != 0)
            {
                entity.PrimaryBenefitID = model.PrimaryBenefitID;
            }

            if (model.InnovationSourceID != 0)
            {
                entity.InnovationSourceID = model.InnovationSourceID;
            }

            return entity;
        }

        /// <summary>
        /// Convert the innovation entity to innovation model
        /// </summary>
        public static InnovationSummaryModel ConvertToInnovationSummaryModel(Innovation innovation)
        {
            var innovationSummary = new InnovationSummaryModel
            {
                ID = innovation.ID,
                InnovationRef = innovation.InnovationRef,
                DateReceipt = CommonUtil.ParseDateToString(innovation.DateReceipt),
                SubmittedForCompetitionDate = CommonUtil.ParseDateToString(innovation.SubmittedForCompetitionDate),
                InnovationTitle = innovation.InnovationTitle,
                Description = innovation.Description,
                InnovationContext = innovation.InnovationContext,
                HowCanWeHelpID = innovation.HowCanWeHelpID.HasValue ? innovation.HowCanWeHelpID.Value : 0,
                IndividualOrSharedInnovationID = innovation.IndividualOrSharedInnovationID.HasValue ? innovation.IndividualOrSharedInnovationID.Value : 0,
                Who = innovation.Who,
                ParentCompany = innovation.ParentCompany,
                ContactFirstName = innovation.ContactFirstName,
                ContactSurName = innovation.ContactSurName,
                ContactTelephone = innovation.ContactTelephone,
                ContactEmail = innovation.ContactEmail,
                ProposingSiteProjectID = innovation.ProposingSiteProjectID.HasValue ? innovation.ProposingSiteProjectID.Value : 0,
                SponsoringChampion = innovation.SponsoringChampion,
                Area = innovation.Area.HasValue ? innovation.Area.Value : 0,
                DetailedDescription = innovation.DetailedDescription,
                Keywords = innovation.Keywords,
                ThemeID = innovation.ThemeID.HasValue ? innovation.ThemeID.Value : 0,
                SubThemeID = innovation.SubThemeID.HasValue ? innovation.SubThemeID.Value : 0,
                Benefit = innovation.Benefit,
                BenefitClearlyUnderstoodId = innovation.BenefitClearlyUnderstoodId.HasValue ? innovation.BenefitClearlyUnderstoodId.Value : 0,
                WhoWillBenefit = innovation.WhoWillBenefit,
                FunctionalSponsor = innovation.FunctionalSponsor,
                SuitablyDevelopedToShareID = innovation.SuitablyDevelopedToShareID.HasValue ? innovation.SuitablyDevelopedToShareID.Value : 0,
                CurrentIRLID = innovation.CurrentIRLID.HasValue ? innovation.CurrentIRLID.Value : 0,
                TargetIRLID = innovation.TargetIRLID.HasValue ? innovation.TargetIRLID.Value : 0,
                DesiredOutcome = innovation.DesiredOutcome,
                Actions = innovation.Actions,
                IsSubmitted = innovation.IsSubmitted,
                IsSafety = innovation.IsSafety.HasValue ? innovation.IsSafety.Value : false,
                IsInspiration = innovation.IsInspiration.HasValue ? innovation.IsInspiration.Value : false,
                IsCollaboration = innovation.IsCollaboration.HasValue ? innovation.IsCollaboration.Value : false,
                IsIntegrity = innovation.IsIntegrity.HasValue ? innovation.IsIntegrity.Value : false,
                IsRespect = innovation.IsRespect.HasValue ? innovation.IsRespect.Value : false,

                Notes = innovation.Notes,
                IdeaMaturityID = innovation.IdeaMaturityID.HasValue ? innovation.IdeaMaturityID.Value : 0,
                IdeaTypeID = innovation.IdeaTypeID.HasValue ? innovation.IdeaTypeID.Value : 0,
                PrimaryBenefitID = innovation.PrimaryBenefitID.HasValue ? innovation.PrimaryBenefitID.Value : 0,
                InnovationSourceID = innovation.InnovationSourceID.HasValue ? innovation.InnovationSourceID.Value : 0,
                HoursDeveloping = innovation.HoursDeveloping,
                TaskGroupMembers = innovation.TaskGroupMembers,
                AssignedInnovationTeamMemberID = innovation.AssignedInnovationTeamMemberID ?? 0,

                IR_Assumptions = innovation.IR_Assumptions,
                IR_DesiredOutcome = innovation.IR_DesiredOutcome,
                IR_FundingSought = innovation.IR_FundingSought,
                IR_InnovativeSol = innovation.IR_InnovativeSol,
                IR_IssueOpportunity = innovation.IR_IssueOpportunity,
                IR_PrimaryBenefit = innovation.IR_PrimaryBenefit
            };

            return innovationSummary;
        }
        #endregion
    }
}