﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;
using Crossrail.IMS.WebApp.DataGatewayService;

namespace Crossrail.IMS.WebApp.Models
{
	public class PeopleSupportModel
	{
		public int ID { get; set; }

		/// <summary>
		/// Gets or sets the Name.
		/// </summary>
		/// <value>
		/// The Name.
		/// </value>
		[Required(ErrorMessage = "Name is required"), MaxLength(128)]
		[RegularExpression(@"^(?!\s*$).+", ErrorMessage = "Name cannot be empty")]
		public string Name { get; set; }

		/// <summary>
		/// Gets or sets the Reason.
		/// </summary>
		/// <value>
		/// The Reason.
		/// </value>
		[MaxLength(500)]
		public string Reason { get; set; }

		/// <summary>
		/// Gets or sets the innovation.
		/// </summary>
		/// <value>
		/// The innovation progress.
		/// </value>
		public int Innovation_ID { get; set; }

		/// <summary>
		/// Gets or sets the org ID.
		/// </summary>
		/// <value>
		/// The org ID.
		/// </value>
		public int? OrgID { get; set; }

        public Guid TemptId { get; set; }

        public static PeopleSupportTemporary ConvertToPeopleSupportEntity(PeopleSupportModel data)
        {
            return new PeopleSupportTemporary()
            {
                ID = data.ID,
                Name = data.Name.Trim(),
                Reason = (data.Reason != null) ? data.Reason.Trim() : null,
                OrgID = data.OrgID,
                TemptId = data.TemptId
            };
        }
	}
}