﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;
using Crossrail.IMS.WebApp.DataGatewayService;

namespace Crossrail.IMS.WebApp.Models
{
	public class ResourcesRequiredModel
	{
		public int ID { get; set; }

		/// <summary>
		/// Gets or sets the Name.
		/// </summary>
		/// <value>
		/// The Name.
		/// </value>
		[Required(ErrorMessage = "Name is required"), MaxLength(128)]
		[RegularExpression(@"^(?!\s*$).+", ErrorMessage = "Name cannot be empty")]
		public string Name { get; set; }

		/// <summary>
		/// Gets or sets the Reason.
		/// </summary>
		/// <value>
		/// The Reason.
		/// </value>
		[Required(ErrorMessage = "Role is required"), MaxLength(128)]
		[RegularExpression(@"^(?!\s*$).+", ErrorMessage = "Role cannot be empty")]
		public string Role { get; set; }

		/// <summary>
		/// Gets or sets the hours.
		/// </summary>
		/// <value>
		/// The hours.
		/// </value>
		public decimal Hours { get; set; }

		/// <summary>
		/// Gets or sets the output.
		/// </summary>
		/// <value>
		/// The output.
		/// </value>
		[MaxLength(500)]
		public string Output { get; set; }

		/// <summary>
		/// Gets or sets the innovation progress.
		/// </summary>
		/// <value>
		/// The innovation progress.
		/// </value>
		public int Innovation_ID { get; set; }

        public Guid TemptId { get; set; }

		public int OrgID { get; set; }

        public static ResourcesRequiredTemporary ConvertToResourcesRequiredEntity(ResourcesRequiredModel data)
        {
            return new ResourcesRequiredTemporary()
            {
                ID = data.ID,
                Name = data.Name.Trim(),
                Hours = data.Hours,
                Output = data.Output != null ? data.Output.Trim() : null,
                Role = data.Role.Trim(),
                //Innovation = new Innovation() { ID = data.Innovation_ID },
                TemptId = data.TemptId,
                OrgID = data.OrgID
            };
        }
	}
}