﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.SqlTypes;
using System.Globalization;
using System.IO;
using System.Configuration;
using Crossrail.IMS.Common;
using System.Security.Policy;

namespace Crossrail.IMS.WebApp.Utilities
{
    public class CommonUtil
    {
        public static string TrimString(string inputString)
        {
            return String.IsNullOrEmpty(inputString) ? null : inputString.Trim();
        }

        /// <summary>
        /// Parse the date string from format {dd-mm-yyyy} to valid datetime.
        /// </summary>
        /// <param name="date">The date.</param>
        /// <returns></returns>
        public static DateTime ParseDate(string date)
        {
            if (String.IsNullOrEmpty(date))
            {
                return SqlDateTime.MinValue.Value;
            }
            string dateString = date;
            string format = "dd/MM/yyyy";

            DateTime dateTime = DateTime.ParseExact(dateString, format,
                CultureInfo.InvariantCulture);

            return dateTime;
        }

        public static string ParseDateToString(DateTime? date)
        {
            if (!date.HasValue || date.Value == SqlDateTime.MinValue.Value)
            {
                return string.Empty;
            }
            string format = "dd/MM/yyyy";
            return date.Value.ToString(format);
        }

        public static void DeleteTemporaryFiles(int innovationId, string innovationRef)
        {
            try
            {
                var directory = Path.Combine(ConfigurationManager.AppSettings["RelatedFilesPath"], innovationRef);
                if (Directory.Exists(directory))
                {
                    DirectoryInfo directoryInfo = new DirectoryInfo(directory);
                    directoryInfo.GetFiles().ToList().ForEach(file =>
                    {
                        System.IO.File.Delete(Path.Combine(directory, file.Name));
                    });

                    directoryInfo.Delete();
                }
            }
            catch (Exception)
            {
                throw;
            }

        }

        public static void DeleteUploadFile(string fullName, string temptId)
        {
            var fileName = Path.GetFileName(fullName);
            var physicalPath = String.IsNullOrEmpty(temptId) ?
                Path.Combine(ConfigurationManager.AppSettings["RelatedFilesPath"], fileName) :
                Path.Combine(ConfigurationManager.AppSettings["RelatedFilesPath"], temptId, fileName);

            // TODO: Verify user permissions

            if (System.IO.File.Exists(physicalPath))
            {
                // The files are not actually removed in this demo
                System.IO.File.Delete(physicalPath);
            }
        }

        public static void CopyAllUploadFile(string from, string des)
        {
            try
            {
                from = Path.Combine(ConfigurationManager.AppSettings["RelatedFilesPath"], from);
                des = Path.Combine(ConfigurationManager.AppSettings["RelatedFilesPath"], des);

                if (Directory.Exists(from))
                {
                    if (!Directory.Exists(des))
                    {
                        Directory.CreateDirectory(des);
                    }

                    DirectoryInfo directoryInfo = new DirectoryInfo(from);
                    directoryInfo.GetFiles().ToList().ForEach(file =>
                    {
                        System.IO.File.Move(Path.Combine(from, file.Name), Path.Combine(des, file.Name));
                    });

                    directoryInfo.Delete();
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }            
        }
    }
}